package es.gva.cit.catalogClient.schemas.parsers;

import java.net.URL;

import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.catalogClient.querys.Coordinates;
import es.gva.cit.catalogClient.schemas.discoverer.Record;
import es.gva.cit.catalogClient.schemas.discoverer.Resource;

public class Iso19139Parser extends Record{
	/**
	 * Constructor
	 * 
	 * 
	 * @param node Node with the answer
	 * @param serverURL Server URL. Necessary to load the image (just Geonetwork)
	 */
	    public  Iso19139Parser(XMLNode node, URL serverURL) {        
	        setNode(node);
	        setTitle(XMLTree.searchNodeValue(node,
	                "has->MD_Metadata->identificationInfo->MD_DataIdentification->citation->CI_Citation->title->gco:CharacterString"));
	        setAbstract_(XMLTree.searchNodeValue(node, "has->MD_Metadata->identificationInfo->MD_DataIdentification->abstract->gco:CharacterString"));
	        setPurpose(XMLTree.searchNodeValue(node, "dataIdInfo->idPurp"));
	        setKeyWords(XMLTree.searchMultipleNodeValue(node,
	                "has->MD_Metadata->identificationInfo->MD_DataIdentification->descriptiveKeywords->MD_Keywords->keyword->gco:CharacterString"));
	        setResources(getResources("has->MD_Metadata->distributionInfo->MD_Distribution->transferOptions->MD_DigitalTransferOptions->onLine->CI_OnlineResource"));
	        setFileID(XMLTree.searchNodeValue(node,"mdFileID"));
	        setServerURL(serverURL);
	        //Caution: getImageUrl uses serverURL and FileID!!!
	        setImage(getImageUrl("dataIdInfo->graphOver"));
	        
	    } 

	/**
	 * 
	 * 
	 * 
	 * @return 
	 * @param label 
	 */
	    private Resource[] getResources(String label) {        
	        XMLNode[] nodes = XMLTree.searchMultipleNode(getNode(), label);
	        Coordinates coordinates = null;
	        String srs = null;
	        
	        if (nodes == null) {
	            return null;
	        }
	        Resource[] resources = new Resource[nodes.length];
	        if (nodes.length > 0){
	        	srs = XMLTree.searchNodeValue(getNode(),"refSysInfo->MdCoRefSys->refSysID->identCode");
	        	coordinates = new Coordinates(XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->westBL"),
	        			XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->northBL"),
						XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->eastBL"),
						XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->southBL"));
	        }
	        	
	        	
	        for (int i = 0; i < resources.length; i++){
	            	resources[i] = new Resource(XMLTree.searchNodeValue(nodes[i],
	                        "linkage->URL"),
	                    XMLTree.searchNodeValue(nodes[i], "protocol->gco:CharacterString"),
	                    XMLTree.searchNodeValue(nodes[i], "name->gco:CharacterString"),
	                    XMLTree.searchNodeValue(nodes[i], "orDesc"),
	                    XMLTree.searchNodeAtribute(nodes[i], "orFunct->OnFunctCd",
	                        "value"),
						srs,	
	            		coordinates);
	            	if (resources[i].getLinkage() == null){
	            		resources[i].setLinkage("");
	            	}
	            	
	        }
	        
	        
	        return resources;
	    }     
	  
}
