
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.catalogClient.srw.drivers;
import es.gva.cit.catalogClient.drivers.AbstractCatalogServiceDriver;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.catalogClient.metadataXML.XMLTreeNumberOfRecordsAnswer;
import es.gva.cit.catalogClient.protocols.HTTPGetProtocol;
import es.gva.cit.catalogClient.protocols.SOAPProtocol;
import es.gva.cit.catalogClient.querys.Query;
import es.gva.cit.catalogClient.srw.parsers.SrwCapabilitiesParser;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

/**
 * This class implements the CSW protocol.
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 * @see http://www.loc.gov/z3950/agency/zing/srw/
 */
public class SRWCatalogServiceDriver extends AbstractCatalogServiceDriver {

/**
 * 
 * 
 */
    private String version = "1.1";

/**
 * 
 * 
 */
    private String recordXPath;

/**
 * 
 * 
 */
    private String resultSetTTL;
/* (non-Javadoc)
     * @see ICatalogServerDriver#GetCapabilities(java.lang.String, java.lang.String)
    */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public Collection getCapabilities(URL url) {        
        SRWMessages messages = new SRWMessages(this);
        
        
        Collection nodes = new HTTPGetProtocol().doQuery(url,
                messages.getHTTPGETCapabilities(true), 0);
        
        

        if (nodes != null) {
        	if (nodes.size() > 0){
            	XMLNode n = (XMLNode)nodes.toArray()[0];
                if (n != null){    	
                	setCommunicationProtocol("GET");
                	return nodes;
                }
        	}
        }
        nodes = new SOAPProtocol().doQuery(url, messages.getSOAPCapabilities(), 0);
        if (nodes != null) {
            setCommunicationProtocol("SOAP");
        }
        return nodes;
    } 
/* (non-Javadoc)
     * @see ICatalogServerDriver#GetRecords(java.lang.String, java.lang.String, java.lang.String, int, int, java.lang.String, java.lang.String)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 * @param query 
 * @param firstRecord 
 */
    public Collection getRecords(URL url, Query query, int firstRecord) {        
        setQuery(query);
        SRWMessages messages = new SRWMessages(this);
        Collection answerNodes = new java.util.ArrayList();
        Collection nodes = new java.util.ArrayList();
        
        if (url.getHost().equals("idee.unizar.es")){
        	try {
				url = new URL("http://idee.unizar.es/SRW/servlet/search/SRW");
			} catch (MalformedURLException e) {
				// It will never throws
			}
        }
        XMLNode root = null;
        if (getCommunicationProtocol().equals("GET")) {
            nodes = new HTTPGetProtocol().doQuery(url,
                    messages.getHTTPGETRecords(getQuery(), firstRecord), firstRecord);
            root = (XMLNode)nodes.toArray()[0];
        }
        
        if (getCommunicationProtocol().equals("POST") || 
                getCommunicationProtocol().equals("SOAP")){
            nodes = new SOAPProtocol().doQuery(url,
                    messages.getSOAPRecords(getQuery(), firstRecord), firstRecord);
            root = (XMLNode)nodes.toArray()[0];
            root = root.getSubNode(0).getSubNode(0);
        }
        
                
        String prefix = new StringTokenizer(root.getName(), ":").nextToken();
        if (prefix.equals(root.getName())) {
            prefix = "";
        } else {
            prefix = prefix + ":";
        }        
        int numberOfRecords = getNumberOfRecords(root,
        		prefix + "numberOfRecords",
				null);
        
        if (numberOfRecords == -1) {
            return null;
        }   
        
        answerNodes = createAnswerTree(numberOfRecords, firstRecord, root,
                url,prefix);
        return answerNodes;
    } 

/**
 * It returns an array of nodes with:
 * 1st position -> tree with information of the answer
 * Next positions -> One result for each position
 * 
 * 
 * @return 
 * @param numberOfRecords Number of records returned by the query
 * @param firstRecord Number of the first record
 * @param node Tree returned by the server
 * @param url Server URL
 * @param prefix Tags Prefix
 */
    public Collection createAnswerTree(int numberOfRecords, int firstRecord, XMLNode node, URL url, String prefix) {        
        Collection answerNodes = new ArrayList();
        
       
        answerNodes.add(XMLTreeNumberOfRecordsAnswer.getNode(numberOfRecords,
                firstRecord, firstRecord + numberOfRecords));
        XMLNode[] auxNodes = XMLTree.searchMultipleNode(node,
                prefix + "records->" + prefix + "record");
        for (int i = 1;
                (i <= numberOfRecords) && (i <= 10) &&
                (i <= (numberOfRecords - firstRecord + 1)); i++)
            answerNodes.add(auxNodes[i - 1]);
        return answerNodes;
    } 

/**
 * 
 * 
 * 
 * @return Returns the recordXPath.
 */
    public String getRecordXPath() {        
        return recordXPath;
    } 

/**
 * 
 * 
 * 
 * @param recordXPath The recordXPath to set.
 */
    public void setRecordXPath(String recordXPath) {        
        this.recordXPath = recordXPath;
    } 

/**
 * 
 * 
 * 
 * @return Returns the resultSetTTL.
 */
    public String getResultSetTTL() {        
        return resultSetTTL;
    } 

/**
 * 
 * 
 * 
 * @param resultSetTTL The resultSetTTL to set.
 */
    public void setResultSetTTL(String resultSetTTL) {        
        this.resultSetTTL = resultSetTTL;
    } 

/**
 * 
 * 
 * 
 * @return Returns the version.
 */
    public String getVersion() {        
        return version;
    } 

/**
 * 
 * 
 * 
 * @param version The version to set.
 */
    public void setVersion(String version) {        
        this.version = version;
    } 
/* (non-Javadoc)
     * @see catalogClient.ICatalogServerDriver#setParameters(java.util.Properties)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param nodes 
 * @param url 
 */
    public boolean setParameters(Collection nodes, URL url) {        
        return new SrwCapabilitiesParser(this).parse((XMLNode)nodes.toArray()[0]);
    } 
/* (non-Javadoc)
     * @see es.gva.cit.catalogClient.drivers.ICatalogServiveDriver#isTheProtocol(java.net.URL)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public boolean isProtocolSupported(URL url) {        
        return SOAPProtocol.isProtocolSupported(url);
    } 
/* (non-Javadoc)
     * @see es.gva.cit.catalogClient.drivers.ICatalogServiceDriver#getServerPofile()
     */

/**
 * 
 * 
 * 
 * @return 
 */
    public String getQueryProfile() {        
        // TODO Auto-generated method stub
        return "SRW";
    } 
 }
