
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.adl.parsers;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.gazetteer.adl.drivers.ADLGazetteerServiceDriver;
import es.gva.cit.gazetteer.adl.protocols.ADLThesaurus;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * This Class parses the ADL getCapabilities file
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class AdlCapabilitiesParser {
/**
 * 
 * 
 */
    ADLGazetteerServiceDriver driver;

/**
 * 
 * 
 * 
 * @param driver 
 */
    public  AdlCapabilitiesParser(ADLGazetteerServiceDriver driver) {        
        this.driver = driver;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param node 
 */
    public boolean parse(XMLNode node) {        
        if (node == null){
            driver.setServerAnswerReady("El servidor no soporta el protocolo " +
            "especificado");
            return false;
        }
        
        if (!(node.getName().equals("gazetteer-service"))){
            driver.setServerAnswerReady("El servidor no soporta el protocolo " +
            "especificado");
            return false;
        }
        
        driver.setServerAnswerReady(XMLTree.searchNodeValue(node,
        "get-capabilities-response->gazetteer-capabilities->description"));
                
        String host = XMLTree.searchNodeAtribute(node,
        "get-capabilities-response->gazetteer-capabilities->thesauri->thesaurus",
        "xlink:href");
        
        if (host == null){
            host = "http://middleware.alexandria.ucsb.edu:8080/thes/FTT";
            driver.setServerAnswerReady("No se ha podido cargar el "+
            "Tesauro correctamante. Se va ha usar el Tesauro de alejandra, " +
            "pero no se puede garantizar que el tesauro el servidor sea el mismo " +
            "(http://middleware.alexandria.ucsb.edu:8080/thes/FTT)");
        }
        
        try {
            ADLThesaurus t = new ADLThesaurus(new URL(host));
            driver.setFeature(t.getFeatures());
         } catch (MalformedURLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            driver.setServerAnswerReady("La URL del tesauro no es correcta");
                    
            return false;
        }
         
        driver.setProjection("EPSG:4326");
        return true;
    } 
 }
