/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameTable;
import com.iver.cit.gvsig.project.documents.layout.fframes.IFFrame;
import com.iver.cit.gvsig.project.documents.layout.fframes.IFFrameGroupSelectable;



/**
 * Extensin preparada para controlar las opciones que se pueden realizar sobre
 * una tabla aadida en el Layout.
 *
 * @author Vicente Caballero Navarro
 */
public class FFrameGroupExtension extends Extension {
	private Layout layout = null;

	/**
	 * @see com.iver.andami.plugins.IExtension#initialize()
	 */
	public void initialize() {
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#execute(java.lang.String)
	 */
	public void execute(String s) {
		layout = (Layout) PluginServices.getMDIManager().getActiveWindow();
		IFFrame[] fframes = layout.getFFrameSelected();

		if (s.equals("NOEDITGROUP")) {
			for (int i = 0; i < fframes.length; i++) {
				if (fframes[i] instanceof IFFrameGroupSelectable) {
					IFFrameGroupSelectable fframeTable=(IFFrameGroupSelectable)fframes[i];
					fframeTable.selectFFrame(false);
				}
			}
			layout.setEditGroup(false);
		} else if (s.equals("JOIN")) {
			for (int i = 0; i < fframes.length; i++) {
				if (fframes[i] instanceof IFFrameGroupSelectable) {
					IFFrameGroupSelectable fframeTable=(IFFrameGroupSelectable)fframes[i].cloneFFrame(layout);
					IFFrame newFFrame=fframeTable.joinFFrame();
					if (newFFrame !=null){
						layout.getEFS().modifyFFrame((IFFrame)fframes[i],(IFFrame)fframeTable);
						layout.updateFFrames();
						layout.refresh();
					}

				}
			}
		} else if (s.equals("EDITGROUP")) {
			for (int i = 0; i < fframes.length; i++) {
				if (fframes[i] instanceof IFFrameGroupSelectable) {
					IFFrameGroupSelectable fframeTable=(IFFrameGroupSelectable)fframes[i];
					fframeTable.selectFFrame(true);
				}
			}
			layout.setEditGroup(true);
		}
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		layout = (Layout) PluginServices.getMDIManager().getActiveWindow();
		if (!layout.isEditable())
			return false;
		return true;
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		IWindow view = PluginServices.getMDIManager().getActiveWindow();
		if (view instanceof Layout) {
			Layout l = (Layout) view;
			IFFrame[] fframes = l.getFFrameSelected();

			for (int i = 0; i < fframes.length; i++) {
				if (fframes[i] instanceof FFrameTable) {
					return true;
				}
			}
		}
		return false;
	}
}
