/*
 * Created on 25-jun-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout;

import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.text.NumberFormat;

import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.Copies;
import javax.print.attribute.standard.Fidelity;
import javax.print.attribute.standard.Media;
import javax.print.attribute.standard.MediaPrintableArea;
import javax.print.attribute.standard.MediaSizeName;
import javax.print.attribute.standard.OrientationRequested;
import javax.print.attribute.standard.PrintQuality;
import javax.print.attribute.standard.Sides;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.project.documents.layout.Size;
import com.iver.utiles.XMLEntity;


/**
 * Atributos del Layout.
 *
 * @author Vicente Caballero Navarro
 */
public class Attributes {
	/** KILOMETROS,METROS,CENTIMETROS,MILIMETROS,MILLAS,YARDAS,PIES,PULGADAS */
	public static final double[] CHANGE = {
			100000, 100, 1, 0.1, 160934.4, 91.44, 30.48, 2.54
		};
	public static final String[] NAMES= {"Kilometros","Metros","Centimetros","Milimetros","Millas","Yardas","Pies","Pulgadas","Grados"};
	public static final double[] UNIT = {
			0.0000025, 0.0025, 0.25, 2.5, 0.0000025, 0.025, 0.025, 0.25
		};
	public static final int RALTO = 0;
	public static final int RNORMAL = 1;
	public static final int RBAJO = 2;

	//	Para impresin
	public final static double PULGADA = 2.54;

	//public final static Size CUSTOM_PAPER_SIZE = new Size(8.5, 11.0);
	public final static Size STANDARD_LETTER_PAPER_SIZE = new Size(8.5, 11.0);
	public final static Size STANDARD_FOLIO_PAPER_SIZE = new Size(8.5, 13.0);
	public final static Size STANDARD_LEGAL_PAPER_SIZE = new Size(8.5, 14.0);
	public final static Size STANDARD_TABLOID_PAPER_SIZE = new Size(11.0, 17.0);
	public final static Size METRIC_A0_PAPER_SIZE = new Size(118.9, 84.1);
	public final static Size METRIC_A1_PAPER_SIZE = new Size(84.1, 59.4);
	public final static Size METRIC_A2_PAPER_SIZE = new Size(59.4, 42.0);
	public final static Size METRIC_A3_PAPER_SIZE = new Size(42.0, 29.7);
	public final static Size METRIC_A4_PAPER_SIZE = new Size(29.7, 21.0);
	public final static Size METRIC_A5_PAPER_SIZE = new Size(21.0, 14.8);
	public final static Size ANSI_ENG_A_PAPER_SIZE = new Size(11.0, 8.5);
	public final static Size ANSI_ENG_B_PAPER_SIZE = new Size(17.0, 11.0);
	public final static Size ANSI_ENG_C_PAPER_SIZE = new Size(22.0, 17.0);
	public final static Size ANSI_ENG_D_PAPER_SIZE = new Size(34.0, 22.0);
	public final static Size ANSI_ENG_E_PAPER_SIZE = new Size(44.0, 34.0);
	public final static Size ANSI_ARCH_A_PAPER_SIZE = new Size(12.0, 9.0);
	public final static Size ANSI_ARCH_B_PAPER_SIZE = new Size(18.0, 12.0);
	public final static Size ANSI_ARCH_C_PAPER_SIZE = new Size(24.0, 18.0);
	public final static Size ANSI_ARCH_D_PAPER_SIZE = new Size(36.0, 24.0);
	public final static Size ANSI_ARCH_E_PAPER_SIZE = new Size(42.0, 30.0);
	public static Size CUSTOM_PAPER_SIZE = new Size(100.0, 100.0);
	public final static int PRINT = 0;
	public final static int CUSTOM = 6;
	public final static int A0 = 5;
	public final static int A1 = 4;
	public final static int A2 = 3;
	public final static int A3 = 2;
	public final static int A4 = 1;
	public static int DPI = 300;
	public static int DPISCREEN = 72;
	public static Rectangle clipRect = new Rectangle();
	private static Point2D defaultGridGap;

	static {
		new Attributes();
	}

	private PrintRequestAttributeSet attributes = new HashPrintRequestAttributeSet();
	private MediaSizeName m_type = MediaSizeName.ISO_A4;
	private boolean m_isLandSel;
	private OrientationRequested m_Orientation;
	private double m_TypeUnit = CHANGE[2]; //CENTIMETROS;
	private String m_NameUnit;
	private Double m_numX=null;
	private Double m_numY=null;
	private double m_unitX = 0;
	private double m_unitY = 0;
	private boolean hasmargin;
	private int m_resolutionSel = RNORMAL;
	private int m_typeSel = 0;
	private int m_selTypeUnit = 2;
	public Size m_sizePaper = new Size(METRIC_A4_PAPER_SIZE.getAlto(),
			METRIC_A4_PAPER_SIZE.getAncho());
	public double[] m_area = { PULGADA, PULGADA, PULGADA, PULGADA };
	private double anchoXalto = m_sizePaper.getAlto() / m_sizePaper.getAncho();
	private Size m_sizeinUnits = m_sizePaper;
	private PrintQuality m_resolution=PrintQuality.NORMAL;
	//private javax.print.attribute.standard.

	/**
	 * 050211, jmorell: Se aaden los Grados como unidades de mapa.
	 * Creates a new Attributes object.
	 */
	public Attributes() {
		m_NameUnit = NAMES[2];
		m_type = MediaSizeName.ISO_A4;
		m_isLandSel = true;
		setSizeinUnits(m_isLandSel);

		hasmargin = false;
		m_Orientation = OrientationRequested.LANDSCAPE;
		attributes.add(new Copies(1));
		attributes.add(MediaSizeName.ISO_A4);
		attributes.add(Sides.ONE_SIDED);
		attributes.add(Fidelity.FIDELITY_FALSE);
		attributes.add(PrintQuality.NORMAL);

		setType(m_typeSel);

		attributes.add(new MediaPrintableArea(0, 0,
				(float) ((m_sizePaper.getAlto() * DPISCREEN) / PULGADA),
				(float) ((m_sizePaper.getAncho() * DPISCREEN) / PULGADA),
				MediaPrintableArea.MM));
	}

	/**
	 * Seleccionar el tipo de unidad que representa el Layout.
	 *
	 * @param s entero.
	 */
	public void setUnits(int s) {
		m_TypeUnit = CHANGE[s];
		m_NameUnit = NAMES[s];
	}

	/**
	 * Rellena las propiedades del Layout.
	 *
	 * @param typeSel tipo de folio.
	 * @param units unidad de medida.
	 * @param isLand si es horizontal o vertical.
	 * @param margin si se deben de coger los margenes que marca la impresora o
	 * 		  no.
	 * @param resolution tipo de resolucin a aplicar.
	 * @param area area imprimible.
	 */
	public void setSelectedOptions(int typeSel, int units, boolean isLand,
		boolean margin, int resolution, double[] area) {
		setType(typeSel);
		setUnit(units);
		setIsLandScape(isLand);
		hasmargin = margin;
		setResolution(resolution);
		m_area = area;
	}

	/**
	 * Devuelve un PageFormat con las propiedades seleccionadas.
	 *
	 * @return PageFormat
	 */
	public PageFormat getPageFormat() {
		PageFormat pf1 = new PageFormat();
		Paper paper = pf1.getPaper();

		if (isLandSpace()) {
			pf1.setOrientation(0);
			paper.setSize((m_sizePaper.getAncho() * DPISCREEN) / PULGADA,
				(m_sizePaper.getAlto() * DPISCREEN) / PULGADA);

			double aux = m_area[0];
			m_area[0] = m_area[3];
			m_area[3] = m_area[1];
			m_area[1] = m_area[2];
			m_area[2] = aux;
			paper.setImageableArea(getInPixels(m_area[2]),
				getInPixels(m_area[0]),
				getInPixels(m_sizeinUnits.getAncho() - m_area[2] - m_area[3]),
				getInPixels(m_sizeinUnits.getAlto() - m_area[0] - m_area[1]));
		} else {
			pf1.setOrientation(1);
			paper.setSize((m_sizePaper.getAlto() * DPISCREEN) / PULGADA,
				(m_sizePaper.getAncho() * DPISCREEN) / PULGADA);
			paper.setImageableArea(getInPixels(m_area[2]),
				getInPixels(m_area[0]),
				getInPixels(m_sizeinUnits.getAlto() - m_area[2] - m_area[3]),
				getInPixels(m_sizeinUnits.getAncho() - m_area[0] - m_area[1]));
		}

		pf1.setPaper(paper);

		return pf1;
	}

	/**
	 * Devuelve en Pixels los centmetros que se le pasan como parmetro.
	 *
	 * @param d centmetros.
	 *
	 * @return Pixels.
	 */
	private double getInPixels(double d) {
		return d * (DPISCREEN / PULGADA);
	}

	/**
	 * Devuelve en centmetros los pixels que se le pasan como parmetro.
	 *
	 * @param d Pixel.
	 *
	 * @return Centmetros.
	 */
	private double getInCM(double d) {
		return d / (DPISCREEN / PULGADA);
	}

	/**
	 * Devuelve un retngulo que representa el rea imprimible.
	 *
	 * @return Rectngulo.
	 */
	public Rectangle2D.Double getArea() {
		Rectangle2D.Double rect = new Rectangle2D.Double();
		rect.setRect(m_area[0], m_area[1],
			m_sizePaper.getAncho() - m_area[0] - m_area[2],
			m_sizePaper.getAlto() - m_area[1] - m_area[3]);

		return rect;
	}

	/**
	 * Se rellenan los atributos a partir de un PageFormat que se introduce
	 * como parametro.
	 *
	 * @param pf PageFormat
	 */
	public void setPageFormat(PageFormat pf) {
		Size size = null;

		if (pf.getOrientation() == 0) {
			setIsLandScape(true);
			m_Orientation = OrientationRequested.LANDSCAPE;
			size = new Size(pf.getHeight(), pf.getWidth());
		} else {
			setIsLandScape(false);
			m_Orientation = OrientationRequested.PORTRAIT;
			size = new Size(pf.getHeight(), pf.getWidth());
		}

		attributes.add(m_Orientation);

		//area
		m_area[0] = getInCM(pf.getImageableY());
		m_area[2] = getInCM(pf.getImageableX());
		m_area[1] = m_sizePaper.getAlto() - getInCM(pf.getImageableHeight()) -
			m_area[0];
		m_area[3] = m_sizePaper.getAncho() - getInCM(pf.getImageableWidth()) -
			m_area[2];

		//	tipo formato
		if (isLandSpace()) {
			//double aux = m_area[0];
			//m_area[0] = m_area[3];
			//m_area[3] = m_area[1];
			//m_area[1] = m_area[2];
			//m_area[2] = aux;
			attributes.add(new MediaPrintableArea((float) (m_area[2] * 10),
					(float) (m_area[0] * 10),
					(float) (m_sizeinUnits.getAlto() - m_area[0] - m_area[1]) * 10,
					(float) (m_sizeinUnits.getAncho() - m_area[2] - m_area[3]) * 10,
					MediaPrintableArea.MM));
		} else {
			attributes.add(new MediaPrintableArea((float) (m_area[0] * 10),
					(float) (m_area[1] * 10),
					(float) (getInCM(pf.getImageableWidth()) * 10),
					(float) (getInCM(pf.getImageableHeight()) * 10),
					MediaPrintableArea.MM));
		}

		setType(getTypePaper(size));
	}

	/**
	 * Preparamos un PrintRequestAttributeSet con nuestras opciones para
	 * pasarselo como parametro al PrintDialog.
	 *
	 * @return
	 */
	public PrintRequestAttributeSet toPrintAttributes() {
		HashPrintRequestAttributeSet resul = new HashPrintRequestAttributeSet();

		setType(m_typeSel);

		resul.add(m_type);

		//units, no hace falta aadirlo a attributes
		resul.add(m_Orientation);
		setArea(m_area);
		resul.add(new MediaPrintableArea(0, 0,
				(float) ((m_sizePaper.getAlto() * DPISCREEN) / PULGADA),
				(float) ((m_sizePaper.getAncho() * DPISCREEN) / PULGADA),
				MediaPrintableArea.MM));

		resul.add(m_resolution);
		//resul.add(new PrinterResolution(DPI,DPI,PrinterResolution.DPI));
		return resul;
	}

	/**
	 * rellenar el rea imprimible.
	 *
	 * @param area rea imprimible.
	 */
	private void setArea(double[] area) {
		if (!isLandSpace()) {
			attributes.add(new MediaPrintableArea((float) (area[2] * 10),
					(float) (area[0] * 10),
					(float) ((m_sizePaper.getAncho() - area[2] - area[3]) * 10),
					(float) ((m_sizePaper.getAlto() - area[0] - area[1]) * 10),
					MediaPrintableArea.MM));

			clipRect.setRect((area[2] / PULGADA * DPI),
				area[0] / PULGADA * DPI,
				(m_sizePaper.getAncho() - area[2] - area[3]) / PULGADA * DPI,
				(m_sizePaper.getAlto() - area[0] - area[1]) / PULGADA * DPI);
		} else {
			attributes.add(new MediaPrintableArea((float) (area[0] * 10),
					(float) (area[3] * 10),
					(float) ((m_sizePaper.getAlto() - area[0] - area[1]) * 10),
					(float) ((m_sizePaper.getAncho() - area[3] - area[2]) * 10),
					MediaPrintableArea.MM));

			clipRect.setRect((area[1] / PULGADA * DPI),
				area[2] / PULGADA * DPI,
				(m_sizePaper.getAncho() - area[1] - area[0]) / PULGADA * DPI,
				(m_sizePaper.getAlto() - area[2] - area[3]) / PULGADA * DPI);
		}
	}

	/**
	 * Devuelve true si est seleccionada la opcin de utilizar los margenes de
	 * impresin.
	 *
	 * @return true si est seleccionada la opcin de utilizar margenes.
	 */
	public boolean isMargin() {
		return hasmargin;
	}

	/**
	 * Devuelve la resolucin para imprimir.
	 *
	 * @return Resolucin seleccionada.
	 */
	public int getResolution() {
		return m_resolutionSel;
	}

	/**
	 * Selecciona la resolucin a aplicar.
	 *
	 * @param i tipo de resolucin.
	 */
	private void setResolution(int i) {
		m_resolutionSel = i;

		switch (i) {
			case (RALTO):

				m_resolution=PrintQuality.HIGH;
				DPI = 600;

				break;

			case (RNORMAL):

				m_resolution=PrintQuality.NORMAL;
				DPI = 300;

				break;

			case (RBAJO):

				m_resolution=PrintQuality.DRAFT;
				DPI = 72;

				break;
		}
	}

	/**
	 * Devuelve los Attributes que se pasan como parametro al PrintDialog.
	 *
	 * @return Attributes de impresin.
	 */
	public PrintRequestAttributeSet getAttributes() {
		return attributes;
	}

	/**
	 * Devuelve true si la posicin elegida para el folio es horizontal y false
	 * si es vertical.
	 *
	 * @return true si la posicin elegida es horizontal.
	 */
	public boolean isLandSpace() {
		return m_isLandSel;
	}

	/**
	 * Devuelve el formato de folio seleccionado.
	 *
	 * @return entero que representa el tipo de folio seleccionado.
	 */
	public int getType() {
		return m_typeSel;
	}

	/**
	 * Devuelve el nmero de pixels que representa a la unidad que est
	 * seleccionada.
	 *
	 * @return nmero de pixels que representa la unidad seleccionada.
	 */
	public double getUnitInPixelsY() {
		return m_unitY;
	}

	/**
	 * Devuelve el nmero de pixels que representa a la unidad que est
	 * seleccionada.
	 *
	 * @return nmero de pixels que representa la unidad seleccionada.
	 */
	public double getUnitInPixelsX() {
		return m_unitX;
	}

	/**
	 * Establece el nmero de unidades para el grid.
	 *
	 * @param d nmero que representa la distancia entre los puntos del grid
	 * 		  del Layout.
	 */
	public void setNumUnitsX(double d) {
		m_numX = new Double(d);
	}

	/**
	 * Devuelve el nmero de unidades del espacio de cuadrcula horizontal en
	 * la medida que est seleccionada,  por defecto en centmetros.
	 *
	 * @return double
	 */
	public double getNumUnitsX() {
		if (m_numX == null)
			m_numX = new Double(getDefaultGridGap().getX());
		return m_numX.doubleValue();
	}

	/**
	 * Devuelve el nmero de unidades del espacio de cuadrcula vertical en la
	 * medida que est seleccionada,  por defecto en centmetros.
	 *
	 * @return double
	 */
	public double getNumUnitsY() {
		if (m_numY == null)
			m_numY = new Double(getDefaultGridGap().getY());
		return m_numY.doubleValue();
	}

	/**
	 * Establece el nmero de unidades para el grid.
	 *
	 * @param d nmero que representa la distancia entre los puntos del grid
	 * 		  del Layout.
	 */
	public void setNumUnitsY(double d) {
		m_numY = new Double(d);
	}

	/**
	 * Rellena la forma de visualizacin del folio, tru si es horizontal y
	 * false si es vertical.
	 *
	 * @param b true si se quiere a horizontal.
	 */
	public void setIsLandScape(boolean b) {
		m_isLandSel = b;
		setType(m_typeSel);

		if (m_isLandSel) {
			m_Orientation = OrientationRequested.LANDSCAPE;
		} else {
			m_Orientation = OrientationRequested.PORTRAIT;
		}
	}

	/**
	 * Seleccin del tipo de folio a mostrar.
	 *
	 * @param t tipo de folio.
	 */
	public void setType(int t) {
		m_typeSel = t;

		//m_sizePaper=getSizePaper(t);
		switch (t) {
			case (PRINT):
				m_type = ((MediaSizeName) attributes.get(Media.class));

				if (isLandSpace()) {
					m_sizePaper = new Size(METRIC_A4_PAPER_SIZE.getAncho(),
							METRIC_A4_PAPER_SIZE.getAlto());
				} else {
					m_sizePaper = METRIC_A4_PAPER_SIZE;
				}

				break;

			case (A4):
				m_type = MediaSizeName.ISO_A4;

				if (isLandSpace()) {
					m_sizePaper = new Size(METRIC_A4_PAPER_SIZE.getAncho(),
							METRIC_A4_PAPER_SIZE.getAlto());
				} else {
					m_sizePaper = METRIC_A4_PAPER_SIZE;
				}

				break;

			case (A3):
				m_type = MediaSizeName.ISO_A3;

				if (isLandSpace()) {
					m_sizePaper = new Size(METRIC_A3_PAPER_SIZE.getAncho(),
							METRIC_A3_PAPER_SIZE.getAlto());
				} else {
					m_sizePaper = METRIC_A3_PAPER_SIZE;
				}

				break;

			case (A2):
				m_type = MediaSizeName.ISO_A2;

				if (isLandSpace()) {
					m_sizePaper = new Size(METRIC_A2_PAPER_SIZE.getAncho(),
							METRIC_A2_PAPER_SIZE.getAlto());
				} else {
					m_sizePaper = METRIC_A2_PAPER_SIZE;
				}

				break;

			case (A1):
				m_type = MediaSizeName.ISO_A1;

				if (isLandSpace()) {
					m_sizePaper = new Size(METRIC_A1_PAPER_SIZE.getAncho(),
							METRIC_A1_PAPER_SIZE.getAlto());
				} else {
					m_sizePaper = METRIC_A1_PAPER_SIZE;
				}

				break;

			case (A0):
				m_type = MediaSizeName.ISO_A0;

				if (isLandSpace()) {
					m_sizePaper = new Size(METRIC_A0_PAPER_SIZE.getAncho(),
							METRIC_A0_PAPER_SIZE.getAlto());
				} else {
					m_sizePaper = METRIC_A0_PAPER_SIZE;
				}

				break;

			case (CUSTOM):
				m_type = MediaSizeName.PERSONAL_ENVELOPE;

				if (isLandSpace()) {
					m_sizePaper = new Size(CUSTOM_PAPER_SIZE.getAncho(),
							CUSTOM_PAPER_SIZE.getAlto());
				} else {
					m_sizePaper = CUSTOM_PAPER_SIZE;
				}

				break;
		}

		setSizeinUnits(isLandSpace());
		m_sizeinUnits = getSizeInUnits();
	}

	/**
	 * Seleccin del tipo de folio a mostrar.
	 *
	 * @param isLand tipo de folio.
	 * @param type Tipo de folio.
	 *
	 * @return Tamao del folio seleccionado.
	 */
	public Size getSizeinUnits(boolean isLand, int type) {
		Size size = null;

		switch (type) {
			case (PRINT):

				if (isLand) {
					size = new Size(METRIC_A4_PAPER_SIZE.getAncho(),
							METRIC_A4_PAPER_SIZE.getAlto());
				} else {
					size = METRIC_A4_PAPER_SIZE;
				}

				break;

			case (A4):

				if (isLand) {
					size = new Size(METRIC_A4_PAPER_SIZE.getAncho(),
							METRIC_A4_PAPER_SIZE.getAlto());
				} else {
					size = METRIC_A4_PAPER_SIZE;
				}

				break;

			case (A3):

				if (isLand) {
					size = new Size(METRIC_A3_PAPER_SIZE.getAncho(),
							METRIC_A3_PAPER_SIZE.getAlto());
				} else {
					size = METRIC_A3_PAPER_SIZE;
				}

				break;

			case (A2):

				if (isLand) {
					size = new Size(METRIC_A2_PAPER_SIZE.getAncho(),
							METRIC_A2_PAPER_SIZE.getAlto());
				} else {
					size = METRIC_A2_PAPER_SIZE;
				}

				break;

			case (A1):

				if (isLand) {
					size = new Size(METRIC_A1_PAPER_SIZE.getAncho(),
							METRIC_A1_PAPER_SIZE.getAlto());
				} else {
					size = METRIC_A1_PAPER_SIZE;
				}

				break;

			case (A0):

				if (isLand) {
					size = new Size(METRIC_A0_PAPER_SIZE.getAncho(),
							METRIC_A0_PAPER_SIZE.getAlto());
				} else {
					size = METRIC_A0_PAPER_SIZE;
				}

				break;

			case (CUSTOM):

				if (isLand) {
					size = new Size(CUSTOM_PAPER_SIZE.getAncho(),
							CUSTOM_PAPER_SIZE.getAlto());
				} else {
					size = CUSTOM_PAPER_SIZE;
				}

				break;
		}

		m_sizeinUnits = new Size(size.getAlto() / m_TypeUnit,
				size.getAncho() / m_TypeUnit);

		return m_sizeinUnits;
	}

	/**
	 * Obtiene el entero que representa el tipo de unidad de medida
	 * seleccionada.
	 *
	 * @return tipo de unidad de medida seleccionada.
	 */
	public int getSelTypeUnit() {
		return m_selTypeUnit;
	}

	/**
	 * Devuelve el nombre de la unidad de medida seleccionada.
	 *
	 * @return String
	 */
	public String getNameUnit() {
		return m_NameUnit;
	}

	/**
	 * Seleccin de la unidad de medida.
	 *
	 * @param sel tipo de unidad de medida.
	 */
	public void setUnit(int sel) {
		m_selTypeUnit = sel;
		setUnits(sel);
		m_numX = m_numY = new Double(UNIT[sel]);
	}

	/**
	 * Actualiza m_sizeinUnits con la altura y anchura del folio en las
	 * unidades de mediada seleccionada.
	 *
	 * @param b True si es en horizontal.
	 */
	public void setSizeinUnits(boolean b) {
		if (b) {
			m_sizeinUnits = new Size(m_sizePaper.getAlto() / m_TypeUnit,
					m_sizePaper.getAncho() / m_TypeUnit);
		} else {
			m_sizeinUnits = new Size(m_sizePaper.getAncho() / m_TypeUnit,
					m_sizePaper.getAlto() / m_TypeUnit);
		}
	}

	/**
	 * Devuelve un double en la unidad de medida seleccionada a partir del
	 * double en centmetros que se le pasa como parametro.
	 *
	 * @param d distancia en centmetros.
	 *
	 * @return distancia en la unidad de medida seleccionada.
	 */
	public double toUnits(double d) {
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);

		String s = String.valueOf(nf.format(d));
		s = s.replace(',', '.');

		return (Double.valueOf(s).doubleValue() / m_TypeUnit);
	}

	/**
	 * Devuelve un double en centmetros a partir del double que se pasa como
	 * parmetro en la unidad de medida establecida.
	 *
	 * @param d distancia en unidad de medida seleccionada.
	 *
	 * @return distancia en centmetros.
	 */
	public double fromUnits(double d) {
		return (d * m_TypeUnit);
	}

	/**
	 * Devuelve los pixels por centmetro.
	 *
	 * @param rect Rectngulo.
	 *
	 * @return Pixels por centmetro.
	 */
	public double getPixXCm(Rectangle2D.Double rect) {
		double value = m_sizePaper.getAncho();
		double cm = CHANGE[2];
		double unidades = 0;
		unidades = ((rect.getMaxX() - rect.getMinX()) /((value / cm)));

		return unidades;
	}

	/**
	 * Establece el nmero de pixels que representan un centmetro.
	 *
	 * @param rect rectngulo que representa el folio en pixels.
	 */
	public void setDistanceUnitX(Rectangle2D.Double rect) {
		double value = m_sizePaper.getAncho();
		double unidades = 0;
		unidades = ((rect.getMaxX() - rect.getMinX()) / ((value / m_TypeUnit) / getNumUnitsX()));
		m_unitX = unidades;
	}

	/**
	 * Establece el nmero de pixels que representan un centmetro.
	 *
	 * @param rect rectngulo que representa el folio en pixels.
	 */
	public void setDistanceUnitY(Rectangle2D.Double rect) {
		double value = m_sizePaper.getAncho();
		double unidades = 0;
		unidades = ((rect.getMaxX() - rect.getMinX()) / ((value / m_TypeUnit) / getNumUnitsY()));
		m_unitY = unidades;
	}

	/**
	 * Obtiene el rectngulo que representa el folio con las caractersticas
	 * que contiene attributes y diferenciando si es para visualizar en
	 * pantalla o para imprimir.
	 *
	 * @param isPrint si es para imprimir.
	 * @param rect rectngulo a modificar.
	 * @param w anchura
	 * @param h altura
	 *
	 * @return Rectngulo que representa el folio en horizontal o vertical
	 * 		   segn se haya seleccionado.
	 */
	public Rectangle2D.Double obtainRect(boolean isPrint,
		Rectangle2D.Double rect, int w, int h) {
		double value1 = 0;
		double value2 = 0;

		if (!isPrint) {
			if (isLandSpace()) {
				anchoXalto = m_sizePaper.getAncho() / m_sizePaper.getAlto();
				rect = getRectangleLandscape(rect, w, h);
			} else {
				anchoXalto = m_sizePaper.getAlto() / m_sizePaper.getAncho();
				rect = getRectanglePortrait(rect, w, h);
			}
		} else {
			value1 = m_sizePaper.getAncho();
			value2 = m_sizePaper.getAlto();
			rect.setRect(0, 0, ((value1 / PULGADA) * DPI),
				((value2 / PULGADA) * DPI));
		}

		setDistanceUnitX(rect);
		setDistanceUnitY(rect);

		return rect;
	}

	/**
	 * Modifica el rectngulo que representa al folio en una posicin
	 * horizontal.
	 *
	 * @param rect Rectngulo que representa el folio en pixels.
	 * @param w anchura del Layout.
	 * @param h altura del Layout.
	 *
	 * @return Rectngulo modificado.
	 */
	public Rectangle2D.Double getRectangleLandscape(Rectangle2D.Double rect,
		int w, int h) {
		Rectangle2D.Double rectaux = new Rectangle2D.Double();
		int x0 = (int) rect.getMinX();
		int y0 = (int) rect.getMinY();
		int y1;
		int x1;
		y1 = (h - (2 * y0));
		x1 = (int) (y1 * anchoXalto);

		if (((int) (((h) - (2 * y0)) * anchoXalto)) > ((w) - (2 * x0))) {
			x1 = ((w) - (2 * x0));
			y1 = (int) (x1 / anchoXalto);
		}

		rectaux.setRect(x0, y0, x1, y1);

		return rectaux;
	}

	/**
	 * Devuelve un Size con las medidas del folio en las unidades de mediada
	 * seleccionada.
	 *
	 * @return Tamao del folio en las unidades seleccionadas.
	 */
	public Size getSizeInUnits() {
		return m_sizeinUnits;
	}

	/**
	 * Modifica el rectngulo que representa al folio en una posicin vertical.
	 *
	 * @param rect Rectngulo que representa el folio en pixels.
	 * @param w anchura del Layout.
	 * @param h altura del Layout.
	 *
	 * @return Rectngulo modificado.
	 */
	public Rectangle2D.Double getRectanglePortrait(Rectangle2D.Double rect,
		int w, int h) {
		Rectangle2D.Double rectaux = new Rectangle2D.Double();
		int x0 = (int) rect.getMinX();
		int y0 = (int) rect.getMinY();
		int y1;
		int x1;
		x1 = (w - (2 * x0));
		y1 = (int) (x1 * anchoXalto);

		if (((int) (((w) - (2 * x0)) * anchoXalto)) > ((h) - (2 * y0))) {
			y1 = (h - (2 * y0));
			x1 = (int) (y1 / anchoXalto);
		}

		rectaux.setRect(x0, y0, x1, y1);

		return rectaux;
	}

	/**
	 * Obtiene el tipo de los diferentes formatos de papel.
	 *
	 * @param size tipo de paper.
	 *
	 * @return Medidas del papel seleccionado.
	 */
	private int getTypePaper(Size size) {
		int tol = 1;
		Size auxSize = size;

		if (isLandSpace()) {
			auxSize = new Size(size.getAncho(), size.getAlto());
		}

		if ((((auxSize.getAncho() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A4_PAPER_SIZE.getAncho())) &&
				(((auxSize.getAlto() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A4_PAPER_SIZE.getAlto()))) {
			return A4;
		} else if ((((auxSize.getAncho() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A3_PAPER_SIZE.getAncho())) &&
				(((auxSize.getAlto() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A3_PAPER_SIZE.getAlto()))) {
			return A3;
		} else if ((((auxSize.getAncho() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A2_PAPER_SIZE.getAncho())) &&
				(((auxSize.getAlto() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A2_PAPER_SIZE.getAlto()))) {
			return A2;
		} else if ((((auxSize.getAncho() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A1_PAPER_SIZE.getAncho())) &&
				(((auxSize.getAlto() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A1_PAPER_SIZE.getAlto()))) {
			return A1;
		} else if ((((auxSize.getAncho() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A0_PAPER_SIZE.getAncho())) &&
				(((auxSize.getAlto() * PULGADA) / DPISCREEN) < (tol +
				METRIC_A0_PAPER_SIZE.getAlto()))) {
			return A0;
		}

		return A4;
	}

	/**
	 * Devuelve un Objeto XMLEntity con la informacin los atributos necesarios
	 * para poder despus volver a crear el objeto original.
	 *
	 * @return XMLEntity.
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className",this.getClass().getName());
		xml.putProperty("DPI", DPI);
		xml.putProperty("DPISCREEN", DPISCREEN);
		xml.putProperty("m_TypeUnit", m_TypeUnit);
		xml.putProperty("m_numX", m_numX);
		xml.putProperty("m_numY", m_numY);
		xml.putProperty("m_unitX", m_unitX);
		xml.putProperty("m_unitY", m_unitY);
		xml.putProperty("hasmargin", hasmargin);
		xml.putProperty("m_resolutionSel", m_resolutionSel);
		xml.putProperty("m_typeSel", m_typeSel);
		xml.putProperty("m_selTypeUnit", m_selTypeUnit);
		xml.addChild(m_sizePaper.getXMLEntity());
		xml.putProperty("m_area", m_area);
		xml.putProperty("anchoXalto", anchoXalto);
		xml.addChild(m_sizeinUnits.getXMLEntity());

		// Landscape
		xml.putProperty("m_isLandSel", m_isLandSel);

		return xml;
	}

	/**
	 * Crea un Objeto de esta clase a partir de la informacin del XMLEntity.
	 *
	 * @param xml XMLEntity
	 *
	 * @return Objeto de esta clase.
	 */
	public static Attributes createAtributes(XMLEntity xml) {
		Attributes atri = new Attributes();
		DPI = xml.getIntProperty("DPI");
		if (DPI==300){
			atri.m_resolution=PrintQuality.NORMAL;
		}else if (DPI==600){
			atri.m_resolution=PrintQuality.HIGH;
		}else if (DPI==72){
			atri.m_resolution=PrintQuality.DRAFT;
		}
		DPISCREEN = xml.getIntProperty("DPISCREEN");
		atri.m_TypeUnit = xml.getDoubleProperty("m_TypeUnit");
		atri.setNumUnitsX(xml.getDoubleProperty("m_numX"));
		atri.setNumUnitsY(xml.getDoubleProperty("m_numY"));
		atri.m_unitX = xml.getDoubleProperty("m_unitX");
		atri.m_unitY = xml.getDoubleProperty("m_unitY");
		atri.hasmargin = xml.getBooleanProperty("hasmargin");
		atri.m_resolutionSel = xml.getIntProperty("m_resolutionSel");
		atri.m_typeSel = xml.getIntProperty("m_typeSel");
		atri.m_selTypeUnit = xml.getIntProperty("m_selTypeUnit");
		atri.m_sizePaper = Size.createSize(xml.getChild(0));

		if (atri.hasmargin) {
			atri.m_area = xml.getDoubleArrayProperty("m_area");
		}

		atri.anchoXalto = xml.getDoubleProperty("anchoXalto");
		atri.m_sizeinUnits = Size.createSize(xml.getChild(1));
		atri.m_isLandSel = xml.getBooleanProperty("m_isLandSel");
		atri.setIsLandScape(atri.m_isLandSel);
		return atri;
	}

	/**
	 * Crea un Objeto de esta clase a partir de la informacin del XMLEntity.
	 *
	 * @param xml XMLEntity
	 *
	 * @return Objeto de esta clase.
	 */
	public static Attributes createAtributes03(XMLEntity xml) {
		Attributes atri = new Attributes();
		DPI = xml.getIntProperty("DPI");
		if (DPI==300){
			atri.m_resolution=PrintQuality.NORMAL;
		}else if (DPI==600){
			atri.m_resolution=PrintQuality.HIGH;
		}else if (DPI==72){
			atri.m_resolution=PrintQuality.DRAFT;
		}
		DPISCREEN = xml.getIntProperty("DPISCREEN");
		atri.m_TypeUnit = xml.getDoubleProperty("m_TypeUnit");
		atri.setNumUnitsX(xml.getDoubleProperty("m_numX"));
		atri.setNumUnitsY(xml.getDoubleProperty("m_numY"));
		atri.m_unitX = xml.getDoubleProperty("m_unitX");
		atri.m_unitY = xml.getDoubleProperty("m_unitY");
		atri.hasmargin = xml.getBooleanProperty("hasmargin");
		atri.m_resolutionSel = xml.getIntProperty("m_resolutionSel");
		atri.m_typeSel = xml.getIntProperty("m_typeSel");
		atri.m_selTypeUnit = xml.getIntProperty("m_selTypeUnit");
		atri.m_sizePaper = Size.createSize(xml.getChild(0));

		if (atri.hasmargin) {
			atri.m_area = xml.getDoubleArrayProperty("m_area");
		}

		atri.anchoXalto = xml.getDoubleProperty("anchoXalto");
		atri.m_sizeinUnits = Size.createSize(xml.getChild(1));
		atri.m_isLandSel = xml.getBooleanProperty("m_isLandSel");
		atri.setIsLandScape(atri.m_isLandSel);
		return atri;
	}

	public static void setDefaultGridGap(double hGap, double vGap) {
		defaultGridGap = new Point2D.Double(hGap, vGap);
	}

	public static Point2D getDefaultGridGap() {
		if (defaultGridGap == null){
			XMLEntity xml = PluginServices.getPluginServices("com.iver.cit.gvsig").getPersistentXML();
			double hGap = xml.contains("DefaultLayoutGridHorizontalGapX") ? xml.getDoubleProperty("DefaultLayoutGridHorizontalGapX"): 0.25;
			double vGap = xml.contains("DefaultLayoutGridHorizontalGapY") ? xml.getDoubleProperty("DefaultLayoutGridHorizontalGapY"): 0.25;
			defaultGridGap = new Point2D.Double(hGap, vGap);
		}
		return defaultGridGap;
	}
}
