/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project.documents.view.toolListeners;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.util.BitSet;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.engine.data.DataSource;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PointListener;
import com.iver.cit.gvsig.gui.panels.LinkPanel;
import com.iver.cit.gvsig.project.documents.view.IProjectView;
import com.iver.cit.gvsig.project.documents.view.gui.View;


/**
 * Listener que implementa PointListener para hacer el hiperlink.
 *
 * @author Vicente Caballero Navarro
 *
 */
public class LinkListener implements PointListener {
	private static Logger logger = Logger.getLogger(InfoListener.class.getName());
	private final Image img = new ImageIcon(MapControl.class.getResource(
				"images/LinkCursor.gif")).getImage();
	private Cursor cur = Toolkit.getDefaultToolkit().createCustomCursor(img,
			new Point(16, 16), "");
	private MapControl mapCtrl;
	private BitSet[] bitset;
	private FLayer[] layers;
	private String field;
	private String ext="";
	private int type;

	/**
	 * Crea un nuevo AreaListenerImpl.
	 *
	 * @param mc DOCUMENT ME!
	 */
	public LinkListener(MapControl mc) {
		this.mapCtrl = mc;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param event DOCUMENT ME!
	 * @throws BehaviorException
	 *
	 * @throws DriverException
	 * @throws SAXException
	 *
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PointListener#point(com.iver.cit.gvsig.fmap.tools.Events.PointEvent)
	 */
	public void point(PointEvent event) throws BehaviorException {
		Point2D pReal = mapCtrl.getMapContext().getViewPort().toMapPoint(event.getPoint());
		View view=(View)PluginServices.getMDIManager().getActiveWindow();
		FLayer[] sel = mapCtrl.getMapContext().getLayers().getActives();

		bitset = new BitSet[sel.length];
		layers = new FLayer[sel.length];

		for (int i = 0; i < sel.length; i++) {
			FLayer laCapa = (FLayer) sel[i];
            if (laCapa instanceof FLyrVect)
            {
                    // Tolerancia de 3 pixels
                FLyrVect lyrVect = (FLyrVect) laCapa;
                double tol = mapCtrl.getViewPort().toMapDistance(3);

                FBitSet newBitSet;
                try {
                    newBitSet = lyrVect.queryByPoint(pReal, tol);
                    bitset[i] = newBitSet;
                    layers[i] = laCapa;
                } catch (DriverException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                    throw new BehaviorException("Fallo al consultar " + lyrVect.getName());
                }
            }
			/* if ((laCapa instanceof VectorialData) &&
					(laCapa instanceof AlphanumericData)) {
				QueryByPointVisitor visitor = new QueryByPointVisitor();
				visitor.setLayer(laCapa);
				visitor.setQueriedPoint(pReal);
				visitor.setTolerance(mapCtrl.getMapContext().getViewPort()
											.toMapDistance(3));
				try {
					((VectorialData) laCapa).process(visitor);
				} catch (DriverException e) {
					throw new BehaviorException("No se pudo procesar la capa",e);
				} catch (VisitException e) {
					throw new RuntimeException(e);
				}
				bitset[i] = visitor.getBitSet();
				layers[i] = laCapa;
			} */
		}

		if (bitset != null) {
			for (int i = 0; i < bitset.length; i++) {
				try {
					IProjectView model=view.getModel();
					type=model.getTypeLink();
					field=model.getSelectedField();
					//field = "NOM_PROVIN";
					//ext=".png";
					ext=model.getExtLink();
					if (layers[i] instanceof AlphanumericData) {

						DataSource ds = ((AlphanumericData) layers[i]).getRecordset();

						ds.start();

						//boolean exist=false;
						int idField;
						idField = ds.getFieldIndexByName(field);
						if (idField != -1)
						{
							for (int j = bitset[i].nextSetBit(0); j >= 0;
									j = bitset[i].nextSetBit(j + 1))
							{
								LinkPanel linkpanel = new LinkPanel(field,
										ds.getFieldValue(j, idField).toString(),ext,type);
								PluginServices.getMDIManager().addWindow(linkpanel);

							}
						}

						logger.debug("ds.stop()");
						ds.stop();
					}else {
						JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),"la_capa_no_tiene_hiperenlace");
					}

				} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
					NotificationManager.addError("Error formateando los resultados", e);
				} catch (DriverException e) {
					NotificationManager.addError("Error formateando los resultados", e);
				}
			}

		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 */
	public Cursor getCursor() {
		return cur;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {
		return false;
	}

	public void pointDoubleClick(PointEvent event) throws BehaviorException {
		// TODO Auto-generated method stub

	}
}
