
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package com.iver.cit.gvsig.fmap.edition;

import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.List;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.values.NumericValue;
import com.hardcode.gdbms.engine.values.StringValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.DriverIOExceptionType;
import com.iver.cit.gvsig.fmap.DriverNotLoadedExceptionType;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.IRow;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.Annotation_Layer;
import com.iver.cit.gvsig.fmap.layers.Annotation_Mapping;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;
import com.vividsolutions.jts.geom.Envelope;


/**
 * Annotation Editable Adapter
 *
 * @author Vicente Caballero Navarro
 */
public class Annotation_EditableAdapter extends VectorialEditableAdapter {
	private static String DEFAULT_ANNOTATION_TEXT = "default_annotation_text";

	private static String DEFAULT_ANNOTATION_TYPEFONT = "default_annotation_typefont";

	private static String DEFAULT_ANNOTATION_ROTATE = "default_annotation_rotate";

	private static String DEFAULT_ANNOTATION_STYLEFONT = "default_annotation_stylefont";

	private static String DEFAULT_ANNOTATION_HEIGHT = "default_annotation_height";

	private static String DEFAULT_ANNOTATION_COLOR = "default_annotation_color";



	private Annotation_Mapping mapping;

    private Annotation_Layer lyrAnnotation;

     public int doAddRow(IRow feat, int sourceType) throws DriverIOException,
			IOException {
		int position = super.doAddRow(feat, sourceType);
		PluginServices ps = PluginServices
				.getPluginServices("com.iver.cit.gvsig.annotation");
		XMLEntity xml = ps.getPersistentXML();
		String newFID=getNewFID();
		boolean cancel = fireBeforeRowAdded(sourceType,newFID);
		if (cancel)
			return -1;
		Value[] values = feat.getAttributes();

		int intColor = Annotation_Mapping.DEFAULTCOLOR;
		String text = Annotation_Mapping.DEFAULTTEXT;
		String type = Annotation_Mapping.DEFAULTTYPEFONT;
		int style = Annotation_Mapping.DEFAULTSTYLEFONT;
		int height = Annotation_Mapping.DEFAULTHEIGHT;
		int rotate = Annotation_Mapping.DEFAULTROTATE;

		if (xml.contains(DEFAULT_ANNOTATION_COLOR)){
			intColor = StringUtilities.string2Color(xml.getStringProperty(DEFAULT_ANNOTATION_COLOR)).getRGB();
			text = xml.getStringProperty(DEFAULT_ANNOTATION_TEXT);
			type = xml.getStringProperty(DEFAULT_ANNOTATION_TYPEFONT);
			style = xml.getIntProperty(DEFAULT_ANNOTATION_STYLEFONT);
			height = xml.getIntProperty(DEFAULT_ANNOTATION_HEIGHT);
			rotate = xml.getIntProperty(DEFAULT_ANNOTATION_ROTATE);
		}
		values[mapping.getColumnText()] = ValueFactory.createValue(text);
		values[mapping.getColumnTypeFont()] = ValueFactory.createValue(type);
		values[mapping.getColumnStyleFont()] = ValueFactory.createValue(style);
		values[mapping.getColumnHeight()] = ValueFactory.createValue(height);
		values[mapping.getColumnRotate()] = ValueFactory.createValue(rotate);
		values[mapping.getColumnColor()] = ValueFactory.createValue(intColor);
//		IGeometry geom;
//		ViewPort vp=lyrAnnotation.getMapContext().getViewPort();
//		geom = lyrAnnotation.getTextWrappingGeometry(height, text, rotate,
//					position,vp);
//		feat = new DefaultFeature(geom, values, feat.getID());
		return position;
	}

    public Annotation_EditableAdapter(Annotation_Layer lyrAnnotation) {
        super();
        this.mapping = lyrAnnotation.getAnnotatonMapping();
        this.lyrAnnotation = lyrAnnotation;
    }

    public IRowEdited[] getFeatures(Rectangle2D r, String strEPSG)
        throws DriverException {
        // En esta clase suponemos random access.
        // Luego tendremos otra clase que sea VectorialEditableDBAdapter
        // que reescribir este mtodo.
        Envelope e = FConverter.convertRectangle2DtoEnvelope(r);
        List l = index.query(e);
        IRowEdited[] feats = new IRowEdited[l.size()];

        try {
            for (int index = 0; index < l.size(); index++) {
                Integer i = (Integer) l.get(index);
                int inverse = getInversedIndex(i.intValue());
                feats[index] = (IRowEdited) getRowAnnotation(inverse);
            }
        } catch (IOException ex) {
            DriverIOExceptionType type = new DriverIOExceptionType();
            type.setDriverName(this.getDriver().getName());
            throw new DriverException(ex, type);
        } catch (DriverLoadException ex) {
            DriverNotLoadedExceptionType type = new DriverNotLoadedExceptionType();
            type.setDriverName(getDriver().getName());
            throw new DriverException(ex, type);
        } catch (DriverIOException ex) {
            DriverIOExceptionType type = new DriverIOExceptionType();
            type.setDriverName(this.getDriver().getName());
            throw new DriverException(ex, type);
        }

        return feats;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.iver.cit.gvsig.fmap.edition.IEditableSource#getRow(int)
     */
    public IRowEdited getRowAnnotation(int index)
        throws DriverIOException, IOException {
        int calculatedIndex = getCalculatedIndex(index);
        Integer integer = new Integer(calculatedIndex);

        // Si no est en el fichero de expansin
        DefaultRowEdited edRow = null;
        ViewPort vp=lyrAnnotation.getMapContext().getViewPort();
        try {
        	IFeature f = null;
            if (!relations.containsKey(integer)) {
                f = ova.getFeature(calculatedIndex);

            } else {
                int num = ((Integer) relations.get(integer)).intValue();
                IRowEdited aux = expansionFile.getRow(num);
                f = (IFeature) aux.getLinkedRow().cloneRow();
            }
            String s = f.getID();

            Annotation_Mapping mapping = lyrAnnotation.getAnnotatonMapping();
            NumericValue vRotation = (NumericValue) f.getAttribute(mapping.getColumnRotate());
            NumericValue vHeight = (NumericValue) f.getAttribute(mapping.getColumnHeight());
            NumericValue vStyle = (NumericValue) f.getAttribute(mapping.getColumnStyleFont());
            StringValue vType = (StringValue) f.getAttribute(mapping.getColumnTypeFont());
            Value vText = f.getAttribute(mapping.getColumnText());
            start();
            IGeometry geom = getShape(index);
            stop();
            geom.transform(vp.getAffineTransform());
            IGeometry geom1 = lyrAnnotation.getTextWrappingGeometryInPixels(vHeight.floatValue()*FConstant.FONT_HEIGHT_SCALE_FACTOR,
                    vText.toString(), vRotation.doubleValue(),vType.getValue(),vStyle.intValue(), index, vp,geom);
           	geom1.transform(vp.getAffineTransform().createInverse());
			f = new DefaultFeature(geom1, f.getAttributes(), s);
            edRow = new DefaultRowEdited(f,
                    DefaultRowEdited.STATUS_ORIGINAL, index);

            return edRow;
        } catch (DriverException e) {
            throw new DriverIOException(e);
        } catch (NoninvertibleTransformException e) {
        	throw new DriverIOException(e);
		}
    }
	/**
	 * Si se intenta modificar una geometra original de la capa en edicin se
	 * aade al fichero de expansin y se registra la posicin en la que se
	 * aadi. Si se intenta modificar una geometria que se encuentra en el
	 * fichero de expansin (por ser nueva o original pero modificada) se invoca
	 * el mtodo modifyGeometry y se actualiza el ndice de la geometria en el
	 * fichero.
	 *
	 * @param calculatedIndex
	 *            DOCUMENT ME!
	 * @param feat
	 *            DOCUMENT ME!
	 *
	 * @return position inside ExpansionFile
	 *
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public int doModifyRow(int calculatedIndex, IRow feat, int sourceType)
			throws IOException, DriverIOException {
		boolean cancel = fireBeforeModifyRow(feat, calculatedIndex, sourceType);
		if (cancel)
			return -1;
		int posAnteriorInExpansionFile = -1;
		Integer integer = new Integer(calculatedIndex);

		IFeature featAnt = null;
		IGeometry geom;
//		ViewPort vp=lyrAnnotation.getMapContext().getViewPort();
		Rectangle2D rec=((DefaultFeature)feat).getGeometry().getBounds2D();
		geom = ShapeFactory.createPoint2D(rec.getX(),rec.getY());
		Value[] values=feat.getAttributes();
		feat = new DefaultFeature(geom, values, feat.getID());
		if (!relations.containsKey(integer)) {
			int newPosition = expansionFile.addRow(feat,
					IRowEdited.STATUS_MODIFIED, actualIndexFields);
			relations.put(integer, new Integer(newPosition));

			if (sourceType == EditionEvent.GRAPHIC) {
				// Se actualiza el ndice espacial
				try {
					featAnt = (DefaultFeature) (ova.getFeature(calculatedIndex));
				} catch (DriverException e) {
					throw new DriverIOException(e);
				}
				IGeometry g = featAnt.getGeometry();
				Rectangle2D rAnt = g.getBounds2D();
				Rectangle2D r = ((IFeature) feat).getGeometry().getBounds2D();
				this.index.remove(new Envelope(rAnt.getX(), rAnt.getX()
						+ rAnt.getWidth(), rAnt.getY(), rAnt.getY()
						+ rAnt.getHeight()), new Integer(calculatedIndex));
				this.index.insert(new Envelope(r.getX(), r.getX()
						+ r.getWidth(), r.getY(), r.getY() + r.getHeight()),
						new Integer(calculatedIndex));
			}
		} else {
			// Obtenemos el ndice en el fichero de expansin
			int num = ((Integer) relations.get(integer)).intValue();
			posAnteriorInExpansionFile = num;

			// Obtenemos la geometra para actualiza el ndice
			// espacialposteriormente
			featAnt = (IFeature) expansionFile.getRow(num).getLinkedRow();

			/*
			 * Se modifica la geometra y nos guardamos el ndice dentro del
			 * fichero de expansin en el que se encuentra la geometra
			 * modificada
			 */
			num = expansionFile.modifyRow(num, feat, actualIndexFields);

			/*
			 * Actualiza la relacin del ndice de la geometra al ndice en el
			 * fichero de expansin.
			 */
			relations.put(integer, new Integer(num));
			if (sourceType == EditionEvent.GRAPHIC) {
				// Se modifica el ndice espacial
				Rectangle2D rAnt = featAnt.getGeometry().getBounds2D();
				Rectangle2D r = ((IFeature) feat).getGeometry().getBounds2D();
				this.index.remove(new Envelope(rAnt.getX(), rAnt.getX()
						+ rAnt.getWidth(), rAnt.getY(), rAnt.getY()
						+ rAnt.getHeight()), new Integer(calculatedIndex));
				this.index.insert(new Envelope(r.getX(), r.getX()
						+ r.getWidth(), r.getY(), r.getY() + r.getHeight()),
						new Integer(calculatedIndex));
			}
		}
		isFullExtentDirty = true;
		fireAfterModifyRow(calculatedIndex, sourceType);
		return posAnteriorInExpansionFile;
	}

}
