/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package com.iver.cit.gvsig.project.documents.view.legend.preferences;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagLayout;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.iver.andami.PluginServices;
import com.iver.andami.preferences.AbstractPreferencePage;
import com.iver.andami.preferences.StoreException;
import com.iver.cit.gvsig.fmap.layers.Annotation_Mapping;
import com.iver.cit.gvsig.gui.panels.ColorChooserPanel;
import com.iver.cit.gvsig.project.documents.view.gui.FontOptions;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.swing.JComboBox;
/**
 *  Default configuration page.
 *  <b><b>
 *  Here the user can establish what settings wants to use by default annotations.
 *
 *
 * @author Vicente Caballero Navarro
 */
public class Annotation_Preferences extends AbstractPreferencePage {
	public static String DEFAULT_ANNOTATION_TEXT = "default_annotation_text";

	public static String DEFAULT_ANNOTATION_TYPEFONT = "default_annotation_typefont";

	public static String DEFAULT_ANNOTATION_ROTATE = "default_annotation_rotate";

	public static String DEFAULT_ANNOTATION_STYLEFONT = "default_annotation_stylefont";

	public static String DEFAULT_ANNOTATION_HEIGHT = "default_annotation_height";

	public static String DEFAULT_ANNOTATION_COLOR = "default_annotation_color";

	protected String id;
	private ImageIcon icon;
	private JTextField txtDefaultText;
	private JComboBox cmbDefaultTypeFont;
	private JComboBox cmbDefaultStyleFont;
	private JTextField txtDefaultRotate;
	private JTextField txtDefaultHeight;
	private ColorChooserPanel jccDefaultColor;
//	private JSlider jsDefaultColorAlpha;
	private boolean panelStarted = false;

	/**
	 * Creates a new panel containing View preferences settings.
	 *
	 */
	public Annotation_Preferences() {
		super();
		id = this.getClass().getName();
		icon = new ImageIcon(this.getClass().getClassLoader().getResource("images/AnnotationProperties.png"));
	}

	public void initializeValues() {
		if (!panelStarted) getPanel();

		PluginServices ps = PluginServices.getPluginServices(this);
		XMLEntity xml = ps.getPersistentXML();

		if (xml.contains(DEFAULT_ANNOTATION_TEXT)) {
			String text=xml.getStringProperty(DEFAULT_ANNOTATION_TEXT);
			txtDefaultText.setText(text);
			Annotation_Mapping.DEFAULTTEXT=text;
		}else{
			txtDefaultText.setText("");
			Annotation_Mapping.DEFAULTTEXT="";
		}
		if (xml.contains(DEFAULT_ANNOTATION_TYPEFONT)) {
			String text=xml.getStringProperty(DEFAULT_ANNOTATION_TYPEFONT);
			cmbDefaultTypeFont.setSelectedItem(text);
			Annotation_Mapping.DEFAULTTYPEFONT=text;
		}else{
			cmbDefaultTypeFont.setSelectedItem(FontOptions.ARIAL);
			Annotation_Mapping.DEFAULTTYPEFONT=FontOptions.ARIAL;
		}
		if (xml.contains(DEFAULT_ANNOTATION_STYLEFONT)) {
			int styleFont=xml.getIntProperty(DEFAULT_ANNOTATION_STYLEFONT);
			if (styleFont!=-1){
				String style=FontOptions.getFontStyles()[styleFont];
				cmbDefaultStyleFont.setSelectedItem(style);
				Annotation_Mapping.DEFAULTSTYLEFONT=styleFont;
			}
		}else{
			cmbDefaultStyleFont.setSelectedItem(FontOptions.PLAIN);
			Annotation_Mapping.DEFAULTSTYLEFONT=Font.PLAIN;
		}
		if (xml.contains(DEFAULT_ANNOTATION_COLOR)) {
			String stringColor=xml.getStringProperty(DEFAULT_ANNOTATION_COLOR);
			Color color=StringUtilities.string2Color(stringColor);
			jccDefaultColor.setColor(color);
			jccDefaultColor.setAlpha(color.getAlpha());
//			jsDefaultColorAlpha.setValue(color.getAlpha());
			Annotation_Mapping.DEFAULTCOLOR=color.getRGB();
		}else{
			Color color=Color.black;
			jccDefaultColor.setColor(color);
			jccDefaultColor.setAlpha(color.getAlpha());
//			jsDefaultColorAlpha.setValue(color.getAlpha());
			Annotation_Mapping.DEFAULTCOLOR=color.getRGB();
		}
		if (xml.contains(DEFAULT_ANNOTATION_HEIGHT)) {
			int height=xml.getIntProperty(DEFAULT_ANNOTATION_HEIGHT);
			txtDefaultHeight.setText(String.valueOf(height));
			Annotation_Mapping.DEFAULTHEIGHT=height;
		}else{
			txtDefaultHeight.setText("10");
			Annotation_Mapping.DEFAULTHEIGHT=10;
		}
		if (xml.contains(DEFAULT_ANNOTATION_ROTATE)) {
			int rotate=xml.getIntProperty(DEFAULT_ANNOTATION_ROTATE);
			txtDefaultRotate.setText(String.valueOf(rotate));
			Annotation_Mapping.DEFAULTROTATE=rotate;
		}else{
			txtDefaultRotate.setText("0");
			Annotation_Mapping.DEFAULTROTATE=0;
		}
	}

	public String getID() {
		return id;
	}

	public String getTitle() {
		return PluginServices.getText(this, "annotation_preferences");
	}

	public JPanel getPanel() {
		if (panelStarted) return this;
		panelStarted = true;

		// just a separator
		addComponent(new JLabel(" "));

		addComponent(new JLabel(PluginServices.getText(this,"change_options_of_annotations")));

		JPanel optionsPanel = new JPanel();
		optionsPanel.setBorder(new TitledBorder(PluginServices.getText(this, "options.annotations")));
		optionsPanel.setLayout(new GridBagLayout());

		addComponent(new JLabel(PluginServices.getText(this,"text")),
				txtDefaultText = new JTextField());

		addComponent(new JLabel(PluginServices.getText(this,"fonttype")),
				cmbDefaultTypeFont = new JComboBox());
		String[] types=FontOptions.getFontTypes();
		for (int i=0;i<types.length;i++){
			cmbDefaultTypeFont.addItem(types[i]);
		}
		cmbDefaultTypeFont.setSelectedItem(FontOptions.ARIAL);
		addComponent(new JLabel(PluginServices.getText(this,"fontstyle")),
				cmbDefaultStyleFont = new JComboBox());
		String[] styles=FontOptions.getFontStyles();
		for (int i=0;i<styles.length;i++){
			cmbDefaultStyleFont.addItem(styles[i]);
		}
		cmbDefaultStyleFont.setSelectedItem(FontOptions.PLAIN);
		addComponent(new JLabel(PluginServices.getText(this,"fontheight")),
				txtDefaultHeight = new JTextField());

		addComponent(new JLabel(PluginServices.getText(this,"fontrotate")),
				txtDefaultRotate = new JTextField());

		addComponent(new JLabel(PluginServices.getText(this,"fontcolor")),
				jccDefaultColor = new ColorChooserPanel());
//				jsDefaultColorAlpha = new JSlider());

		//addComponent(optionsPanel);
		addComponent(new JLabel(" "));

		initializeValues();
		return this;
	}

	public void storeValues() throws StoreException {
		Color fontColor=jccDefaultColor.getColor();
		String text=txtDefaultText.getText();
		String fontType=(String)cmbDefaultTypeFont.getSelectedItem();
		int fontStyle=cmbDefaultStyleFont.getSelectedIndex();
		int fontHeight=Integer.parseInt(txtDefaultHeight.getText());
		int fontRotate=Integer.parseInt(txtDefaultRotate.getText());

		PluginServices ps = PluginServices.getPluginServices(this);
		XMLEntity xml = ps.getPersistentXML();

		xml.putProperty(DEFAULT_ANNOTATION_COLOR,
			StringUtilities.color2String(fontColor));
		xml.putProperty(DEFAULT_ANNOTATION_TEXT,
			text);
		xml.putProperty(DEFAULT_ANNOTATION_TYPEFONT,
			fontType);
		xml.putProperty(DEFAULT_ANNOTATION_STYLEFONT,
				fontStyle);
		xml.putProperty(DEFAULT_ANNOTATION_HEIGHT,
				fontHeight);
		xml.putProperty(DEFAULT_ANNOTATION_ROTATE,
				fontRotate);
	}


	public void initializeDefaults() {
		Color fontColor=Color.black;
		String text="";
		String fontType=FontOptions.ARIAL;
		int fontStyle=Font.PLAIN;
		int fontHeight=10;
		int fontRotate=0;

		jccDefaultColor.setColor(fontColor);
//		jsDefaultColorAlpha.setValue(fontColor.getAlpha());

		txtDefaultText.setText(text);
		cmbDefaultTypeFont.setSelectedItem(fontType);
		cmbDefaultStyleFont.setSelectedItem(FontOptions.PLAIN);
		txtDefaultHeight.setText(String.valueOf(fontHeight));
		txtDefaultRotate.setText(String.valueOf(fontRotate));

		Annotation_Mapping.DEFAULTCOLOR=fontColor.getRGB();
		Annotation_Mapping.DEFAULTTEXT=text;
		Annotation_Mapping.DEFAULTTYPEFONT=fontType;
		Annotation_Mapping.DEFAULTSTYLEFONT=fontStyle;
		Annotation_Mapping.DEFAULTHEIGHT=fontHeight;
		Annotation_Mapping.DEFAULTROTATE=fontRotate;
	}

	public ImageIcon getIcon() {
		return icon;
	}

	public boolean isValueChanged() {
		return super.hasChanged();
	}

	public void setChangesApplied() {
		setChanged(false);
	}
}
