/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.panels;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.project.documents.view.gui.View;
/**
 * Panel que contiene opciones de georreferenciacin a elegir por
 * el usuario. Esta clase no maneja eventos.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class OptionsPanel extends JPanel implements ActionListener{

	
	private JPanel pGeneral = null;
	private JPanel pShowPoints = null;
	private JCheckBox CbShowPoints = null;
	private JLabel jLabel1 = null;
	private GeoreferencingDialog grd = null;
	private String text = null;
	
	/**
	 * This is the default constructor
	 */
	public OptionsPanel(GeoreferencingDialog grd, String text) {
		super();
		this.grd = grd;
		this.text = text;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        this.setLayout(new BorderLayout());
        this.setPreferredSize(new java.awt.Dimension(370,20));
        this.setSize(new java.awt.Dimension(370,20));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getPGeneral(), java.awt.BorderLayout.CENTER);
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			pGeneral = new JPanel();
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.setPreferredSize(new java.awt.Dimension(370,20));
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.gridy = 0;
			gridBagConstraints5.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints5.gridheight = 1;
			gridBagConstraints5.gridwidth = 1;
			pGeneral.add(getPShowPoints(), gridBagConstraints5);
		}
		return pGeneral;
	}
	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPShowPoints() {
		if (pShowPoints == null) {
			FlowLayout flowLayout6 = new FlowLayout();
			pShowPoints = new JPanel();
			pShowPoints.setLayout(flowLayout6);
			pShowPoints.setPreferredSize(new java.awt.Dimension(370,20));
			flowLayout6.setAlignment(java.awt.FlowLayout.LEFT);
			flowLayout6.setHgap(2);
			flowLayout6.setVgap(0);
			pShowPoints.add(getCbCheckBox(), null);
		}
		return pShowPoints;
	}
	/**
	 * This method initializes jCheckBox1	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	public JCheckBox getCbCheckBox() {
		if (CbShowPoints == null) {
			CbShowPoints = new JCheckBox();
			CbShowPoints.setSelected(true);
			CbShowPoints.setText(PluginServices.getText(this, this.text));
			CbShowPoints.setName(this.text);
		}
		return CbShowPoints;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		
		FLyrPoints lyrPoints = grd.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getLyrPoints();
		View theView = null;
		try{
			theView = (View) PluginServices.getMDIManager().getActiveWindow();
		}catch(ClassCastException exc){
			return;
		}
		
		if(e.getSource() == getCbCheckBox()){
			
			if(getCbCheckBox().isSelected()){ //Mostramos el nmero de punto
				grd.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getLyrPoints().setShowNumber(true);
				grd.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().draw();
				grd.getConectorPanel().getAdjustGeorefPanel().getZoomRight().draw();
				theView.getMapControl().getMapContext().invalidate();
			}else{ //Ocultamos el nmero de punto
				grd.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getLyrPoints().setShowNumber(false);
				grd.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().draw();
				grd.getConectorPanel().getAdjustGeorefPanel().getZoomRight().draw();
				theView.getMapControl().getMapContext().invalidate();
			}
		}

	}

	public String getText() {
		return text;
	}
	
}
