/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 */
package es.prodevelop.cit.gvsig.fmap.drivers.jdbc.oracle;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;

import com.hardcode.gdbms.engine.data.SourceInfo;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.driver.FileDriver;
import com.hardcode.gdbms.engine.data.driver.GDBMSDriver;
import com.hardcode.gdbms.engine.data.driver.ReadAccess;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.data.file.AbstractFileDataSource;
import com.hardcode.gdbms.engine.data.file.FileDataSource;
import com.hardcode.gdbms.engine.data.file.FileDataSourceFactory;
import com.hardcode.gdbms.engine.data.file.FileDataWare;
import com.hardcode.gdbms.engine.data.file.FileSourceInfo;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import java.io.File;
import java.io.IOException;

import java.sql.Types;


/**
 * This class is a simple FileDataSource adapter used to create a data source
 * based on a DBF file which contains the oracle <--> epsg codes.
 *
 * @author jldominguez
 *
 */
public class OFileDataSourceAdapter extends AbstractFileDataSource
    implements FileDataSource {
    private File file;
    private FileDriver driver;
    private int sem = 0;
    private int fieldCount = -1;

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#start()
     */
    public void start() throws DriverException {
        try {
            if (sem == 0) {
                driver.open(file);
            }

            sem++;
        }
        catch (IOException e) {
            throw new DriverException(e);
        }
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#stop()
     */
    public void stop() throws DriverException {
        try {
            sem--;

            if (sem == 0) {
                driver.close();
            }
            else if (sem < 0) {
                throw new RuntimeException("DataSource closed too many times");
            }
        }
        catch (IOException e) {
            throw new DriverException(e);
        }
    }

    /**
     * Asigna el driver al adaptador
     *
     * @param driver
     *            The driver to set.
     */
    public void setDriver(FileDriver driver) {
        this.driver = driver;
    }

    /**
     * Sets the source information of the DataSource
     *
     * @param sourceInfo
     *            The file to set.
     */
    public void setSourceInfo(SourceInfo sourceInfo) {
        super.setSourceInfo(sourceInfo);
        file = new File(((FileSourceInfo) sourceInfo).file);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getDriver()
     */
    public ReadAccess getReadDriver() {
        return driver;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getName()
     */
    public String getName() {
        return sourceInfo.name;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.DataSource#getPrimaryKeys()
     */
    public int[] getPrimaryKeys() throws DriverException {
        // The last field is the pk/row in FileDataSources
        return new int[] { getFieldCount() - 1 };
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getFieldCount()
     */
    public int getFieldCount() throws DriverException {
        if (fieldCount == -1) {
            fieldCount = getReadDriver().getFieldCount() + 1;
        }

        return fieldCount;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getFieldName(int)
     */
    public String getFieldName(int fieldId) throws DriverException {
        // last field is the virtual primary key
        if (fieldId == (getFieldCount() - 1)) {
            return "PK";
        }

        return getReadDriver().getFieldName(fieldId);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getFieldType(int)
     */
    public int getFieldType(int i) throws DriverException {
        // last field is the virtual primary key
        if (i == (getFieldCount() - 1)) {
            return Types.BIGINT;
        }

        return getReadDriver().getFieldType(i);
    }

    /**
     * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getFieldValue(long,
     *      int)
     */
    public Value getFieldValue(long rowIndex, int fieldId)
        throws DriverException {
        // last field is the virtual primary key
        if (fieldId == (getFieldCount() - 1)) {
            return ValueFactory.createValue(rowIndex);
        }

        Value v = getReadDriver().getFieldValue(rowIndex, fieldId);

        return (v == null) ? ValueFactory.createNullValue() : v;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.file.FileDataSource#getDriver()
     */
    public Driver getDriver() {
        return driver;
    }

    /**
     * @throws DriverException
     * @see com.hardcode.gdbms.engine.data.DataSource#getDataWare(int)
     */
    public DataWare getDataWare(int mode) throws DriverException {
        try {
            FileDataWare dw = FileDataSourceFactory.newDataWareInstance();
            FileDriver driver;
            driver = (FileDriver) getDataSourceFactory().getDriverManager()
                                      .getDriver(getDriver().getName());
            ((GDBMSDriver) driver).setDataSourceFactory(getDataSourceFactory());
            dw.setDriver(driver);
            dw.setDataSourceFactory(dsf);
            dw.setSourceInfo(getSourceInfo());

            return dw;
        }
        catch (DriverLoadException e) {
            throw new DriverException(e);
        }
    }

    public int getFieldWidth(int i) throws DriverException {
        return getReadDriver().getFieldWidth(i);
    }

    public boolean isVirtualField(int fieldId) throws DriverException {
        // last field is the virtual primary key
        if (fieldId == (this.getFieldCount() - 1)) {
            return true;
        }

        return false;
    }

    public void reload() throws DriverException {
        try {
            sem = 0;
            driver.close();
        }
        catch (IOException e) {
            throw new DriverException(e);
        }

        this.fieldCount = -1;

        this.start();

        this.raiseEventReloaded();
    }
}
