/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 */
package es.prodevelop.cit.gvsig.fmap.drivers.jdbc.oracle;

import com.hardcode.gdbms.engine.values.Value;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;

// import oracle.spatial.geometry.JGeometry;

import oracle.sql.ROWID;
import oracle.sql.STRUCT;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;


/**
 * Oracle feature iterator. An instance of this class is returned whaen gvSIG
 * asks for a feature iterator for a new viewport.
 *
 * @author jldominguez
 *
 */
public class OracleSpatialFeatureIterator implements IFeatureIterator {
    private OracleSpatialDriver driver;
    private ResultSet rs = null;
    private int oneBasedGeoColInd = 0;
    private boolean useGeotools = false;
    private Statement st;

    /**
     * Constructor.
     *
     * @param parent the driver that creates it
     * @param _rs the result set to be iterated (already computed by the driver)
     * @param _st the statement that generated the result set. The iterator will close it.
     * @param geoColIndex index of the geometry field
     * @param _useGeotools a switch to decide if the geotools classes
     * must be used to deal with geometris
     */
    public OracleSpatialFeatureIterator(OracleSpatialDriver parent,
        ResultSet _rs, Statement _st, int geoColIndex, boolean _useGeotools) {
        driver = parent;
        rs = _rs;
        st = _st;
        useGeotools = _useGeotools;
        oneBasedGeoColInd = geoColIndex;
    }

    public boolean hasNext() throws DriverException {
        if (rs == null) {
            return false;
        }

        try {
            boolean _resp = rs.next();

            if (!_resp) {
                rs.close();
                st.close();
            }

            return _resp;
        }
        catch (SQLException se) {
            System.err.println("Error while doing hasNext(): " +
                se.getMessage());
        }

        return false;
    }

    public IFeature next() throws DriverException {
        if (rs == null) {
            return null;
        }

        IFeature ife = null;

        try {
            ROWID ri = (ROWID) rs.getObject(1);
            Value[] atts = driver.getAttributes(rs);
            String gid = ri.stringValue();
            STRUCT _st = (oracle.sql.STRUCT) rs.getObject(oneBasedGeoColInd);
            IGeometry theGeom = driver.getGeometryUsing(_st, useGeotools);
            ife = new DefaultFeature(theGeom, atts, gid);
        }
        catch (SQLException se) {
            System.err.println("Error while doing next(): " + se.getMessage());
        }

        return ife;
    }

    public void closeIterator() throws DriverException {
        try {
            rs.close();
            st.close();
        }
        catch (SQLException se) {
            throw new DriverException("SQL Exception: " + se.getMessage());
        }
    }

    /**
     * Utility method to get the oracle geometry type as a human-readable String.
     *
     * @param type the oracle geometry type
     * @return a human-readable String describing it.
     */
    public static String getJGeometryTypeName(int type) {
        String resp = "Unknown JGeometry type (" + type + ")";

        switch (type) {
        case OracleSpatialDriver.JGeometry_GTYPE_COLLECTION:
            resp = "Collection";

            break;

        case OracleSpatialDriver.JGeometry_GTYPE_CURVE:
            resp = "Curve";

            break;

        case OracleSpatialDriver.JGeometry_GTYPE_MULTICURVE:
            resp = "Multi-curve";

            break;

        case OracleSpatialDriver.JGeometry_GTYPE_MULTIPOINT:
            resp = "Multi-point";

            break;

        case OracleSpatialDriver.JGeometry_GTYPE_MULTIPOLYGON:
            resp = "Multi-polygon";

            break;

        case OracleSpatialDriver.JGeometry_GTYPE_POINT:
            resp = "Point";

            break;

        case OracleSpatialDriver.JGeometry_GTYPE_POLYGON:
            resp = "Polygon";

            break;
        }

        return resp;
    }
}
