package es.prodevelop.cit.gvsig.jdbc_spatial;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.SourceInfo;
import com.hardcode.gdbms.engine.data.file.FileSourceInfo;
import com.iver.cit.gvsig.fmap.drivers.dbf.DBFDriver;
import com.iver.cit.gvsig.fmap.edition.EditableAdapter;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.project.documents.table.ProjectTable;
import com.iver.cit.gvsig.project.documents.table.ProjectTableFactory;

import es.prodevelop.cit.gvsig.fmap.drivers.jdbc.oracle.OFileDataSourceAdapter;
import es.prodevelop.cit.gvsig.fmap.drivers.jdbc.oracle.OracleSpatialDriver;

public class OraEpsgTableLoader {
	
	private static Logger logger = Logger.getLogger(OraEpsgTableLoader.class.getName());	
	
    public boolean createOracleEpsgTable() {
    	
    	logger.debug("createOracleEpsgTable() -- ORACLE_EPSG_TABLE_NAME = "
    			+ OracleSpatialDriver.ORACLE_EPSG_TABLE_NAME);
    	
        SourceInfo si = LayerFactory.getDataSourceFactory()
                                    .getDriverInfo(OracleSpatialDriver.ORACLE_EPSG_TABLE_NAME);
        
    	logger.debug("createOracleEpsgTable() -- si = " + si);

        // Create 'oracle codes - epsg codes' table
        DBFDriver dbfdrv = new DBFDriver();
        
        logger.debug("createOracleEpsgTable() -- dbfdrv = " + dbfdrv);        

        // dbfdrv.setDataSourceFactory()
        OFileDataSourceAdapter fdsa = new OFileDataSourceAdapter();
        
        logger.debug("createOracleEpsgTable() -- fdsa = " + fdsa);        

        fdsa.setDriver(dbfdrv);

        // ---------------------------------------------
        FileSourceInfo fsi = new FileSourceInfo();
        fsi.file = createFileString("dbf", OracleSpatialDriver.ORACLE_EPSG_FILE_NAME);
        
        logger.debug("createOracleEpsgTable() -- fsi.file = " + fsi.file);        

        fsi.spatial = false;
        fsi.name = OracleSpatialDriver.ORACLE_EPSG_TABLE_NAME;
        fsi.driverName = dbfdrv.getName();

        fdsa.setSourceInfo(fsi);

        SelectableDataSource sds = null;
        EditableAdapter ea = new EditableAdapter();
        ProjectTable pt = null;

        try {
            sds = new SelectableDataSource(fdsa);
            logger.debug("createOracleEpsgTable() -- sds = " + sds);
            ea.setOriginalDataSource(sds);
            pt = ProjectTableFactory.createTable(
            		OracleSpatialDriver.ORACLE_EPSG_TABLE_NAME, ea);
        } catch (Exception ex) {
            logger.error("While creating datasource: " + ex.getMessage());
            return false;
        }
        
        logger.debug("createOracleEpsgTable() -- pt = " + pt);

        sds.setSourceInfo(fsi);

        DataSourceFactory dsf = LayerFactory.getDataSourceFactory();
        logger.debug("createOracleEpsgTable() -- dsf = " + dsf);
        dsf.addFileDataSource(fsi.driverName, fsi.name, fsi.file);
        sds.setDataSourceFactory(dsf);
        
        return true;
    }
    
    private String createFileString(String folder, String _filename) {
    	
    	String filename = _filename.toUpperCase();
    	
    	logger.debug("createFileString -- filename = " + filename);
    	java.net.URL dbf_url;
    	
        try {
        	dbf_url = createResourceUrl(folder + "/" + filename);
        	if (dbf_url == null) {
        		dbf_url = createResourceUrl(folder + "/" + filename.toLowerCase()); 
        	}
        } catch (Throwable th) {
        	logger.error("Unable to find resource: " + _filename);
            return folder + "/" + _filename;
        }
        return dbf_url.getFile();
    }
    
    private java.net.URL createResourceUrl(String path) {
        return getClass().getClassLoader().getResource(path);
    }
}
