package com.iver.cit.gvsig.gui.panels;

import java.awt.Color;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultListModel;
import javax.swing.JOptionPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;
import org.gvsig.gui.beans.Messages;
import org.gvsig.gui.beans.filterPanel.filterQueryPanel.FilterQueryJPanel;
import org.gvsig.remoteClient.gml.schemas.XMLElement;
import org.gvsig.remoteClient.gml.types.IXMLType;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.instruction.IncompatibleTypesException;
import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.ComplexValue;
import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.IWindowListener;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.edition.EditableAdapter;
import com.iver.cit.gvsig.fmap.edition.VectorialEditableAdapter;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.gui.filter.ExpressionDataSource;
import com.iver.cit.gvsig.gui.filter.ExpressionListener;
import com.iver.cit.gvsig.gui.filter.FilterException;
import com.iver.cit.gvsig.gui.panels.attributesTree.AttributesTreeTableModel;
import com.iver.cit.gvsig.gui.panels.sqlQueryValidation.SQLQueryValidation;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.documents.table.ProjectTable;
import com.iver.cit.gvsig.project.documents.table.ProjectTableFactory;
import com.iver.cit.gvsig.project.documents.table.gui.Table;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.utiles.DefaultCharSet;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.exceptionHandling.ExceptionHandlingSupport;
import com.iver.utiles.exceptionHandling.ExceptionListener;
import com.iver.utiles.stringNumberUtilities.StringNumberUtilities;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This will be the tab for add a filter to a WFS query.
 * This class gets the graphical interface from FilterQueryJPanel and add logic.
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class WFSFilterPanel extends FilterQueryJPanel implements IWindow, IWindowListener {
	private static Logger logger = Logger.getLogger(Table.class.getName());
	private WFSParamsPanel parent = null;
	private ArrayList expressionListeners = new ArrayList();
	private ExpressionDataSource model = null;
	private NumberFormat nf = NumberFormat.getNumberInstance();
	private ExceptionHandlingSupport exceptionHandlingSupport = new ExceptionHandlingSupport();
	private AttributesTreeTableModel attributesTreeTableModel;
	private boolean panelAsATabForWFSLayersLoad;
	private TreePath currentPath;
	private String featureName;
	private Map allFieldsAndValuesKnownOfCurrentLayer; // This will have all values (not repeated) known of all fields (not repeated)
	
	///// GUI METHODS ////
	
	/**
	 * This method initializes
	 * 
	 * @param parent A reference to the parent container component of this component
	 */
	public WFSFilterPanel(WFSParamsPanel parent) {
		super();
		this.parent = parent;
		currentPath = null;
		featureName = null;
		allFieldsAndValuesKnownOfCurrentLayer = new HashMap(); // Initial capacity = 0

		// At beginning, the JList is disabled (and its set a particular color for user could knew it)
		super.getValuesJList().setEnabled(false);
		getValuesJList().setBackground(new Color(220, 220, 220));
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#initialize()
	 */
	protected void initialize() {
		super.initialize();
		this.resizeHeight(380);

		defaultTreeModel = (DefaultTreeModel)fieldsJTree.getModel();
		
		this.addNewListeners();
		panelAsATabForWFSLayersLoad = true;		
	}
	
	/**
	 * Adds some more listener to the components of the panel
	 */
	private void addNewListeners() {
		
		// Enable "Apply" button when user changes the filter query
		txtExpression.getDocument().addDocumentListener(new DocumentListener() {
			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
			 */
			public void changedUpdate(DocumentEvent e) {
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
			 */
			public void insertUpdate(DocumentEvent e) {
				if (!panelAsATabForWFSLayersLoad)
					parent.isApplicable(true);
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
			 */
			public void removeUpdate(DocumentEvent e) {
				if (!panelAsATabForWFSLayersLoad)
					parent.isApplicable(true);
			}
		});
		
		// Listener for "fieldsJTree" 
		getFieldsJTree().addMouseListener(new MouseAdapter() {
			/*
			 *  (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				int row = fieldsJTree.getRowForLocation(e.getX(), e.getY());
				TreePath treePath = fieldsJTree.getPathForLocation(e.getX(), e.getY());

				if (row > -1) {
					switch (e.getClickCount()) {
						case 2:							
							putSymbolOfSelectedByMouseBranch(treePath);
							break;
					}
				}
			}
		});
		
		// Listener for "valuesJList"
		getValuesJList().addMouseListener(new MouseAdapter() {
			/*
			 *  (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				int index = getValuesJList().getSelectedIndex();
				
				// Avoids exception when no value is in the list
				if (index == -1)
					return;
					
				if (e.getClickCount() == 2){
					String valor = (String) valuesListModel.getElementAt(index);
					
					// If value is an string -> set it between apostrophes
					if (getNodeOfCurrentPath().getEntityType().getName().compareTo("xs:string") == 0) {
						putSymbol("'" + valor + "'");
					}
					else {
						putSymbol(valor);
					}
				}
			}
		});
		
		// Listener for a branch of the tree selection
		getFieldsJTree().addTreeSelectionListener(new TreeSelectionListener() {
			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
			 */
			public void valueChanged(TreeSelectionEvent e) {
				if (!panelAsATabForWFSLayersLoad) {					
					DataReturnedOfDataLoadingFromActiveView data = DataLoadingFromActiveView.getDefaultExpressionDataSource();
		
					if ((data != null) && (data.getData() != null)) {
//						setModel(data.getData());
						currentPath = e.getPath();
						fillValuesByPath(currentPath);
					}
				}
			}			
		});
		
		// Listener: when a user writes something on the textarea -> set it's foreground color to black
		getTxtExpression().getDocument().addDocumentListener(new DocumentListener() {
			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
			 */
			public void changedUpdate(DocumentEvent e) {
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
			 */
			public void insertUpdate(DocumentEvent e) {
				getTxtExpression().setForeground(Color.BLACK);
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
			 */
			public void removeUpdate(DocumentEvent e) {				
				getTxtExpression().setForeground(Color.BLACK);
			}			
		});
	}
	
	/**
	 * Gets the element that the 'currentPath' field aims
	 * 
	 * @return An XMLElement
	 */
	private XMLElement getNodeOfCurrentPath() {
		
		if (currentPath != null) {
			Object node = currentPath.getLastPathComponent();
			
			if ((node != null) && (node instanceof XMLElement)) {
//				XMLElement element = (XMLElement) node;
				return (XMLElement) node;
			}
		}
		
		return null;
	}
	
	/**
	 * Puts the symbol of selected brach
	 * 
	 * @param mouseEvent A MouseEvent with information  of the selected branch
	 */
	public void putSymbolOfSelectedByMouseBranch(TreePath treePath) {
		// Sets the node selected
		if (treePath != null) {
			Object node = treePath.getLastPathComponent();
			if ((node != null) && (node instanceof XMLElement)) {
				XMLElement element = (XMLElement) node;
				XMLElement parent = element.getParentElement();
				String path = element.getName();
				while (parent.getParentElement() != null){
					path = parent.getName() + "/" + path;
					parent = parent.getParentElement();
				}
				putSymbol("\"" + path + "\"");
			}
		}
	}
	
	/**
	 * Gets the value of the inner attribute: 'panelAsATabForWFSLayersLoad'
	 * 
	 * @return A boolean value
	 */
	public boolean getWFSFilterPanelIsAsTabForWFSLayersLoad() {
		return this.panelAsATabForWFSLayersLoad;
	}
	
	/**
	 * Sets the value of the inner attribute: 'panelAsATabForWFSLayersLoad'
	 * 
	 * @param b A boolean value
	 */
	public void setWFSFilterPanelIsAsTabForWFSLayersLoad (boolean b) {
		this.panelAsATabForWFSLayersLoad = b;
		
		if (this.panelAsATabForWFSLayersLoad == true) {
			// At beginning, the JList is disabled (and its set a particular color for user could knew it)
			super.getValuesJList().setEnabled(false);
			super.getValuesJList().setBackground(new Color(220, 220, 220)); // a grey color
			
			this.allFieldsAndValuesKnownOfCurrentLayer.clear();
		}
		else {
			// Unselect the selected path in the tree (if there was any selected)
			if (this.currentPath != null) {
				this.currentPath = null;
				this.getFieldsJTree().removeSelectionPath(this.getFieldsJTree().getSelectionPath());
			}
			
			// Resets data loaded
			super.getValuesJList().setEnabled(true);
			super.getValuesJList().setBackground(Color.WHITE);
			
			this.getFieldsJTree().removeAll();
			this.getValuesJList().removeAll();
			
			// Updates data associated to view with the new layer data
			this.setNewDataToTable();
			
			// If theres is any table associated to the current view -> update that table/s
//			this.updateTablesThatHasNewData();
			
			// Reads that new data
			DataReturnedOfDataLoadingFromActiveView data = DataLoadingFromActiveView.getDefaultExpressionDataSource();
			
			if ((data != null) && (data.getData() != null)) {
				setModel(data.getData());
//				currentPath = e.getPath();
//				fillValuesByPath(currentPath);
			}
			
			// Loads values known of fields
			this.setValuesKnownOfFields();
		}
	}
	
	/**
	 * Refresh all information about fields
	 * 
	 * @param feature
	 */
	public void refresh(WFSLayerNode feature) {
		featureName = feature.getTitle();
		setFields(feature);
	}
	
	///// END GUI METHODS /////

	///// METHODS FOR THE FILTER QUERY /////
	
	/**
	 * Gets the query that will be send to the server
	 * @return SQL query (just the where part)
	 */
	public String getQuery(){
		String writtenQuery = txtExpression.getText().trim();
		
		// Validate expression
		if (!this.validateExpression(writtenQuery)) {
			getTxtExpression().setForeground(Color.red);
			return null;
		}
		else
			// Codify expression (only if the validation has been successful)
			return this.codifyExpression(writtenQuery); // Ignores the spaces at beginning and end of the chain of characters
	}
	
	/**
	 * Gets the filter expression from the user interface
	 */
	public String getFilterExpressionFromInterface() {
		return getTxtExpression().getText();
	}
	
	/**
	 * Writes the filter expression into the user interface
	 * @param filterExpression An string
	 */
	public void setFilterExpressionIntoInterface(String filterExpression){
		getTxtExpression().setText(filterExpression);
	}
	
	/**
	 * Removes text in the JTextArea that has the filter subconsultation
	 */
	public void removeFilterExpression() {
		getTxtExpression().setText("");
	}
	
	/**
	 * Codifies the expression to ISO ISO-8859_1 and a format that the SQL parser could validate
	 * 
	 * @param expression The expression to be codified 
	 * 
	 * @return The expression codified
	 */
	private String codifyExpression(String expression) {
		String result = new String("");
		
		try
		{			
			// Encode each string of the query
			int index = 0;
			int lastIndex = 0;
			boolean endInnerLoop;
			// Encodes all inner strings to the equivalent codification in ISO-8859_1 with each ' symbol converted to ''
			while (index != -1) {
				index = expression.indexOf("'", index);
				
				// Add the parts of the chain of characters that not are string
				if (index == -1) {
					result += expression.substring(lastIndex, expression.length());
				}
				else {
					result += expression.substring(lastIndex, index).replaceAll(" [ ]+", " ");
				}
				
				lastIndex = index;
				endInnerLoop = false;
				
				// Tries to find each first apostrophe of each string of the query
				if ((index > 0) && (expression.charAt(index - 1) == ' ')) {
					index++;
					
					// Ignore all inner apostrophes and try to find the last of the string
					while (!endInnerLoop)  {
						index = expression.indexOf("'", index);
						index++;
						
						// If we haven't arrived to the finish of the string
						if (index != expression.length()) {
							if ((index == -1) || (expression.charAt(index) == ' ')) {
								result += translateString(expression.substring(lastIndex, index));
								endInnerLoop = true;
							}
						}
						else {
							result += translateString(expression.substring(lastIndex, index));
							endInnerLoop = true;
							index = -1; // Force to finish the external loop
						}
					}
					lastIndex = index;
				}
			}
			
			// Field names are transformated in xix variables that will be analyzed
			// Date(date) is substituted by the correct date format
			result = translateDates(result);
			result = translateNumber(result);
			result = translateWord(result, "true", "1");
			result = translateWord(result, "false", "0");
	
			logger.debug(result);			
		}
		catch(Exception e) {
			JOptionPane.showMessageDialog(this, Messages.getText("error_coding_filter_query"), Messages.getText("error"), JOptionPane.ERROR_MESSAGE);
			return null;
		}

		return result;
	}
	
	/**
	 * Checks the filter expression if it's correct
	 * 
	 * @param query The query expression to analyze
	 * @return True if it's valid or false if not
	 */
	private boolean validateExpression(String query) {
		SQLQueryValidation sQLQueryValidation = new SQLQueryValidation(query, true);
		
		// Analyzes tokens in query
		StringTokenizer tokens = new StringTokenizer(query, " ");
		String token;
		Set operatorNamesAndSymbols = getOperatorNamesAndSymbols();

		while (tokens.hasMoreTokens()) {
			token = tokens.nextToken().trim();
			
			// Tries to find an invalid token
			if (token.length() > 0) {
				// Validates if a supposed field exists
				if ( (token.length() > 2) && (token.charAt(0) == '\"') && (token.charAt(token.length()-1) == '\"') ) {
					if (! this.isAField(token.substring(1, token.length()-1))) {
						JOptionPane.showMessageDialog(null, PluginServices.getText(null, "error_validating_sql_filter_query") + ":\n" + PluginServices.getText(null, "the_token") + ": " + token + " " + PluginServices.getText(null, "isnt_a_field_of_layer"), PluginServices.getText(null, "error"), JOptionPane.ERROR_MESSAGE);
						return false;
					}
				}
				else {
					// If it's an string -> ignore
					if (! ((token.charAt(0) == token.charAt(token.length() - 1)) && (token.charAt(0) == '\''))) {
						
						// If it's a date -> ignore
						int returnValue = validateDate(token);
						
						if (returnValue == 1) {
							JOptionPane.showMessageDialog(null, PluginServices.getText(null, "error_validating_sql_filter_query") + ":\n" + PluginServices.getText(null, "date_format") + " " + token.substring(5, 16) + " " + PluginServices.getText(null, "incorrect"), PluginServices.getText(null, "error"), JOptionPane.ERROR_MESSAGE);
							return false;
						}
						
						if (returnValue == 2) {								
							// Else -> Checks if the current token is a valid number or symbol
							if ((! StringNumberUtilities.isNumber(token)) && (! this.isAnOperatorNameOrSymbol(token, operatorNamesAndSymbols))) {
								JOptionPane.showMessageDialog(null, PluginServices.getText(null, "error_validating_sql_filter_query") + ":\n" + PluginServices.getText(null, "not_valid_token") + ": " + token, PluginServices.getText(null, "error"), JOptionPane.ERROR_MESSAGE);
								return false;
							}
							else
								return true;
						}
					}
				}
			}
		}

		// Tries to validate the query, and if fails shows a message
		if (!sQLQueryValidation.validateQuery()) {
			JOptionPane.showMessageDialog(null, PluginServices.getText(null, "error_validating_sql_filter_query") + ":\n" + sQLQueryValidation.getErrorMessage(), PluginServices.getText(null, "error"), JOptionPane.ERROR_MESSAGE);
			return false;
		}
		else	
			return true;
	}
	
	/**
	 * Returns true if there is a field with the same name as 'text'
	 * 
	 * @param text An string
	 * @return A boolean value
	 */
	private boolean isAField(String text) {
		return this.allFieldsAndValuesKnownOfCurrentLayer.containsKey(text);		
	}
	
	/**
	 * Validates if a text has a correct date format as Date(dd-mmm-yyyy)  (Ex. Date(03-feb-2004) )
	 * 
	 * @param text
	 * @return 0 -> if has a date format; 1 -> if it's a date that has a but format; 2 -> if it isn't a date
	 */
	private int validateDate(String text) {
		// If it's a date -> check if format is correct (Ex.  Date(01-feb-2004) )
		if ( ((text.length() == 17) && (text.startsWith("Date(")) && (text.endsWith(")"))) && (text.charAt(7) == '-') && (text.charAt(11) == '-') ) {
			if ( (StringNumberUtilities.isNaturalNumber(text.substring(5, 7))) && (StringNumberUtilities.isNaturalNumber(text.substring(12, 16))) ) {
				try {
					// If can parse the date -> date with a correct format 
					DateFormat.getDateInstance().parse(text.substring(5, 16));
					return 0;
				} catch (ParseException e) {
					// If can't parse the date -> date with an incorrect format 
					return 1;
				}
			}
			else {
				return 1;
			}
		}
		
		return 2;

	}

	/**
	 * Returns true if there is the 'text' is a symbol or a operator name
	 * 
	 * @param text An string
	 * @return A boolean value
	 */
	private boolean isAnOperatorNameOrSymbol(String text, Set operatorNamesAndSymbols) {		
		return operatorNamesAndSymbols.contains(text);
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 * @deprecated In this moments its a reference of the old Validate Expression method
	 */
	private String oldValidateExpressionMethod() throws ParseException {
		String expression = txtExpression.getText();
//		HashSet variablesIndexes = new HashSet();
//
//		StringBuffer traducida = new StringBuffer();

		//Se transforman los nombres de los campos en las variables xix que analizarn
		//Se quitan los Date(fecha) y se mete la fecha correspondiente
		expression = translateDates(expression);
		expression = translateNumber(expression);
		expression = translateWord(expression, "true", "1");
		expression = translateWord(expression, "false", "0");

		String replacement;
		Pattern patron = Pattern.compile("[^<>!]=");
		Matcher m = patron.matcher(expression);
		int index = 0;

		while (m.find(index)) {
			index = m.start();
			replacement = expression.charAt(index) + "==";
			m.replaceFirst(replacement);
			index++;
		}

		expression = expression.replaceAll("[^<>!]=", "==");

		logger.debug(expression);

		return expression;
	}
	
	///// END METHODS FOR THE FILTER QUERY /////

	///// METHODS FOR TRANSLATE DATA IN FILTER SENTENCES /////

	/** 
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param word DOCUMENT ME!
	 * @param translation DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateWord(String expresion, String word,	String translation) throws ParseException {
		int booleanIndex = 0;
		int endIndex = 0;
		StringBuffer res = new StringBuffer();

		while ((booleanIndex = getIndex(expresion, word, booleanIndex)) != -1) {
			res.append(expresion.substring(endIndex, booleanIndex));
			endIndex = booleanIndex + word.length();
			booleanIndex++;
			res.append(translation);
		}

		if (endIndex < expresion.length()) {
			res.append(expresion.substring(endIndex));
		}

		return res.toString();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateDates(String expresion) throws ParseException {
		//Se obtiene el valor de la fecha
		String date = StringUtilities.substringDelimited(expresion, "Date(", ")", 0);

		if (date == null) {
			return expresion;
		}

		//Se comprueba que no est entre comillas 
		int startIndex = expresion.indexOf(date);

		while (startIndex != -1) {
			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex - 5) +
					expresion.substring(startIndex).replaceFirst(date + "\\)",							
						new Long((filterButtonsJPanel.getDateFormat().parse(date)).getTime()).toString());

			} else {
				startIndex += date.length();
			}
			
			if (date == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(date, startIndex);
		}

		return expresion;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateNumber(String expresion) throws ParseException {
		DefaultCharSet ss = new DefaultCharSet();
		ss.addInterval('0', '9');
		ss.addCharacter(',');
		ss.addCharacter('.');

		String number = StringUtilities.substringWithSymbols(expresion, ss, 0);

		if (number == null) {
			return expresion;
		}

		int startIndex = expresion.indexOf(number);

		while (startIndex != -1) {
			Number n = nf.parse(number);

			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex) +
					expresion.substring(startIndex).replaceFirst(number,
						n.toString());
			} else {
				startIndex += n.toString().length();
			}

			number = StringUtilities.substringWithSymbols(expresion, ss,
					startIndex);

			if (number == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(number, startIndex);
		}

		return expresion;
	}
	
	/**
	 * Encodes an string to ISO 8859_1 with each ' symbol converted to '' 
	 * 
	 * @param text An string started and finished with simple apostrophes
	 * 
	 * @return An string started and finished with simple apostrophes
	 */
	private String translateString(String text) {
		// Encode to the string to ISO 8859_1 (the URL codification)
		try {
			
			// Ignore the first and last apostrophes
			if (text.length() > 2) {
				text = text.substring(1, text.length() -1);
				
				// Convert the string to ISO 8859_1 codification
				text = URLEncoder.encode(text, "8859_1");
				
				// Change '  (%27 code) to '' for the SQL parser (bebore sent the query)
				text = text.replaceAll("\\%27", "\\'\\'");
			}

		} catch (UnsupportedEncodingException e1) {
			e1.printStackTrace();
		}
		
		return "'" + text + "'";
	}
	
	///// END METHODS FOR TRANSLATE DATA IN FILTER SENTENCES /////
	
	///// METHODS FOR MANIPULATE 'fields' and 'values' /////
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	public void setModel(ExpressionDataSource t) {
		try {
			model = t;
            model.start();
        } catch (DriverException e1) {
            NotificationManager.addError(e1.getMessage(), e1);
        }

        try {
        	int numberOfFields = model.getFieldCount();

        	if (numberOfFields > 0) {
        		Vector fields = new Vector(0, 1);
        		int j = 0;
        	
				for (int i = 0; i < numberOfFields; i++) {
					 Object field = model.getFieldName(i);
					
					if (field != null) {
						fields.add(field);
						j++;
					}
				}
			
				attributesTreeTableModel = new AttributesTreeTableModel(fields.toArray());			
        	}
		} catch (FilterException e) {
			throwException(e);
		}
	}
	
	/**
	 * If there is a field selected, show its new values
	 */
	public void updateFieldValues() {
		if (currentPath != null) {
			
			DataReturnedOfDataLoadingFromActiveView data = DataLoadingFromActiveView.getDefaultExpressionDataSource();
						
			if ((data != null) && (data.getData() != null)) {
				setModel(data.getData());				
				fillValuesByPath(currentPath);
				
//				valuesListModel.clear();
				
				// Updates all tables that their data is about the changed view
				this.updateTablesThatHasNewData();				
				
				// Adjust JScrollPanes to the the beginning
				getFieldsJScrollPane().getHorizontalScrollBar().setValue(-1);
				getFieldsJScrollPane().getVerticalScrollBar().setValue(-1);

				// Adjust JScrollPanes to the the beginning
				getValuesJScrollPane().getHorizontalScrollBar().setValue(-1);
				getValuesJScrollPane().getVerticalScrollBar().setValue(-1);

				// Adjusts valuesJList to the default position
				getValuesJList().setSelectedIndex(-1); // No item selected
			}
		}
	}
	
	/**
	 * Sets Fields
	 *
	 * @param feature A Layer node with fields information
	 */
	private void setFields(WFSLayerNode feature) {
		Vector fields = feature.getFields();
				
		this.resetFieldsAndValuesData();
		
		int numberOfFields = fields.size();
		
		if (numberOfFields > 0) {
			Vector fieldBranches = new Vector(0, 1);
			
			for (int i=0; i<fields.size(); i++) {
				XMLElement field = (XMLElement)fields.get(i);

				IXMLType type = field.getEntityType();
				
				if (type != null) {					
					
					switch (type.getType()) {
						case IXMLType.GML_GEOMETRY: // Don't add branch / field
							break;
						case IXMLType.COMPLEX: case IXMLType.SIMPLE: // Add branch / field
							fieldBranches.add(field);
							
							break;
					}
				}
			}
			
			attributesTreeTableModel = new AttributesTreeTableModel(fieldBranches.get(0));
			fieldsJTree.setModel(new AttributesTreeTableModel(fieldBranches.get(0), false));
			
			// Stores the name of all leafs (fields) of treeTableModel			
			Object root = attributesTreeTableModel.getRoot();
			
			if (root != null) {
				Vector fieldsNames = attributesTreeTableModel.getLeafsFromNodeBranch(root);
				
				for (int j = 0; j < fieldsNames.size(); j++) {
					Object field = fieldsNames.get(j);
					
					// Avoid errors
					if ( (! (field instanceof XMLElement)) || (field == null) )
						continue;
					
					// Don't load a geometry field					
					if (((XMLElement)field).getEntityType() != null){
						if ( ((XMLElement)field).getEntityType().getType() == IXMLType.GML_GEOMETRY ){
							continue;
						}
					}				
					
					if (! allFieldsAndValuesKnownOfCurrentLayer.containsKey( ((XMLElement)field).getName()) ) {
						allFieldsAndValuesKnownOfCurrentLayer.put(((XMLElement)field).getName(), new HashMap());
					}
				}
			}
		}
	}

	/**
	 * This method load all values known of all fields known
	 * (It's used when a new layer is load)
	 */
	private void setValuesKnownOfFields() {
		// Desde el modelo se debera acceder a los campos y sus valores cargados		
		try {
			for (int i = 0; i < model.getFieldCount(); i++) {
				String fieldName = model.getFieldName(i);
				HashMap fieldValues = (HashMap) allFieldsAndValuesKnownOfCurrentLayer.get(fieldName);
				
				if (fieldValues != null) {
					for (int j = 0; j < model.getRowCount(); j++) {
						Value value = model.getFieldValue(j, i);					
						
						if (value instanceof NullValue)
						    continue;
						
						Object obj = (Object)value;
						
						if (obj == null)
							continue;
						
						fieldValues.put(obj.toString(), obj.toString());
						
					}
				}
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Resets the data of fields and their values of the current layer feature, and removes the branches of JTree
	 */
	private void resetFieldsAndValuesData() {
		fieldsJTree.setModel(new DefaultTreeModel(new DefaultMutableTreeNode()));
		txtExpression.setText("");
		((DefaultListModel)valuesJList.getModel()).removeAllElements();
	}
	
	/** 
	 * FIlls list with the values of selected field
	 * 
	 * @param treePath A path in the tree
	 */
	private void fillValuesByPath(TreePath treePath) {
		// Duplicates are removed
		TreeSet conjunto = new TreeSet(new Comparator() {
			public int compare(Object o1, Object o2) {
				if ((o1 != null) && (o2 != null)) {
					Value v2 = (Value) o2;
					Value v1 = (Value) o1;
					BooleanValue boolVal;
					
					try {
						boolVal = (BooleanValue) (v1.greater(v2));
						
						if (boolVal.getValue()) {
							return 1;
						}
						
						boolVal = (BooleanValue) (v1.less(v2));
						
						if (boolVal.getValue()) {
							return -1;
						}
					} catch (IncompatibleTypesException e) {
						throw new RuntimeException(e);
					}
				}
				
				return 0;
			}
		}); // For ordernation
		
		// Remove the previous items
		valuesListModel.clear();
		
		try {
			//Object root = treePath.getPath()[0];
			XMLElement element = ((XMLElement)treePath.getLastPathComponent());
			
			// Gets the values associated to the selected branch 
			switch (element.getEntityType().getType()) {
				case IXMLType.SIMPLE:
					
					if(element.getParentElement().getParentElement() == null) {					
						// Find the selected field and try to obtein values related
						for (int i = 0; i < model.getFieldCount(); i++) {					
							String name = model.getFieldName(i);
							
							// If we find the field (this means that are loaded its values and we can obtein them)
							if (name.equals(element.getName())) {						
								for (int j = 0; j < model.getRowCount(); j++) {					
									Value value = model.getFieldValue(j, i);
									
									if (value instanceof NullValue)
									    continue;
									
									if (!conjunto.contains(value)) {
										conjunto.add(value);
									}
								}
								
								break;
							}
						}
					}else{
						//create a vector with the parent names from the leaf until the root
						XMLElement parent = element.getParentElement();
						Vector parentNames = new Vector();
						parentNames.add(element.getName());
						while (parent != null){
							parentNames.add(parent.getName());
							parent = parent.getParentElement();							
						}
						
						//The field name (in the gvSIG table) is the second field name
						String fieldName = (String)parentNames.get(parentNames.size()-2);
						
						for (int i = 0; i < model.getFieldCount(); i++) {					
							String name = model.getFieldName(i);
							
							// If we find the field (this means that are loaded its values and we can obtein them)
							if (name.equals(fieldName)) {						
								for (int j = 0; j < model.getRowCount(); j++) {					
									Value value = model.getFieldValue(j, i);
															
									if (value instanceof NullValue)
									    continue;
									
									if (value instanceof ComplexValue){
										for (int k=parentNames.size()-3 ; k>=0 ; k--){
											ComplexValue complex = (ComplexValue)value;
											Value childValue = (Value)complex.get(parentNames.get(k));
											if (k==0){
												if (!conjunto.contains(childValue)) {
													conjunto.add(childValue);
												}
											}else{
												value = childValue;
											}
										}
									}
								}
								
								break;
							}
						}
					}
					break;
				case IXMLType.COMPLEX:
					break;
				default:
					// Do Nothing
			}
			
			// Add the values to the model of the graphic list
			Iterator it = conjunto.iterator();
			Object[] objects = currentPath.getPath();

			if (objects.length == 0)
				return;
			
			String selectedField = ((XMLElement)objects[objects.length-1]).getName(); // Gets the selected field
			
			if (selectedField != null) { // If there is a selected field
				Map fieldValues = (HashMap)allFieldsAndValuesKnownOfCurrentLayer.get(selectedField); // Gets valus stored associated to this field
				String currentValue = null;
				
				// If the field doesn't exits -> create a new Map with its values
				if (fieldValues == null) {
					fieldValues = new HashMap();
					allFieldsAndValuesKnownOfCurrentLayer.put(selectedField, fieldValues);	
				
					while (it.hasNext()) {
						currentValue = new String(); 
						
						currentValue = it.next().toString();
						
						if (currentValue.compareTo("") != 0) {										
							fieldValues.put(currentValue, currentValue);
						
							// All values loaded in this loop must be at beginning of the list (and in a differenciated color)
							if ( ! valuesListModel.contains(currentValue) )
								valuesListModel.addElement(currentValue);							
						}
					}
				}
				else { // Else -> Adds the new ones, and changes element labels that have changed  (before were loaded and now not, or before weren't loaded but now yes)
					
					// Changes element labels that have changed  (before were loaded and now not, or before weren't loaded but now yes)
//					((DefaultListModelForJLabelAsCell)valuesListModel).setAllElementsToNotLoaded();
					
					// For each current value associated to de current selected field -> if its loaded -> put it at beginning of the list and change to 'JLabelLoadedValue' 
					while (it.hasNext()) {
						String text = it.next().toString();
						//int elementPosition = ((DefaultListModelForJLabelAsCell)valuesListModel).getIndexOfJLabelText(text);
						int elementPosition = valuesListModel.indexOf(text);
						
//						if (! ((DefaultListModelForJLabelAsCell)valuesListModel).containsJLabelText(text) )
						if (elementPosition == -1) // If it must be added						
							valuesListModel.addElement(text);
//						else
//							((DefaultListModelForJLabelAsCell)valuesListModel).changeElementThatHasTextToJLabelLoadedValue(text); // Change to 'JLabelLoadedValue'
						
							
//						// A label with text with yellow background color for values that are loaded in the layer
//						currentValueLabel = new JLabelValueLoaded();
//						
//						currentValueLabel.setText(it.next().toString());											
//						
//						
//						if ( ! fieldValues.containsKey(currentValueLabel.getText()) ) {
//							fieldValues.put(currentValueLabel.getText(), currentValueLabel.getText());
//						}
//						
//						// All values loaded in this loop must be at beginning of the list (and in a differenciated color)
//						if ( ! (valuesListModel).contains(currentValueLabel) )
//							valuesListModel.addElement(currentValueLabel);
//						
//						else
					}
				}
			
//				// Load the rest of the values associated to the current selected field (This version is more simply)
//				if (fieldValues != null) {
//					// A label with text with yelow background color for values that are loaded in the layer
//					currentValueLabel = new JLabelValueNotLoaded();
//
//					Set values = fieldValues.keySet();
//
//					it = values.iterator();
//
//					while (it.hasNext()) {
//						String name = it.next().toString();
//						
//						if ( ! ((DefaultListModelForJLabelAsCell)valuesListModel).containsJLabelText(name) )
//							valuesListModel.addElement(new JLabelValueNotLoaded(name));
//					}
//				}
			}
			
		} catch (Exception e) {
			throwException(e);
		}
	}
	
//	/** 
//	 * FIlls list with the values of selected field
//	 * 
//	 * @param treePath A path in the tree
//	 */
//	private void fillValuesByPath(TreePath treePath) {
//		// Duplicates are removed
//		TreeSet conjunto = new TreeSet(new Comparator() {
//			public int compare(Object o1, Object o2) {
//				if ((o1 != null) && (o2 != null)) {
//					Value v2 = (Value) o2;
//					Value v1 = (Value) o1;
//					BooleanValue boolVal;
//					
//					try {
//						boolVal = (BooleanValue) (v1.greater(v2));
//						
//						if (boolVal.getValue()) {
//							return 1;
//						}
//						
//						boolVal = (BooleanValue) (v1.less(v2));
//						
//						if (boolVal.getValue()) {
//							return -1;
//						}
//					} catch (IncompatibleTypesException e) {
//						throw new RuntimeException(e);
//					}
//				}
//				
//				return 0;
//			}
//		}); // For ordernation
//		
//		// Remove the previous items
//		valuesListModel.clear();
//		
//		try {
//			//Object root = treePath.getPath()[0];
//			XMLElement element = ((XMLElement)treePath.getLastPathComponent());
//			
//			// Gets the values associated to the selected branch 
//			switch (element.getEntityType().getType()) {
//				case IXMLType.SIMPLE:
//					
//					if(element.getParentElement().getParentElement() == null) {					
//						// Find the selected field and try to obtein values related
//						for (int i = 0; i < model.getFieldCount(); i++) {					
//							String name = model.getFieldName(i);
//							
//							// If we find the field (this means that are loaded its values and we can obtein them)
//							if (name.equals(element.getName())) {						
//								for (int j = 0; j < model.getRowCount(); j++) {					
//									Value value = model.getFieldValue(j, i);
//									
//									if (value instanceof NullValue)
//									    continue;
//									
//									if (!conjunto.contains(value)) {
//										conjunto.add(value);
//									}
//								}
//								
//								break;
//							}
//						}
//					}else{
//						//create a vector with the parent names from the leaf until the root
//						XMLElement parent = element.getParentElement();
//						Vector parentNames = new Vector();
//						parentNames.add(element.getName());
//						while (parent != null){
//							parentNames.add(parent.getName());
//							parent = parent.getParentElement();							
//						}
//						
//						//The field name (in the gvSIG table) is the second field name
//						String fieldName = (String)parentNames.get(parentNames.size()-2);
//						
//						for (int i = 0; i < model.getFieldCount(); i++) {					
//							String name = model.getFieldName(i);
//							
//							// If we find the field (this means that are loaded its values and we can obtein them)
//							if (name.equals(fieldName)) {						
//								for (int j = 0; j < model.getRowCount(); j++) {					
//									Value value = model.getFieldValue(j, i);
//															
//									if (value instanceof NullValue)
//									    continue;
//									
//									if (value instanceof ComplexValue){
//										for (int k=parentNames.size()-3 ; k>=0 ; k--){
//											ComplexValue complex = (ComplexValue)value;
//											Value childValue = (Value)complex.get(parentNames.get(k));
//											if (k==0){
//												if (!conjunto.contains(childValue)) {
//													conjunto.add(childValue);
//												}
//											}else{
//												value = childValue;
//											}
//										}
//									}
//								}
//								
//								break;
//							}
//						}
//					}
//					break;
//				case IXMLType.COMPLEX:
//					break;
//				default:
//					// Do Nothing
//			}
//			
//			// Add the values to the model of the graphic list
//			Iterator it = conjunto.iterator();
//			Object[] objects = currentPath.getPath();
//
//			if (objects.length == 0)
//				return;
//			
//			String selectedField = ((XMLElement)objects[objects.length-1]).getName(); // Gets the selected field
//			
//			if (selectedField != null) { // If there is a selected field
//				Map fieldValues = (HashMap)allFieldsAndValuesKnownOfCurrentLayer.get(selectedField); // Gets valus stored associated to this field
//				JLabel currentValueLabel = null;
//				
//				// If the field doesn't exits -> create a new Map with its values
//				if (fieldValues == null) {
//					fieldValues = new HashMap();
//					allFieldsAndValuesKnownOfCurrentLayer.put(selectedField, fieldValues);	
//				
//					while (it.hasNext()) {
//						// A label with text with yelow background color for values that are loaded in the layer (This version in the branch is more simple)
//						currentValueLabel = new JLabelValueNotLoaded(); 
//						
//						currentValueLabel.setText(it.next().toString());
//						
//						if (currentValueLabel.getText().compareTo("") != 0) {										
//							fieldValues.put(currentValueLabel.getText(), currentValueLabel.getText());
//						
//							// All values loaded in this loop must be at beginning of the list (and in a differenciated color)
//							if ( ! valuesListModel.contains(currentValueLabel) )
//								valuesListModel.addElement(currentValueLabel);							
//						}
//					}
//				}
//				else { // Else -> Adds the new ones, and changes element labels that have changed  (before were loaded and now not, or before weren't loaded but now yes)
//					
//					// Changes element labels that have changed  (before were loaded and now not, or before weren't loaded but now yes)
//					((DefaultListModelForJLabelAsCell)valuesListModel).setAllElementsToNotLoaded();
//					
//					// For each current value associated to de current selected field -> if its loaded -> put it at beginning of the list and change to 'JLabelLoadedValue' 
//					while (it.hasNext()) {
//						String text = it.next().toString();
//						int elementPosition = ((DefaultListModelForJLabelAsCell)valuesListModel).getIndexOfJLabelText(text);
//						
////						if (! ((DefaultListModelForJLabelAsCell)valuesListModel).containsJLabelText(text) )
//						if (elementPosition == -1) // If it must be added						
//							valuesListModel.addElement(new JLabelValueNotLoaded(text));
//						else
//							((DefaultListModelForJLabelAsCell)valuesListModel).changeElementThatHasTextToJLabelLoadedValue(text); // Change to 'JLabelLoadedValue'
//						
//							
////						// A label with text with yellow background color for values that are loaded in the layer
////						currentValueLabel = new JLabelValueLoaded();
////						
////						currentValueLabel.setText(it.next().toString());											
////						
////						
////						if ( ! fieldValues.containsKey(currentValueLabel.getText()) ) {
////							fieldValues.put(currentValueLabel.getText(), currentValueLabel.getText());
////						}
////						
////						// All values loaded in this loop must be at beginning of the list (and in a differenciated color)
////						if ( ! (valuesListModel).contains(currentValueLabel) )
////							valuesListModel.addElement(currentValueLabel);
////						
////						else
//					}
//				}
//			
////				// Load the rest of the values associated to the current selected field (This version is more simply)
////				if (fieldValues != null) {
////					// A label with text with yelow background color for values that are loaded in the layer
////					currentValueLabel = new JLabelValueNotLoaded();
////
////					Set values = fieldValues.keySet();
////
////					it = values.iterator();
////
////					while (it.hasNext()) {
////						String name = it.next().toString();
////						
////						if ( ! ((DefaultListModelForJLabelAsCell)valuesListModel).containsJLabelText(name) )
////							valuesListModel.addElement(new JLabelValueNotLoaded(name));
////					}
////				}
//			}
//			
//		} catch (Exception e) {
//			throwException(e);
//		}
//	}
	
	///// END METHODS FOR MANIPULATE 'fields' and 'values' /////
	
	///// METHODS FOR PARENT NOTIFICATIONS /////
	
	/**
	 * @see WFSParamsPanel#isApplicable(boolean)
	 * 
	 * This also loads values of fields 
	 * 
	 * @param b A boolean value
	 */
	private void setApplicate(boolean b) {
		parent.isApplicable(b);
	}
	
	///// END METHODS FOR PARENT NOTIFICATIONS /////
	
	///// METHOS FOR 'allFieldsAndValuesKnownOfCurrentLayer' /////
	
	/**
	 * Sets all fields and values known about the current layer
	 * 
	 * @param _allFieldsAndValuesKnownOfCurrentLayer A Map object
	 */
	public void setAllFieldsAndValuesKnownOfCurrentLayer(Map _allFieldsAndValuesKnownOfCurrentLayer) {
		if (_allFieldsAndValuesKnownOfCurrentLayer == null)
			this.allFieldsAndValuesKnownOfCurrentLayer = new HashMap();
		else
			this.allFieldsAndValuesKnownOfCurrentLayer = _allFieldsAndValuesKnownOfCurrentLayer;
	}
	
	/**
	 * Gets all fields and values known about the current layer
	 * 
	 * @return _allFieldsAndValuesKnownOfCurrentLayer A Map object
	 */
	public Map getAllFieldsAndValuesKnownOfCurrentLayer() {
		return allFieldsAndValuesKnownOfCurrentLayer;
	}
	
	///// METHOS FOR 'allFieldsAndValuesKnownOfCurrentLayer' /////
	
	///// IMPLEMENTATION OF METHODS OF 'IWindowListener' /////
	
    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewActivated()
     */
    public void windowActivated() {
    }

    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewClosed()
     */
    public void windowClosed() {
        try {
            model.stop();
        } catch (DriverException e) {
            NotificationManager.addError(e.getMessage(), e);
        }        
    }
    
    ///// END IMPLEMENTATION OF METHODS OF 'IWindowListener' /////

	///// IMPLEMENTATION OF METHODS OF 'IWindow' /////    
    
    /*
     *  (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.IWindow#getWindowInfo()
     */
	public WindowInfo getWindowInfo() {
		return null;
	}
	
	///// END IMPLEMENTATION OF METHODS OF 'IWindow' /////
	
	///// UPDATE TABLES DATA /////
	
	/**
	 * Updates all tables that their data is about the changed view
	 */
	private void updateTablesThatHasNewData() {
		boolean oneTimeNewDataToTableAdded = false;
		
		IWindow[] activeNoModalWindows = PluginServices.getMDIManager().getAllWindows();
		
		for (int i = 0; i < activeNoModalWindows.length; i++) {
			IWindow window = activeNoModalWindows[i];
			if (window instanceof Table) {
				Table table = (Table) window;
				
				int pos1 = featureName.indexOf(':');
				
				if ((pos1 >= 0) && (pos1 < featureName.length()))						
					featureName = featureName.substring(pos1 +1, featureName.length());
				
//					String featureOfTable = ((XMLElement)currentPath.getParentPath().getLastPathComponent()).getName();
					String featureOfTable = table.getModel().getName();
					int pos2 = featureOfTable.indexOf(':');
												
					if ((pos2 >= 0) && (pos2 < featureName.length()))
						featureOfTable = featureOfTable.substring(pos2 +1, featureOfTable.length());						
				
				if (featureName.trim().compareTo(featureOfTable.trim()) == 0) {
					// Only add the new data associated to the table one time
					if (oneTimeNewDataToTableAdded == false) {
						setNewDataToTable();
						oneTimeNewDataToTableAdded = true;
					}
				
					// Refresh the table with the new data
					table.refresh();
				}
			}
		}		
	}	
	
	/**
	 * This method is a modification of the "execute" method from the "ShowTable" class 
	 *
	 * @see com.iver.cit.gvsig.ShowTable#execute(String)
	 */
	private void setNewDataToTable() {
		View vista = (View) PluginServices.getMDIManager().getActiveWindow();
		FLayer[] actives = vista.getModel().getMapContext().getLayers().getActives();

		try {
			for (int i = 0; i < actives.length; i++) {
				if (actives[i] instanceof AlphanumericData) {
					AlphanumericData co = (AlphanumericData) actives[i];

					//SelectableDataSource dataSource;
					//dataSource = co.getRecordset();

					ProjectExtension ext = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);

					ProjectTable projectTable = ext.getProject().getTable(co);
					EditableAdapter ea=null;
					ReadableVectorial rv=((FLyrVect)actives[i]).getSource();
					if (rv instanceof VectorialEditableAdapter){
						ea=(EditableAdapter)((FLyrVect)actives[i]).getSource();
					}else{
						ea=new EditableAdapter();
						SelectableDataSource sds=((FLyrVect)actives[i]).getRecordset();
						ea.setOriginalDataSource(sds);
					}

					if (projectTable == null) {
						projectTable = ProjectFactory.createTable(PluginServices.getText(this, "Tabla_de_Atributos") + ": " + actives[i].getName(),
								ea);
						projectTable.setProjectDocumentFactory(new ProjectTableFactory());
						projectTable.setAssociatedTable(co);
						ext.getProject().addDocument(projectTable);
					}
					projectTable.setModel(ea);
					
					// Removed part of code that created another (visual) table
//					Table t = new Table();
//					t.setModel(projectTable);
//					PluginServices.getMDIManager().addWindow(t);
				}
			}
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            NotificationManager.addError(PluginServices.getText(this,"No_se_pudo_obtener_la_tabla_de_la_capa"), e);
        } catch (com.iver.cit.gvsig.fmap.DriverException e) {
			e.printStackTrace();
			NotificationManager.addError(PluginServices.getText(this,"No_se_pudo_obtener_la_tabla_de_la_capa"), e);
        }
	}
	
	///// END UPDATE TABLES DATA /////
	
	///// OTHER METHODS /////
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param substring DOCUMENT ME!
	 * @param startingPos DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private int getIndex(String expresion, String substring, int startingPos) {
		int index = startingPos;

		do {
			index = expresion.indexOf(substring, index);
		} while ((StringUtilities.isBetweenSymbols(expresion, index, "\"")) &&
				(index != -1));

		return index;
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean addExpressionListener(ExpressionListener arg0) {
		return expressionListeners.add(arg0);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean removeExpressionListener(ExpressionListener arg0) {
		return expressionListeners.remove(arg0);
	}
	/**
	 * DOCUMENT ME!
	 *
	 * @param o DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean removeExceptionListener(ExceptionListener o) {
		return exceptionHandlingSupport.removeExceptionListener(o);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	private void throwException(Throwable t) {
		exceptionHandlingSupport.throwException(t);
	}
	
	///// END OTHER METHODS ///// 
}
