/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter;

import java.awt.image.DataBuffer;
import es.gva.cit.jgdal.Gdal;


/**
 * Raster con acceso a los elementos individuales (pixeles o datos de celda).
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public interface IRaster {
    public final static int TYPE_UNDEFINED = DataBuffer.TYPE_UNDEFINED;
    public final static int TYPE_BYTE = DataBuffer.TYPE_BYTE;
    public final static int TYPE_SHORT = DataBuffer.TYPE_SHORT;
    public final static int TYPE_USHORT = DataBuffer.TYPE_USHORT;
    public final static int TYPE_INT = DataBuffer.TYPE_INT;
    public final static int TYPE_FLOAT = DataBuffer.TYPE_FLOAT;
    public final static int TYPE_DOUBLE = DataBuffer.TYPE_DOUBLE;
    public final static int TYPE_IMAGE = -1;

    /**
     * Ancho del raster
     * @return Entero con el ancho del raster
     */
    public int getWidth();

    /**
     * Alto del raster
     * @return Entero con el alto del raster
     */
    public int getHeight();

    /**
     * Nmero de bandas
     * @return Entero con el nmero de bandas
     */
    public int getBandCount();

    /**
     * Devuelve el tipo de datos del raster
     * @see java.awt.image.DataBuffer
     */
    public int getDataType();

    /**
     * Obtiene sobre un array de bandas el valor del raster en la
     * coordenada que se le pasa. Los elementos son de 16 bits.
     * @param x        coordenada X
     * @param y coordenada Y
     * @param px Array de 4 elementos para las bandas de la imagen ARGB
     */
    public void getElemShort(int x, int y, short[] px);

    /**
     * Obtiene sobre un array de bandas el valor del raster en la
     * coordenada que se le pasa. Los elementos son de 32 bits.
     * @param x        coordenada X
     * @param y coordenada Y
     * @param px Array de 4 elementos para las bandas de la imagen ARGB
     */
    public void getElemInt(int x, int y, int[] px);

    /**
     * Obtiene el valor del raster en la coordenada que se le pasa.
     * Devuelve un int.
     * @param x        coordenada X
     * @param y coordenada Y
     * @param band banda
     * @return Entero
     */
    public int getElemInt(int x, int y, int band);
}
