/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.bands;

import org.cresques.filter.RasterFilter;
import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Palette;

/**
 * Clase basa para los filtros de paleta.
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */

public abstract class PaletteFilter extends RasterFilter {

	protected Palette palette = null;	
	protected boolean interpolate = false;
	protected RasterBuf rasterResult = null;
	public boolean wmsFilterFlag = false;
	
	/**
	 * Obtiene si la paleta ser aplicada con interpolacin de colores o no.
	 * @return true si es aplicada con interpolacin de colores y false si no lo es
	 */
	public boolean isInterpolate() {
		return interpolate;
	}

	/**
	 * Asigna si la paleta ser aplicada con interpolacin de colores o no.
	 * @param interpolate true si queremos que se aplique con interpolacin y false si no queremos.
	 */
	public void setInterpolate(boolean interpolate) {
		this.interpolate = interpolate;
	}

	public PaletteFilter(){
		super();
	}
	
	public void pre() {
		this.palette = ((Palette) params.get("palette"));
		this.interpolate = ((Boolean)params.get("interpolate")).booleanValue();
	}

	public void post() {
		// TODO Auto-generated method stub

	}

	/**
	 * Obtiene la paleta
	 * @return Palette
	 */
	public Palette getPalette() {
		return palette;
	}

	/**
	 * Asigna una paleta
	 * @param palette 
	 */
	public void setPalette(Palette palette) {
		this.palette = palette;
	}

}
