/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter.bands;

import java.awt.image.BufferedImage;

import org.cresques.filter.RasterFilter;
import org.cresques.io.data.RasterBuf;


/**
 * Renderiza el raster.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class RasterIntToImageFilter extends RasterToImageFilter {
	private int[] 		px = {0, 0, 0, 255};
	 
    /**
     * Constructor
     *
     */
    public RasterIntToImageFilter() {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
       super.pre();
    }

    /* (non-Javadoc)
     * @see org.cresques.px.PxRaster.RasterFilter#process(int, int)
     */
    public void process(int col, int line) {
        raster.getElemInt(line, col, px);
        rgb = (alpha & ((((int)255) & 0xff) << 24)) | ((px[0] & 0xff) << 16) |
              ((px[1] & 0xff) << 8) | (px[2] & 0xff);
        /*rgb = (alpha & ((px[3] & 0xff) << 24)) | ((px[0] & 0xff) << 16) |
        ((px[1] & 0xff) << 8) | (px[2] & 0xff);*/

        image.setRGB(col, line, rgb);
    }
    
	public void processSuperSampling(int col, int line){
		raster.getElemInt(line, col, px);
        rgb = (alpha & ((((int)255) & 0xff) << 24)) | ((px[0] & 0xff) << 16) |
              ((px[1] & 0xff) << 8) | (px[2] & 0xff);
        /*rgb = (alpha & ((px[3] & 0xff) << 24)) | ((px[0] & 0xff) << 16) |
        ((px[1] & 0xff) << 8) | (px[2] & 0xff);*/
        
        for(int j = col; j < raster.getWidth() && j < (col + stepX[contX + 1]); j++)
			for(int i = line; i < raster.getHeight() && i < (line + stepY[contY + 1]); i++)
				image.setRGB(j, i, rgb);
	}

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        return super.getResult(name);
    }

    /* (non-Javadoc)
     * @see org.cresques.px.PxRaster.RasterFilter#processLines(int)
     */
    public void processLine(int y) {
        for (int x = 0; x < width; x++) {
            int[][] line = raster.getLineInt(y);
            rgb = (alpha & ((px[3] & 0xff) << 24)) |
                  ((line[x][0] & 0xff) << 16) | ((line[x][1] & 0xff) << 8) |
                  (line[x][2] & 0xff);

            image.setRGB(x, y, rgb);
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return super.getInRasterDataType();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return super.getOutRasterDataType();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    	super.post();
    }
}
