/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter.enhancement;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;


/**
 * Filtro de transparencia aplicada a un rango de valores de un pixel sobre un objeto image
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class TransparencyImageFilter extends TransparencyFilter {
    /**
     * Constructor
     *
     */
    public TransparencyImageFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.image = (Image) params.get("raster");
        height = image.getHeight(null);
        width = image.getWidth(null);
        super.pre();
    }

    /**
     * Asigna al pixel pasado como parmetro como transparente con 
     * el color de transparencia que est seleccionado
     * @param px Pixel a asignarle transparencia
     */
    private void setPixelTransparent(int[] px){
		px[0] = this.alpha;
        px[1] = this.transparencyColorRed;
        px[2] = this.transparencyColorGreen;
        px[3] = this.transparencyColorBlue;	
    }
    
    /**     
     * @param range rango
     * @param banda banda
     * @param px    pixel
     */
    private void processRange(int[] px) {
    	for (int i = 0; i < rangesList.size(); i++) {
    		TransparencyRange tr = ((TransparencyRange)rangesList.get(i));

    		if(tr.isAnd()){
    			if(tr.getRed() != null)
    				if (px[1] < tr.getRed()[0] || px[1] > tr.getRed()[1])
    					continue;
    			if(tr.getGreen() != null)
    				if (px[2] < tr.getGreen()[0] || px[2] > tr.getGreen()[1])
    					continue;
    			if(tr.getBlue() != null)
    				if (px[3] < tr.getBlue()[0] || px[3] > tr.getBlue()[1])
    					continue;
    			if(tr.getRed() != null || tr.getGreen() != null || tr.getBlue() != null)
    				setPixelTransparent(px);
    			
    		}else{
    			if(tr.getRed() != null){
    				if (px[1] >= tr.getRed()[0] && px[1] <= tr.getRed()[1]){
    					setPixelTransparent(px);
    					continue;
    				}
    			}
    			if(tr.getGreen() != null){
    				if (px[2] >= tr.getGreen()[0] && px[2] <= tr.getGreen()[1]){
    					setPixelTransparent(px);
    					continue;
    				}
    			}
    			if(tr.getBlue() != null){
    				if (px[3] >= tr.getBlue()[0] && px[3] <= tr.getBlue()[1]){
    					setPixelTransparent(px);
    					continue;
    				}
    			}
    		}
    	}
    	
    }
    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int col, int line) {
        int pt = ((BufferedImage) image).getRGB(col, line);
        int[] px4 = {
                        (pt & 0xff000000) >> 24, (pt & 0xff0000) >> 16,
                        (pt & 0x00ff00) >> 8, (pt & 0x0000ff)
                    };

        processRange(px4);
       
        ((BufferedImage) image).setRGB(col, line,
                                       (((px4[0] << 24) & 0xff000000) |
                                       ((px4[1] << 16) & 0x00ff0000) |
                                       ((px4[2] << 8) & 0x0000ff00) |
                                       (px4[3] & 0x0000ff)));
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#processSuperSampling(int, int)
     */
    public void processSuperSampling(int col, int line) {
    	int pt = ((BufferedImage) image).getRGB(col, line);
        int[] px4 = {
                        (pt & 0xff000000) >> 24, (pt & 0xff0000) >> 16,
                        (pt & 0x00ff00) >> 8, (pt & 0x0000ff)
                    };

        processRange(px4);
    
        for(int j = col; j < image.getWidth(null) && j < (col + stepX[contX + 1]); j++)
			for(int i = line; i < image.getHeight(null) && i < (line + stepY[contY + 1]); i++)
 			    ((BufferedImage) image).setRGB(j, i, 	(((px4[0] << 24) & 0xff000000) |
 			    										((px4[1] << 16) & 0x00ff0000) |
 			    										((px4[2] << 8) & 0x0000ff00) |
 			    										(px4[3] & 0x0000ff)));
	}
    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.image;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }
}
