/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.io.data;



public interface IQueryableRaster {	  
	public final static int INTERPOLATION_Undefined			= 0;
	public final static int INTERPOLATION_NearestNeighbour	= 1;
	public final static int INTERPOLATION_Bilinear			= 2;
	public final static int INTERPOLATION_InverseDistance	= 3;
	public final static int INTERPOLATION_BicubicSpline		= 4;
	public final static int INTERPOLATION_BSpline			= 5;

	/**
	 * Obtiene el ancho de la imagen
	 * @return  Ancho en pixeles de la imagen
	 */
	public int getWidth();
	
	/**
	 * Obtiene el alto de la imagen
	 * @return  Alto en pixeles de la imagen
	 */
	public int getHeight();
	
	/**
	 * Tamao de celda en X
	 * @return 
	 */
	public double getXCellSize();
	
	/**
	 * Tamao de celda en Y
	 * @return
	 */
	public double getYCellSize();
	
	/**
	 * Asigna un area de interes desde la posicin x,y en pixeles de ancho w y alto h
	 * @param x Coordenada x de la esquina superior izquierda
	 * @param y Coordenada y de la esquina superior izquierda
	 * @param w Ancho del grid
	 * @param h Alto del grid
	 */
	public void setAreaOfInterest(int x, int y, int w, int h);
	
	/**
	 * Obtiene un buffer desde la posicin x,y en pixeles de ancho w y alto h
	 * @param x Coordenada x de la esquina superior izquierda
	 * @param y Coordenada y de la esquina superior izquierda
	 * @param w Ancho del grid
	 * @param h Alto del grid
	 * @return Buffer de datos
	 */
	public RasterBuf getData(int x, int y, int w, int h);
	
	/**
	 * Devuelve el tipo de datos del raster
	 * @see java.awt.image.DataBuffer
	 */
	public int getDataType();

	/**
	 * Asigna tipo de datos del raster
	 * @see java.awt.image.DataBuffer
	 */
	public void setDataType(int dt);

	/**
	 * Devuelve true si la celda contiene un valor de NODATA
	 * @param x	coordenada X
	 * @param y coordenada Y
	 * @return
	 */
	public boolean isNoData(int x, int y, int band);
	
	/**
	 * Obtiene el nmero de bandas
	 * @return Nmero de bandas
	 */
	public int getBandCount();
	
}
