/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.cmd;

import org.cresques.ui.CQCursor;
import org.cresques.ui.CQMapCanvas;

import java.awt.geom.Point2D;


/**
 * Comando pan.
 * Aade al canvas la capacidad de desplazar la vista.
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class CmdPan extends Cmd {
    private Point2D ptIni = null;

    /**
     * Construye un nuevo CmdPan para el Canvas.
     * @param canvas
     */
    public CmdPan(CQMapCanvas canvas) {
        super(canvas);
        eventsWanted = LEFT | PRESS | RELEASE;
        cursor = CQCursor.getCursor(CQCursor.PAN_CURSOR);
    }

    /**
     * Recibe los eventos del ratn.
     */
    public void cmd(Point2D pt, int btn, int mouseEvent) {
        if (mouseEvent == Cmd.RELEASE) {
            pan(ptIni, pt);
        } else if (mouseEvent == Cmd.PRESS) {
            ptIni = pt;
        }
    }

    /**
     * Realiza un desplazamiento de la vista.
     * @param ptIni Punto inicial.
     * @param ptFin Punto final.
     */
    void pan(Point2D ptIni, Point2D ptFin) {
        canvas.getVPData().pan(ptIni, ptFin);
        canvas.viewPortChanged();
        canvas.repaint();
    }
}
