/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * Panel que contiene la lista de ficheros cargados desde donde se leen las bandas visualizadas.
 * Contiene controles para aadir y eliminar los ficheros, as como un control para seleccionar
 * el nmero de bandas a visualizar.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class FileList extends JPanel {
    final private static long serialVersionUID = -3370601314380922368L;
    private JScrollPane jScrollPane = null;
    private JPanel jPanel = null;
    private JButton jButton2 = null;
    private JButton jButton3 = null;
    private Vector fileNames = null;
    private JList jList = null;
    private JPanel jPanel1 = null;
    private JComboBox jComboBox = null;
    private JPanel jPanel2 = null;
    
    /**
     * Variable accesible para la traduccin.
     */
    public JLabel lbandasVisibles = null;

    /**
    * This is the default constructor
    */
    public FileList() {
        super();
        initialize();
    }

    /**
     * This method initializes jScrollPane
     *
     * @return javax.swing.JScrollPane
     */
    private JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new JScrollPane();
            jScrollPane.setViewportView(getJList());
        }

        return jScrollPane;
    }

    /**
     * This method initializes jPanel
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel() {
        if (jPanel == null) {
            FlowLayout flowLayout2 = new FlowLayout();
            jPanel = new JPanel();
            jPanel.setLayout(flowLayout2);
            flowLayout2.setAlignment(java.awt.FlowLayout.RIGHT);
            jPanel.add(getJPanel1(), null);
        }

        return jPanel;
    }

    /**
     * This method initializes jButton2
     *
     * @return javax.swing.JButton
     */
    public JButton getJButtonAdd() {
        if (jButton2 == null) {
            jButton2 = new JButton("Aadir");
            jButton2.setPreferredSize(new java.awt.Dimension(80, 25));
        }

        return jButton2;
    }

    /**
     * This method initializes jButton3
     *
     * @return javax.swing.JButton
     */
    public JButton getJButtonRemove() {
        if (jButton3 == null) {
            jButton3 = new JButton("Eliminar");
            jButton3.setPreferredSize(new java.awt.Dimension(80, 25));
        }

        return jButton3;
    }

    /**
     * This method initializes jList
     *
     * @return javax.swing.JList
     */
    public JList getJList() {
        if (jList == null) {
            fileNames = new Vector();
            jList = new JList(fileNames);
        }

        return jList;
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    private void initialize() {
        this.setLayout(new BorderLayout());
        this.setSize(300, 248);
        this.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
        this.add(getJPanel(), java.awt.BorderLayout.EAST);
    }

    /**
     * This method initializes jPanel1
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel1() {
        if (jPanel1 == null) {
            jPanel1 = new JPanel();
            jPanel1.setPreferredSize(new java.awt.Dimension(86, 180));
            jPanel1.add(getJButtonAdd(), null);
            jPanel1.add(getJButtonRemove(), null);
            jPanel1.add(getJPanel2(), null);
        }

        return jPanel1;
    }

    /**
     * Aade el nombre de un fichero a la lista
     * @param fName
     */
    public void addFName(String fName) {
        fileNames.add(fName);
        jList = new JList(fileNames);
        jScrollPane.setViewportView(jList);
        jList.show();
    }

    /**
     * Elimina un fichero de la lista
     * @param fName
     */
    public void removeFName(String fName) {
        for (int i = 0; i < fileNames.size(); i++) {
            if (((String) fileNames.get(i)).endsWith(fName)) {
                fileNames.remove(i);
            }
        }

        jList = new JList(fileNames);
        jScrollPane.setViewportView(jList);
        jList.show();
    }

    /**
     * Obtiene el nmero de ficheros en la lista
     * @return
     */
    public int getNFiles() {
        return fileNames.size();
    }

    /**
     * Asigna al combo del nmero de bandas la lista de ellas
     */
    public void setList(String[] list) {
        if (jComboBox != null) {
            jComboBox.removeAllItems();

            for (int i = 0; i < list.length; i++)
                jComboBox.addItem(list[i]);

            jComboBox.setSelectedIndex(list.length - 1);
        }
    }

    /**
     * This method initializes jComboBox
     *
     * @return javax.swing.JComboBox
     */
    public JComboBox getJComboBox() {
        if (jComboBox == null) {
            String[] list = { "1", "2", "3" };
            jComboBox = new JComboBox(list);
            jComboBox.setSelectedIndex(2);
            jComboBox.setPreferredSize(new java.awt.Dimension(36, 25));
        }

        return jComboBox;
    }

    /**
     * This method initializes jPanel2
     *
     * @return javax.swing.JPanel
     */
    private JPanel getJPanel2() {
        if (jPanel2 == null) {
            lbandasVisibles = new JLabel();
            jPanel2 = new JPanel();
            lbandasVisibles.setText("Bandas");
            jPanel2.add(lbandasVisibles, null);
            jPanel2.add(getJComboBox(), null);
        }

        return jPanel2;
    }
    
    public void setResize (int nWidth, int nHeight){
    	this.jScrollPane.setSize(this.jScrollPane.getSize().width + 1, this.jScrollPane.getSize().height + 1);
    	this.jList.setSize(this.jList.getSize().width + 1, this.jList.getSize().height + 1);
    	this.setSize(428 + nWidth, 110 + nHeight/2);
    }
} //  @jve:decl-index=0:visual-constraint="10,10"
