/*
 * Created on 11-ago-2006
 *
 */
/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.ui.raster;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.cresques.ui.raster.listener.PalettePanelListener;
import org.gvsig.i18n.Messages;

/**
 * Ventana para la seleccin del nombre al salvar una paleta de color.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */

public class NameChooserDialog extends BaseComponent implements ActionListener{

	private int 					HBUTTONS = 30;
	private JPanel					buttons = null;
	private JPanel					pMain = null;
	private int 					width = 315, height = 115;
	private int 					widthMargin = 300;
	private JButton					bAccept = null;
	private JButton					bCancel = null;
	private JFrame 					window = new JFrame();
	private JTextField				tName = null;
	private PalettePanelListener	listener = null;
	private PalettePanel			panel = null;
	
	
	/**
	 * Constructor
	 * @param pp PalettePanel
	 */
	public NameChooserDialog(PalettePanel pp, PalettePanelListener listener){
		this.panel = pp;
		this.listener = listener;
		
		window.getContentPane().add(getPMain());
		init();
	}
	
	public void init(){
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		window.setLocation( (((int)d.getWidth()) >> 1) - (width >> 1), 
	    					(((int)d.getHeight()) >> 1) - height);
		window.setSize(width, height);
		window.setFocusable(true);
		window.setResizable(false);
		window.setTitle(Messages.getText("nombre_paleta"));
		window.show();
		if(panel.getCbList().getSelectedIndex() != 0)
			getTName().setText(panel.getCbList().getSelectedItem().toString());
		else
			getTName().setText("Default Name");
	}
	
	/**
	 * Obtiene el panel principal que contiene el panel de botones y cuadro
	 * de texto
	 * @return JPanel
	 */
	private JPanel getPMain(){
		if(pMain == null){
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridy = 0;
			gridBagConstraints.gridx = 0;
			gridBagConstraints.insets = new Insets (0,15,10,0);
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.gridx = 0;
			pMain = new JPanel();
			pMain.setLayout(new GridBagLayout());
			pMain.setPreferredSize(new java.awt.Dimension(widthMargin, height));
			pMain.add(getTName(), gridBagConstraints);
			pMain.add(getPButtons(), gridBagConstraints1);
		}
		return pMain;
	}
	
	/**
	 * Obtiene el panel de botones
	 * @return JPanel
	 */
	private JPanel getPButtons(){
		if(buttons == null){
			buttons = new JPanel();
			buttons.setPreferredSize(new java.awt.Dimension(widthMargin, HBUTTONS));
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(java.awt.FlowLayout.RIGHT);
			flowLayout.setVgap(0);
			flowLayout.setHgap(2);
			buttons.setLayout(flowLayout);
			buttons.add(getBAccept(), null);
			buttons.add(getBCancel(), null);
		}
		return buttons;
	}
	
	/**
	 * Obtiene el botn de aceptar
	 * @return JButton
	 */
	public JButton getBAccept(){
		if(bAccept == null){
			bAccept = new JButton("aceptar");
			bAccept.addActionListener(this);
		}
		return bAccept;
	}
	
	/**
	 * Obtiene el botn de cancelar
	 * @return JButton
	 */
	public JButton getBCancel(){
		if(bCancel == null){
			bCancel = new JButton("cancelar");
			bCancel.addActionListener(this);
		}
		return bCancel;
	}

	/**
	 * Al aceptar asignamos el valor cuadro de texto a la propiedad name y al cancelar cerramos la ventana
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == bAccept){
			if (!getTName().getText().equals("")){
				listener.savePalette(panel.getPalettesPath(),getTName().getText());
				panel.getCbList().addItem(getTName().getText());
				panel.getCbList().setSelectedItem(getTName().getText());
				
			}
			window.hide();
		}
		if(e.getSource() == bCancel){
			window.hide();
		}
	}

	public JFrame getWindow() {
		return window;
	}

	public JTextField getTName() {
		if (tName == null){
			tName = new JTextField();
			tName.setPreferredSize(new Dimension(230,20));
		}
		return tName;
	}
} 
