/*
 * Created on 23-ago-2006
 *
 */
/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.ui.raster;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Palette;
import org.cresques.ui.raster.listener.PalettePanelListener;
import org.cresques.util.Utilities;
import org.gvsig.gui.beans.table.TableContainer;
import org.gvsig.gui.beans.table.exceptions.NotInitializeException;
import org.gvsig.gui.beans.table.models.TableButtonModel;
import org.gvsig.i18n.Messages;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/**
 * Interfaz de usuario para la seleccin de paletas de color
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */

public class PalettePanel extends BaseComponent {

	private int 					HSELECT = 52;
	private int 					HOPTIONS = 25;
	private int 					HBUTTONS = 30;
	private int 					MARGIN = 12;
	
	private JPanel 					pMain = null;
	private JPanel 					pNorth = null;
	private JPanel 					pButtons = null;
	private TableContainer 			pTable = null;
	private JPanel 					pSelect = null;
	private JPanel 					pOptions = null;
	private JButton 				bAccept = null;
	private JButton 				bApply = null;
	private JButton 				bCancel = null;
	private JButton 				bSave = null;
	private JButton 				bSaveAs = null;
	
	private int 					width = 500;
	private int 					height = 600;
	private String[] 				columnNames = {Messages.getText("selec"), Messages.getText("clase"), "RGB", Messages.getText("valor"), Messages.getText("alpha")};
	private int[] 					columnWidths = {50, 90, 94, 95, 65};
	private JCheckBox 				cActive = null;
	private JComboBox 				cbList = null;
	private JCheckBox 				cbInterpolar = null;
	private PalettePanelListener	listener  = null;
	private String 					palettesPath = "";
	private JLabel 					lPredefinidas = null;

	
	/**
	 * This method initializes 
	 * 
	 */
	public PalettePanel(int w, int h, JFrame mainFrame) {
		super();
		this.width = w;
		this.height = h;
		listener = new PalettePanelListener(this);
		initialize();
	}
	
	/**
	 * This method initializes 
	 * 
	 */
	public PalettePanel(int w, int h, String palettesPath) {
		//super();
		this.width = w;
		this.height = h;
		this.palettesPath = palettesPath;
		listener = new PalettePanelListener(this);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.add(getPMain(), null);
		listener.setControlListeners();
		loadPaletteList(palettesPath);
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public JPanel getPMain() {
		if (pMain == null) {
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints4.gridy = 1;
			gridBagConstraints4.gridx = 0;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints3.gridy = 0;
			gridBagConstraints3.gridx = 0;
			pMain = new JPanel();
			pMain.setLayout(new GridBagLayout());
			pMain.setPreferredSize(new java.awt.Dimension(width, height));
			pMain.add(getPNorth(), gridBagConstraints3);
			pMain.add(getPButtons(), gridBagConstraints4);
		}
		return pMain;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public JPanel getPNorth() {
		if (pNorth == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints2.gridy = 2;
			gridBagConstraints2.gridx = 0;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints.gridy = 0;
			gridBagConstraints.gridx = 0;
			pNorth = new JPanel();
			pNorth.setLayout(new GridBagLayout());
			pNorth.setPreferredSize(new java.awt.Dimension(width, height - HBUTTONS));
			pNorth.add(getPSelect(), gridBagConstraints);
			pNorth.add(getPTable(), gridBagConstraints1);
			pNorth.add(getPOptions(), gridBagConstraints2);
		}
		return pNorth;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPButtons() {
		if (pButtons == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(java.awt.FlowLayout.RIGHT);
			flowLayout.setVgap(0);
			flowLayout.setHgap(3);
			pButtons = new JPanel();
			pButtons.setPreferredSize(new java.awt.Dimension(width, HBUTTONS));
			pButtons.setLayout(flowLayout);
			pButtons.add(getBSaveAs(), null);
			pButtons.add(getBSave(), null);
			pButtons.add(getBAccept(), null);
			pButtons.add(getBApply(), null);
			pButtons.add(getBCancel(), null);
		}
		return pButtons;
	}

	/**
	 * This method initializes jPanel3	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public TableContainer getPTable() {
		if (pTable == null) {
			pTable = new TableContainer(width -MARGIN, height - HSELECT - HOPTIONS - HBUTTONS - MARGIN, columnNames, columnWidths);
			pTable.setModel("TableButtonModel");
			pTable.initialize();
		}
		return pTable;
	}

	/**
	 * This method initializes jPanel4	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPSelect() {
		if (pSelect == null) {
			lPredefinidas = new JLabel();
			lPredefinidas.setText(Messages.getText("predefinidas")+":");
			
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints6.gridx = 1;
			gridBagConstraints6.gridy = 1;
			gridBagConstraints6.weightx = 1.0;
			gridBagConstraints6.insets = new java.awt.Insets(0,0,0,8);
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.insets = new java.awt.Insets(0,5,0,0);
			gridBagConstraints5.gridy = 0;
			gridBagConstraints5.gridx = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints4.gridx = 1;
			gridBagConstraints4.gridy = 0;
			gridBagConstraints4.insets = new java.awt.Insets(0,0,0,0);
			pSelect = new JPanel();
			pSelect.setLayout(new GridBagLayout());
			pSelect.setPreferredSize(new java.awt.Dimension(width, HSELECT));
			pSelect.add(getCActive(), gridBagConstraints5);
			pSelect.add(getCbList(), gridBagConstraints6);
			pSelect.add(lPredefinidas, gridBagConstraints4);
		}
		return pSelect;
	}

	/**
	 * This method initializes jPanel5	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPOptions() {
		if (pOptions == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(java.awt.FlowLayout.LEFT);
			flowLayout1.setVgap(0);
			flowLayout1.setHgap(3);
			pOptions = new JPanel();
			pOptions.setLayout(flowLayout1);
			pOptions.setPreferredSize(new java.awt.Dimension(width, HOPTIONS));
			pOptions.add(getCbInterpolar(), null);
		}
		return pOptions;
	}

	/**
	 * This method initializes bApply	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBAccept() {
		if (bAccept == null) {
			bAccept = new JButton(Messages.getText("aceptar"));
		}
		return bAccept;
	}

	/**
	 * This method initializes bApply	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBApply() {
		if (bApply == null) {
			bApply = new JButton(Messages.getText("aplicar"));
		}
		return bApply;
	}

	/**
	 * This method initializes bCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBCancel() {
		if (bCancel == null) {
			bCancel = new JButton(Messages.getText("cancelar"));
		}
		return bCancel;
	}
	
	/**
	 * This method initializes bSave	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBSave() {
		if (bSave == null) {
			bSave = new JButton(Messages.getText("salvar"));
		}
		return bSave;
	}

	/**
	 * This method initializes bSaveAs	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBSaveAs() {
		if (bSaveAs == null) {
			bSaveAs = new JButton(Messages.getText("salvar_como"));
		}
		return bSaveAs;
	}
	
	/**
	 * Asigna el tamao del componente
	 * @param w Nuevo ancho del componente padre
	 * @param h Nuevo alto del componente padre
	 */
	public void setComponentSize(int w, int h){
		this.width = w;
		this.height = h;
		this.setSize(w, h);
		this.setPreferredSize(new java.awt.Dimension(w, h));
		this.getPMain().setPreferredSize(new java.awt.Dimension(w, h));
		this.getPNorth().setPreferredSize(new java.awt.Dimension(w, h - HBUTTONS));
		this.getPSelect().setPreferredSize(new java.awt.Dimension(width, HSELECT));
		this.getPTable().setComponentSize(width -MARGIN, height - HSELECT - HOPTIONS - HBUTTONS - MARGIN);
		this.getPOptions().setPreferredSize(new java.awt.Dimension(width, HOPTIONS));
		this.getPButtons().setPreferredSize(new java.awt.Dimension(width, HBUTTONS));
	}

	/**
	 * This method initializes jCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	public JCheckBox getCActive() {
		if (cActive == null) {
			cActive = new JCheckBox();
			cActive.setText(Messages.getText("activar"));
			cActive.setPreferredSize(new java.awt.Dimension(180,23));
			cActive.addActionListener(listener);
			cActive.setSelected(true);
		}
		return cActive;
	}

	/**
	 * This method initializes jComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	public JComboBox getCbList() {
		if (cbList == null) {
			cbList = new JComboBox();
		}
		return cbList;
	}

	/**
	 * This method initializes jCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	public JCheckBox getCbInterpolar() {
		if (cbInterpolar == null) {
			cbInterpolar = new JCheckBox();
			cbInterpolar.setText(Messages.getText("interpolar"));
		}
		return cbInterpolar;
	}
	
	//----------------------------------------------------------------
	//MTODOS DEL CONTROL
	
	/**
	 * Convierte la tabla en un array de objetos para poder crear con el el objeto Palette
	 * @return 
	 * @throws NotInitializeException
	 */
	public Object[][] getTablePalette() throws NotInitializeException{
		int cols = 6;
		if(getPTable().getRowCount() <= 0)
			return null;
		Object[][] pal = new Object[getPTable().getRowCount()][cols];
		TableButtonModel model = (TableButtonModel)getPTable().getModel();
		for(int iRow = 0; iRow < getPTable().getRowCount(); iRow ++){
			pal[iRow][0] = model.getValueAt(iRow, 1);
			String[] rgb = ((String)model.getValueAt(iRow, 2)).split(",");
			pal[iRow][1] = Integer.valueOf(rgb[0]);
			pal[iRow][2] = Integer.valueOf(rgb[1]);
			pal[iRow][3] = Integer.valueOf(rgb[2]);
			pal[iRow][4] = Double.valueOf((String)model.getValueAt(iRow, 3));
			pal[iRow][5] = Integer.valueOf((String)model.getValueAt(iRow, 4));
		}
		return pal;
	}
	
	/**
	 * Carga la tabla de datos desde un objeto Palette. 
	 * @param pal
	 */
	public void loadPanelFromPalette(Palette pal){
		deleteAllRows();
		int[] paleta = pal.getPalette();
		String[] names = pal.getNameClass();
		
		//Obtenemos los datos de la clase paleta para construir cada fila de la tabla	
		for(int iEntry = 0; iEntry < paleta.length; iEntry ++){
			int[] argb = Utilities.getARGBFromIntToIntArray(paleta[iEntry]);
			Color color = new Color(argb[0], argb[1], argb[2]);
						
			if(	pal.getType() == RasterBuf.TYPE_BYTE | 
				pal.getType() == RasterBuf.TYPE_SHORT | 
				pal.getType() == RasterBuf.TYPE_INT){
				int[] range = pal.getIntRange();
				addRowToTable(color, names[iEntry], range[iEntry]+"", String.valueOf(argb[3]));
			}else if(	pal.getType() == RasterBuf.TYPE_FLOAT | 
						pal.getType() == RasterBuf.TYPE_DOUBLE ){
				double[] range = pal.getDoubleRange();
				addRowToTable(color, names[iEntry], range[iEntry]+"", String.valueOf(argb[3]));
			}	
		}
	}
	
	/**
	 * Aade una fila a la tabla asignando el color por parmetro. Este
	 * color asignado ser el que aparezca en el botn y en el texto RGB
	 * @param color 
	 */
	public void addRowToTable(Color color, String name, String range, String alpha){
		getBSave().setEnabled(true);
		getBSaveAs().setEnabled(true);
	
		try {
			getPTable().addRow(new Object[]{color, name, color.getRed()+","+color.getGreen()+","+color.getBlue(), range, alpha});
		} catch (NotInitializeException e1) {
			return;
		}
	
	}
	
	/**
	 * Asigna valores a una fila de la tabla
	 * @param row fila de la tabla a asignar valores
	 * @param color Color del botn y RGB
	 * @param name Nombre de la clase
	 * @param range Valor de la entrada
	 * @param alpha Valor de la transparencia
	 */
	public void setValues(int row, Color color, String name, String range, String alpha){
		TableButtonModel model = (TableButtonModel)getPTable().getModel();

		model.setValueAt(color, row, 0);
		model.setValueAt(name, row, 1);
		model.setValueAt(color.getRed()+","+color.getGreen()+","+color.getBlue(), row, 2);
		model.setValueAt(range, row, 3);
		model.setValueAt(alpha, row, 4);
	}
	
	/**
	 * Elimina la fila seleccionada de la tabla
	 */
	public void deleteSelectedRow(){
		int index = 0;
		try {
			index = getPTable().getSelectedIndex();
			getPTable().delRow(index);
			if(getPTable().getRowCount() == 0){
				getBSave().setEnabled(false);
				getBSaveAs().setEnabled(false);
			}
		} catch (NotInitializeException e1) {
			return;
		}
	}
	
	/**
	 * Elimina todas las filas de la tabla
	 */
	public void deleteAllRows(){
		try {
			getPTable().removeAllRows();
		} catch (NotInitializeException e1) {
			return;
		}
		getBSave().setEnabled(false);
		getBSaveAs().setEnabled(false);
	}
	
	/**
	 * Obtiene el indice de la fila seleccionada
	 * @return Indice de la fila
	 */
	public int getSelectedIndex(){
		 try {
			return getPTable().getSelectedIndex();
		} catch (NotInitializeException e) {
			return -1;
		}
	}
	
	/**
	 * Obtiene el valor del alpha de la fila seleccionada en la tabla
	 * @return Entero que representa el alpha 
	 */
	public int getAlpha(){
		TableButtonModel model = (TableButtonModel)getPTable().getModel();
		String value;
		try {
			value = ((JButton)model.getValueAt(getPTable().getSelectedIndex(), 4)).getText();
			return Integer.valueOf(value).intValue();
		} catch (NotInitializeException e) {
			return -1;
		} catch (NumberFormatException e) {
			return -1;
		}
	}
	
	/**
	 * Ordena las filas de la paleta de color con valores decrecientes de rango. En las primeras
	 * filas coloca los valores mayores y en la ltima los menores. Esta funcin es utilizada
	 * cada vez que se da aplicar o aceptar ya que es necesario cuando se pasa el filtro que los
	 * valores esten ordenados.
	 */
	public void orderRows(){
		class Row{
			public String name = null;
			public Color rgb = null;
			public String value = null;
			public String alpha = null;
		}
		
		try {
			Row row = null;
			Hashtable map = new Hashtable(); 
			TableButtonModel model = (TableButtonModel)getPTable().getModel();
			
			//Comprobamos que el tipo de datos sea igual para todas las filas.
			boolean intType = true;
			for(int iRow = 0; iRow < getPTable().getRowCount(); iRow ++){
				try{
					Integer.valueOf((String)model.getValueAt(iRow, 3));
				}catch(NumberFormatException exc){
					//Si alguno de los datos da un formatException es que no es entero por lo
					//que suponemos que es decimal. Ojo!, ya se ha validado que sea numerico
					intType = false;
				}
			}
				
			double[] doubleRange = null;
			int[] intRange = null;
			if(intType)
				intRange = new int[getPTable().getRowCount()];
			else
				doubleRange = new double[getPTable().getRowCount()];
			
			int lenght = getPTable().getRowCount();
			
			//Cargamos las filas en un treemap cuya clave es el rango
			for(int iRow = 0; iRow < lenght; iRow ++){
				row = new Row();
				row.name = (String)model.getValueAt(iRow, 1);
				row.rgb = (Color)model.getValueAt(iRow, 0);
				row.value = (String)model.getValueAt(iRow, 3);
				row.alpha = (String)model.getValueAt(iRow, 4);
				if(intType){		
					intRange[iRow] = Integer.valueOf(row.value).intValue();
					map.put(intRange[iRow]+" ", row);
				}else{
					doubleRange[iRow] = Double.valueOf(row.value).intValue();
					map.put(doubleRange[iRow]+" ", row);
				}
			}
						
			//Ordenamos la lista por el valor del rango
			if(intType)
				Arrays.sort(intRange);
			else
				Arrays.sort(doubleRange);
						
			int rowAssigned = lenght - 1;
			for(int iRow = 0; iRow < lenght; iRow ++){
				if(intType)
					row = (Row)map.get(intRange[iRow]+" ");
				else
					row = (Row)map.get(doubleRange[iRow]+" ");
				setValues(rowAssigned, row.rgb, row.name, row.value, row.alpha);
				rowAssigned --;
			}
		} catch (NotInitializeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * Carga en el listBox de paletas predefinidas los nombres que hay en la cabecera del fichero 
	 * de paletas predefinidas.
	 * 
	 * @param palettesPath Camino al ficher de paletas predefinidas.
	 * 
	 */
	public void loadPaletteList(String palettesPath){
		
		File palettesFile = new File(palettesPath);
		if(!palettesFile.exists())
			return;
		
		ArrayList nameList = new ArrayList();
		
		try {
			KXmlParser parser = new KXmlParser();
			FileInputStream fileInputStream = new FileInputStream(palettesPath);
			parser.setInput(fileInputStream, null);
			int tag = parser.nextTag();
			parser.require(KXmlParser.START_TAG, null, "palettes");
			tag = parser.nextTag();
			parser.require(KXmlParser.START_TAG, null, "palette_list");
					
			tag = parser.nextTag();
			while (tag == KXmlParser.START_TAG){
				parser.require(KXmlParser.START_TAG, null, "name");
				nameList.add(parser.nextText());
				parser.require(KXmlParser.END_TAG, null, "name");
				tag = parser.nextTag();
			}
			parser.require(KXmlParser.END_TAG, null, "palette_list");
			
			getCbList().removeAllItems();			   //
			for (int i = 0; i<nameList.size(); i++)    // Solo se aaden elementos al combo si se ha leido la lista 
				getCbList().addItem(nameList.get(i));  // completa de nombres de paleta sin errores.
			fileInputStream.close();
			return;
			
			
		}catch (FileNotFoundException fnfEx) {
			fnfEx.printStackTrace();
		}catch (XmlPullParserException xmlEx) {
			System.out.println("El fichero de paletas predeterminadas no tiene la estructura correcta:\n	" + xmlEx.getMessage());
		}catch (IOException e) {
		}
	}

	public String getPalettesPath() {
		return palettesPath;
	}

	/**
	 * Obtiene el listener
	 * @return PalettePanelListener
	 */
	public PalettePanelListener getListener() {
		return listener;
	}
	
	/**
	 * Activar o desactivar los componentes del panel
	 */
	public void setEnabled(boolean enabled){
		getCbList().setEnabled(enabled);
		getCbInterpolar().setEnabled(enabled);
		getPTable().setEnabled(enabled);
	}
	
	/**
	 * Funcin que informa de si est acivo o no la seleccin de paleta en el panel
	 * @return true si est activo y false si no lo est.
	 */
	public boolean isActivePanel(){
		return this.getCActive().isSelected();
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"

