/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import org.cresques.io.GeoRasterFile;

import org.cresques.ui.DefaultDialogPanel;

import javax.swing.JPanel;


/**
 * Dialogo para configurar las propiedades del raster.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class RasterPropsDialogPanel extends DefaultDialogPanel {
    final private static long serialVersionUID = -3370601314380922368L;

    /**
     * Constructor
     */
    public RasterPropsDialogPanel() {
        super();
        initialize();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    public void initialize() {
        this.setBounds(0, 0, 355, 230);
    }

    /**
     * Obtiene el panel BandSetupPanel con los controles
     * @return BandSetupPanel
     */
    protected BandSetupPanel getBandSetupPanel() {
        return (BandSetupPanel) getContentPanel();
    }

    /**
     * Obtiene el contentPane
     */
    protected JPanel getContentPanel() {
        if (contentPane == null) {
            contentPane = new BandSetupPanel();
            contentPane.setBounds(14, 12, 345, 174);
        }

        return contentPane;
    }

    /**
     * Aade una lista de GeoRasterFile al panel de bandas
     * @param files Array de GeoRasterFile
     */
    public void addFiles(GeoRasterFile[] files) {
        getBandSetupPanel().addFiles(files);
        getBandSetupPanel().repaint();
        this.repaint();
    }

    /**
     * Mantiene la asignacin entre R, G o B y la banda de la imagen que le corresponde 
     * @param nBand	Banda de la imagen que corresponde
     * @param flag	R, G o B se selecciona por medio de un flag que los identifica
     */
    public void assignBand(int nBand, int flag) {
        getBandSetupPanel().assignBand(nBand, flag);
    }

    /**
     * Obtiene la correspondencia entre el R, G o B y la banda asignada
     * @param flag	R, G o B se selecciona por medio de un flag que los identifica
     * @return	Banda de la imagen asignada al flag pasado por parmetro
     */
    public int getAssignedBand(int flag) {
        return getBandSetupPanel().getAssignedBand(flag);
    }
}
