/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*/
package org.cresques.ui.raster.listener;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JColorChooser;

import org.cresques.ui.raster.NameChooserDialog;
import org.cresques.ui.raster.PalettePanel;
import org.gvsig.gui.beans.table.TableControlerPanel;
import org.gvsig.gui.beans.table.exceptions.NotInitializeException;
import org.kxml2.io.KXmlParser;
import org.kxml2.io.KXmlSerializer;
import org.xmlpull.v1.XmlPullParserException;

/**
 * Listener para eventos del panel de paleta
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 */
public class PalettePanelListener implements KeyListener, ActionListener{

	private PalettePanel			panel = null;	
	public static boolean 			comboEventEnable = true;
	private TableControlerPanel		control = null;
	private JColorChooser			colorSelector = null;;
	private NameChooserDialog 		nameChooser;
	private String					savePaletteName;
		
	/**
	 * Constructor 
	 * @param p Panel 
	 */
	public PalettePanelListener(PalettePanel p){
		panel = p;
	}
	
	/**
	 * Asigna a los controles que el listener por defecto sea esta clase.
	 * El panel tendr que llamar a este mtodo explicitamente si quiere que  
	 * los listener sean asignados.
	 */
	public void setControlListeners(){
		control = panel.getPTable().getControl();
		
		panel.getBSave().addActionListener(this);
		panel.getBSaveAs().addActionListener(this);
		panel.getBAccept().addActionListener(this);
		panel.getBApply().addActionListener(this);
		panel.getBCancel().addActionListener(this);
		panel.getCActive().addActionListener(this);
		panel.getCbInterpolar().addActionListener(this);
		panel.getCbList().addActionListener(this);

		control.getBNew().addActionListener(this);
		control.getBDelPoint().addActionListener(this);
		control.getBClear().addActionListener(this);
		panel.getPTable().setEnableControlsListener(false);
		
		/*panel.getPTable().addKeyListener(this);
		panel.getBAccept().addKeyListener(this);
		panel.getBApply().addKeyListener(this);
		panel.getBCancel().addKeyListener(this);
		panel.getCActive().addKeyListener(this);
		panel.getCbInterpolar().addKeyListener(this);
		panel.getCbList().addKeyListener(this);*/
	}
	
	/**
	 * Controlamos las acciones de:
	 * <UL>
	 * <LI>Nueva fila: Asigna un nuevo botn con su listener para que pueda se controlado desde esta misma clase</LI>
	 * <LI>Lista de botones: Para cada botn de la tabla asignamos el listener que hace que se muestre el 
	 * dialogo de color. El valor retornado por este se pondra en la columna RGB y el botn se colorear</LI>
	 * </UL>
	 * @throws NotInitializeException 
	 */
	public void actionPerformed(ActionEvent e) {
		
		//Botn salvar como
		if(e.getSource() == panel.getBSaveAs()){
			if (nameChooser == null)
				nameChooser = new NameChooserDialog(panel, this);
			else
				nameChooser.init();
		}
		
		//Botn de salvar tabla
		if(e.getSource() == panel.getBSave()){
			if(panel.getCbList().getSelectedIndex() != 0){ //Ya existe. Hay que sustituir la que hay
				replacePalette(panel.getPalettesPath(), panel.getCbList().getSelectedItem().toString());
			}else{ //No hay nombre seleccionado -> se comporta como salvar como
				if (nameChooser == null)
					nameChooser = new NameChooserDialog(panel, this);
				else
					nameChooser.init();
			}
		}
		
		//Botn de nueva fila
		if(e.getSource() == control.getBNew())
			try {
				panel.addRowToTable(Color.WHITE, "C" + panel.getPTable().getRowCount(), "0", "255");
			} catch (NotInitializeException e1) {
				panel.addRowToTable(Color.WHITE, "C0" , "0", "255");
			}
		
		
		//Botn eliminar fila
		if(e.getSource() == control.getBDelPoint())
			panel.deleteSelectedRow();
				
		//Botn eliminar todas las filas
		if(e.getSource() == control.getBClear())
			panel.deleteAllRows();
		
		
		//Comobo elegir paleta predeterminada.
		if(e.getSource() == panel.getCbList()){
			if (panel.getCbList().getSelectedItem()!=null){
				loadPalette(panel.getPalettesPath(), panel.getCbList().getSelectedItem().toString());
				panel.orderRows();
			}
		}
				
		//Check de activo
		if(e.getSource() == panel.getCActive()){
			if(panel.getCActive().isSelected())
				panel.setEnabled(true);
			else
				panel.setEnabled(false);
		}
		
	}

	
	
	public void keyPressed(KeyEvent e) {
		/*if(e.getKeyCode() == 10){//Enter=Aceptar
						
		}
		
		if(e.getKeyCode() == 27){ //Escape=Cancelar
			
		}*/
	}

	public void keyReleased(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	
	/**
	 * Lee una paleta del fichero xml de paletas y la carga en la tabla del panel.
	 * 
	 *  @param palettesPath Camino al fichero de paletas predefinidas.
	 *  @param paletteName Nombre de paleta a cargar desde el fichero xml.
	 * 
	 */
	public void loadPalette(String palettesPath, String paletteName){
		
		File palettesFile = new File(palettesPath);
		if(!palettesFile.exists())
			return;
		
		try {
			KXmlParser parser = new KXmlParser();
			FileInputStream fileInputStream = new FileInputStream(palettesPath);
			parser.setInput(fileInputStream, null);
			int tag = parser.nextTag();
			
			parser.require(KXmlParser.START_TAG, null, "palettes");
			tag = parser.nextTag();
			parser.require(KXmlParser.START_TAG, null, "palette_list");
			parser.skipSubTree();
			parser.require(KXmlParser.END_TAG, null, "palette_list");
			tag = parser.nextTag();
			
			while (tag == KXmlParser.START_TAG){
				parser.require(KXmlParser.START_TAG, null, "palette");
				if(parser.getAttributeCount()==2) 
					if (parser.getAttributeValue(0).equals(paletteName)){	
						boolean interpolate = Boolean.valueOf(parser.getAttributeValue(1)).booleanValue();
						tag = parser.nextTag();
						parser.require(KXmlParser.START_TAG, null, "table");
						tag = parser.nextTag();
						try {
							panel.getPTable().removeAllRows();
						} catch (NotInitializeException e1) {
							e1.printStackTrace();
						}
						
						ArrayList rows = new ArrayList();
						
						while(tag == KXmlParser.START_TAG){
							parser.require(KXmlParser.START_TAG, null, "entry");
							if (parser.getAttributeCount()==3){
								
								String rgb = parser.getAttributeValue(1).substring(parser.getAttributeValue(1).indexOf(",")+1,
										parser.getAttributeValue(1).length());
								
								int a = Integer.valueOf(parser.getAttributeValue(1).substring(0,parser.getAttributeValue(1).indexOf(","))).intValue();
								int r= Integer.valueOf(rgb.substring(0,rgb.indexOf(","))).intValue();
								int g=Integer.valueOf(rgb.substring(rgb.indexOf(",")+1,rgb.lastIndexOf(","))).intValue();
								int b=Integer.valueOf(rgb.substring(rgb.lastIndexOf(",")+1,rgb.length())).intValue();
						
								//No aado filas a la tabla hasta que no se ha leido correctamente la paleta completa:
								Object row[] = {new Color(r,g,b), parser.getAttributeValue(0), parser.getAttributeValue(2), String.valueOf(a)};
								rows.add(row);
							}
							tag = parser.nextTag();
							parser.require(KXmlParser.END_TAG, null, "entry");
							tag = parser.nextTag();
						}
						parser.require(KXmlParser.END_TAG, null, "table");
						
						// Rellenar la tabla con las entradas leidas:
						for (int i=0;i<rows.size();i++)
							panel.addRowToTable((Color)((Object[])rows.get(i))[0], (String)((Object[])rows.get(i))[1], 
									(String)((Object[])rows.get(i))[2], (String)((Object[])rows.get(i))[3]);
						panel.getCbInterpolar().setSelected(interpolate);
						fileInputStream.close();
						return;
				}
				parser.skipSubTree();
				parser.require(KXmlParser.END_TAG, null, "palette");
				tag = parser.nextTag();
			}
					
			parser.require(KXmlParser.END_TAG, null, "palettes");
			panel.deleteAllRows();
			fileInputStream.close();
			return;
			
		}catch (FileNotFoundException fnfEx) {
			fnfEx.printStackTrace();
		}catch (XmlPullParserException xmlEx) {
			System.out.println("El fichero de paletas predeterminadas no tiene la estructura correcta:\n	" + xmlEx.getMessage());
		}catch (IOException e) {
		}
	}

	/**
	 * Salva la paleta que hay en la tabla al fichero xml de paletas conel nombre especificado.
	 * 
	 *  @param palettesPath Camino al fichero de paletas predefinidas.
	 *  @param paletteName Nombre de paleta a cargar desde el fichero xml.
	 * 
	 */
	public void savePalette(String palettesPath, String paletteName){
		
		File tmp = new File(palettesPath.substring(0,palettesPath.lastIndexOf(File.separator)+1)+"palettes.tmp");
		File palettes = new File(palettesPath);
		
		try {
			if (!tmp.exists())
				tmp.createNewFile();
			
			Reader reader = new FileReader(palettesPath);
			BufferedReader bReader = new BufferedReader(reader);
			
			Writer writer = new FileWriter(palettesPath.substring(0,palettesPath.lastIndexOf(File.separator)+1)+"palettes.tmp");
			BufferedWriter bWriter = new BufferedWriter(writer);
			
			String line = bReader.readLine();
			while (line != null){
				
				if (line.indexOf("</palette_list>")>=0){
					bWriter.write("\t\t<name>"+paletteName+"</name>");
					bWriter.newLine();
				}else if (line.indexOf("</palettes>")>=0){
					bWriter.newLine();
					bWriter.write(createXmlStringFromTable(paletteName));
					bWriter.newLine();
				}
					
				bWriter.write(line);
				bWriter.newLine();
				
				line = bReader.readLine();
			}
			
			bReader.close();
			bWriter.close();
		
			palettes.delete();
			tmp.renameTo(palettes);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Sustituye la paleta que hay en la tabla al fichero xml de paletas conel nombre especificado.
	 * 
	 *  @param palettesPath Camino al fichero de paletas predefinidas.
	 *  @param paletteName Nombre de paleta a sustituir
	 * 
	 */
	public void replacePalette(String palettesPath, String paletteName){
		
		File tmp = new File(palettesPath.substring(0,palettesPath.lastIndexOf(File.separator)+1)+"palettes.tmp");
		File palettes = new File(palettesPath);
		
		try {
			if (!tmp.exists())
				tmp.createNewFile();
			
			Reader reader = new FileReader(palettesPath);
			BufferedReader bReader = new BufferedReader(reader);
			
			Writer writer = new FileWriter(palettesPath.substring(0,palettesPath.lastIndexOf(File.separator)+1)+"palettes.tmp");
			BufferedWriter bWriter = new BufferedWriter(writer);
			
			String line = bReader.readLine();
			boolean write = true;
			while (line != null){
				
				if (line.indexOf("<palette name=\""+paletteName+"\"") >= 0)
					write = false;
									
				if(write){
					bWriter.write(line);
					bWriter.newLine();
				}else{
					bWriter.write(createXmlStringFromTable(paletteName));
					bWriter.newLine();
					while ((line.indexOf("</palette>") < 0) && (line != null))
						line = bReader.readLine();
					write = true;
				}
				
				line = bReader.readLine();
			}
			
			bReader.close();
			bWriter.close();
		
			palettes.delete();
			tmp.renameTo(palettes);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Crea un String con el bloque XML correspondiente a la paleta que hay en la tabla.
	 * 
	 *  @param palettesPath Camino al fichero de paletas predefinidas.
	 *  @param paletteName Nombre de paleta a cargar desde el fichero xml.
	 * 
	 */
	private String createXmlStringFromTable(String paletteName){
		Writer writer = new  StringWriter();
		try {
			KXmlSerializer serializer = new KXmlSerializer();
			serializer.setOutput(writer);
			
			serializer.text("\t");
			serializer.startTag(null,"palette").attribute(null,"name",paletteName).attribute(null,"interpolate",
					Boolean.toString(panel.getCbInterpolar().isSelected()));
			serializer.text("\n");
			serializer.text("\t\t");
			serializer.startTag(null,"table");
			serializer.text("\n");
			
			
			for (int row = 0; row<panel.getPTable().getRowCount();row++){
				
				String name = (String)panel.getPTable().getModel().getValueAt(row,1);
				String value = (String)panel.getPTable().getModel().getValueAt(row,3);
				String rgb = (String)panel.getPTable().getModel().getValueAt(row,2);
				String argb = (String)panel.getPTable().getModel().getValueAt(row,4) + "," + rgb;
				
				serializer.text("\t\t\t");
				serializer.startTag(null,"entry").attribute(null,"class",name);
				serializer.attribute(null,"argb",argb);
				serializer.attribute(null,"value",value).endTag(null,"entry");
				serializer.text("\n");
			}
				
			serializer.text("\t\t");
			serializer.endTag(null,"table");
			serializer.text("\n");
			serializer.text("\t");
			serializer.endTag(null,"palette");
			
		}catch (FileNotFoundException fnfEx) {
			fnfEx.printStackTrace();
		}catch (IOException e) {
		}catch (NotInitializeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		return writer.toString();
	}

	public String getSavePaletteName() {
		return savePaletteName;
	}

	public void setSavePaletteName(String savePaletteName) {
		this.savePaletteName = savePaletteName;
	}
}
