/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.print.attribute.PrintRequestAttributeSet;
import javax.swing.ImageIcon;

import org.apache.log4j.Logger;
import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.IVectorialDatabaseDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ComposedLayer;
import com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint;
import com.iver.cit.gvsig.fmap.layers.layerOperations.LayerCollection;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.XMLItem;
import com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;
import com.iver.utiles.IPersistance;
import com.iver.utiles.XMLEntity;
import com.iver.utiles.extensionPoints.ExtensionPoint;
import com.iver.utiles.extensionPoints.ExtensionPointsSingleton;
import com.iver.utiles.swing.threads.Cancellable;

/**
 * Representa un nodo interno en el rbol de capas
 */
public class FLayers extends FLyrDefault implements VectorialData, LayerCollection, InfoByPoint
{
	//ArrayList de Listener registrados.
	protected ArrayList layerCollectionListeners = new ArrayList();


	protected List layers = Collections.synchronizedList(new ArrayList());
	protected MapContext fmap;
	protected static Logger logger = Logger.getLogger(FLayers.class.getName());
	/**
	 * Crea un nuevo FLayers.
	 *
	 * @param fmap
	 * @param parent
	 */
	public FLayers(MapContext fmap, FLayers parent) {
		setParentLayer(parent);
		this.fmap = fmap;
	}

	/**
	 * Aade un listener de los eventos de la coleccin de capas
	 *
	 * @param listener
	 */
	public void addLayerCollectionListener(LayerCollectionListener listener) {
		if (!layerCollectionListeners.contains(listener))
			layerCollectionListeners.add(listener);
	}


	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.LayerCollection#setAllVisibles(boolean)
	 */
	public void setAllVisibles(boolean visible) {
		FLayer lyr;

		for (int i = 0; i < layers.size(); i++) {
			lyr = ((FLayer) layers.get(i));
			lyr.setVisible(visible);

			if (lyr instanceof LayerCollection) {
				((LayerCollection) lyr).setAllVisibles(visible);
			}
		}
	}

	/**
	 * Elimina un listener de los eventos de la coleccin de capas
	 *
	 * @param listener
	 */
	public void removeLayerCollectionListener(LayerCollectionListener listener) {
		layerCollectionListeners.remove(listener);
	}

	/**
	 * Aade una capa.
	 *
	 * @param layer
	 */
//	private void doAddLayer(FLayer layer) {
//		layers.add(layer);
//		layer.setParentLayer(this);
//	}
	private void doAddLayer(int pos,FLayer layer) {
		layers.add(pos,layer);
		layer.setParentLayer(this);
	}
	public void addLayer(FLayer layer) {
		addLayer(layers.size(),layer);
	}
	/**
	 * Aade una capa en la posicin pos de la lista de subcapas
	 *
	 * @param layer
	 *
	 * @throws CancelationException
	 */
	public void addLayer(int pos,FLayer layer) throws CancelationException {
		//Notificamos a la capa que va a ser aadida
		//FLyrDefault layerDef = (FLyrDefault)layer;
		//if (!layerDef.isUnavailable()) {
			if (layer instanceof FLyrDefault)
				((FLyrDefault)layer).wakeUp();

			if (layer instanceof FLayers){
				FLayers layers=(FLayers)layer;
				fmap.addAsCollectionListener(layers);
			}
			callLayerAdding(LayerCollectionEvent.createLayerAddingEvent(layer));
		//}

		doAddLayer(pos,layer);

		//if (!layerDef.isUnavailable()) {
			callLayerAdded(LayerCollectionEvent.createLayerAddedEvent(layer));
		//}
	}

	/**
	 * Mueve una capa de la posicin from a la posicin to. (Se supone que
	 * estn dentro del mismo nivel). Para mover entre niveles, usar otro
	 * mtodo (por hacer)
	 *
	 * @param from origen.
	 * @param to destino.
	 *
	 * @throws CancelationException
	 */
	public void moveTo(int from, int to) throws CancelationException {
		int newfrom=layers.size()-from-1;
		int newto=layers.size()-to-1;
		if ( newfrom < 0 || newfrom >=layers.size() || newto < 0 || newto >= layers.size()) return;
		FLayer aux = (FLayer) layers.get(newfrom);
		callLayerMoving(LayerPositionEvent.createLayerMovingEvent(aux, newfrom, newto));
		layers.remove(newfrom);
		layers.add(newto, aux);
		callLayerMoved(LayerPositionEvent.createLayerMovedEvent(aux, newfrom, newto));
	}

	/**
	 * Borra una capa del ArrayList que contiene a las capas.
	 *
	 * @param lyr FLayer.
	 */
	private void doRemoveLayer(FLayer lyr) {
			layers.remove(lyr);
	}

	/**
	 * Borra una capa de la coleccin de capas.
	 *
	 * @param lyr FLayer
	 *
	 * @throws CancelationException
	 */
	public void removeLayer(FLayer lyr) throws CancelationException {
		callLayerRemoving(LayerCollectionEvent.createLayerRemovingEvent(lyr));
		doRemoveLayer(lyr);
		callLayerRemoved(LayerCollectionEvent.createLayerRemovedEvent(lyr));
	}

	/**
	 * Borra una FLayer del vector a partir del identificador.
	 *
	 * @param idLayer
	 */
	public void removeLayer(int idLayer) {
		FLayer lyr = (FLayer) layers.get(idLayer);
		callLayerRemoving(LayerCollectionEvent.createLayerRemovingEvent(lyr));
		layers.remove(idLayer);
		callLayerRemoved(LayerCollectionEvent.createLayerRemovedEvent(lyr));
	}

	/**
	 * Mtodo de conveniencia para borrar una capa con ese nombre
	 *
	 * @param layerName
	 */
	public void removeLayer(String layerName) {
		FLayer lyr;

		for (int i = 0; i < layers.size(); i++) {
			lyr = ((FLayer) layers.get(i));

			if (lyr.getName().compareToIgnoreCase(layerName) == 0) {
				removeLayer(i);

				break;
			}
		}
	}
	/**
	 * Mtodo de conveniencia para reemplazar una capa con ese nombre
	 *
	 * @param layerName
	 */
	public void replaceLayer(String layerName, FLayer layer)
	{
		FLayer lyr;
		FLayer parent;
		for (int i = 0; i < layers.size(); i++) {
			lyr = ((FLayer) layers.get(i));

			if (lyr.getName().compareToIgnoreCase(layerName) == 0) {
				parent = lyr.getParentLayer();
				removeLayer(i);
				if (parent != null)
				//Notificamos a la capa que va a ser aadida
				if (layer instanceof FLyrDefault)
					((FLyrDefault)layer).wakeUp();

				if (layer instanceof FLayers){
					FLayers layers=(FLayers)layer;
					fmap.addAsCollectionListener(layers);
				}
				callLayerAdding(LayerCollectionEvent.createLayerAddingEvent(layer));

				layers.add(i,layer);
				layer.setParentLayer(this);

				callLayerAdded(LayerCollectionEvent.createLayerAddedEvent(layer));
				break;
			}
		}
	}


	/**
	 * Obtiene el array de capas visibles que penden del arbol cuya raz es
	 * este nodo
	 *
	 * @return Vector de FLayer.
	 */
	public FLayer[] getVisibles() {
		ArrayList array = new ArrayList();
		LayersIterator iter = new LayersIterator(this) {
			public boolean evaluate(FLayer layer) {
				return layer.isVisible();
			}

		};

		while (iter.hasNext()) {
			array.add(iter.nextLayer());
		}

		return (FLayer[]) array.toArray(new FLayer[0]);
	}

	/**
	 * Obtiene el hijo i-simo (de abajo a arriba) directo de esta coleccin.
	 *
	 * @param index ndice.
	 *
	 * @return capa seleccionada.
	 */
	public FLayer getLayer(int index) {
		return (FLayer) layers.get(index);
	}

	/**
	 * Mtodo de conveniencia para buscar una capa por su nombre Es conveniente
	 * usar getLayer(int) siempre que se pueda, es ms directo.
	 *
	 * @param layerName nombre.
	 *
	 * @return capa seleccionada.
	 */
	public FLayer getLayer(String layerName) {
		FLayer lyr;
		FLayer lyr2;
		ArrayList layerList;

		for (int i = 0; i < layers.size(); i++) {
			lyr = ((FLayer) layers.get(i));

			if (lyr.getName().compareToIgnoreCase(layerName) == 0) {
					return lyr;
			}

			layerList = new ArrayList();
			getLayerFromGroup(lyr,layerList);

			for(int j = 0; j<layerList.size(); j++ )
			{
				lyr2 = ((FLayer)layerList.get(j));
				if (lyr2.getName().compareToIgnoreCase(layerName) == 0) {
					return lyr2;
				}
			}

		}

		return null;
	}
    // looks in the layertree including FLayers
	private void getLayerFromGroup(FLayer layer, ArrayList result){
		int i;
		FLayers layerGroup;
		if (layer instanceof FLayers)
		{
			result.add(layer);
			layerGroup = (FLayers)layer;
			for (i=0; i < layerGroup.getLayersCount(); i++ )
			{
				getLayerFromGroup(layerGroup.getLayer(i),result);
			}
		}
		else
		{
			result.add(layer);
		}
	}


	// this is to to split up a layer group in order to get a layer by name
	// when there are layergroups.
	private void splitLayerGroup(FLayer layer, ArrayList result)
	{
		int i;
		FLayers layerGroup;
		if (layer instanceof FLayers)
		{
			layerGroup = (FLayers)layer;
			for (i=0; i < layerGroup.getLayersCount(); i++ )
			{
				splitLayerGroup(layerGroup.getLayer(i),result);
			}
		}
		else
		{
			result.add(layer);
		}
	}

	/**
	 * Obtiene el nmero de capas que hay en su mismo nivel Es decir, no cuenta
	 * las subcapas de un FLayers
	 *
	 * @return nmero de capas.
	 */
	public int getLayersCount() {
		return layers.size();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#draw(java.awt.image.BufferedImage,
	 * 		java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort)
	 */
	public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
			Cancellable cancel,double scale) throws DriverException {
		draw(image,g,viewPort,cancel, scale, null);
	}


	public ComposedLayer draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
			Cancellable cancel,double scale, ComposedLayer group) throws DriverException {
	    //double scale = getFMap().getScaleView();
//		Iterator iter = layers.iterator();
//        try
//        {
        	boolean bNeedRecalculateCache = false;

   			///// CHEMA ComposedLayer
		    long tg1 = System.currentTimeMillis();
   			///// CHEMA ComposedLayer



//    		while (iter.hasNext())
//    		{
    		 for (int i=0; i < layers.size(); i++) {
                if (cancel.isCanceled())
                    break; // Ms que nada porque las capas raster no son interrumpibles por ahora.
//    			FLayer lyr = (FLayer) iter.next();
    			FLayer lyr = (FLayer) layers.get(i);



    		/*	if (lyr instanceof FLyrVect && ((FLyrVect)lyr).isBroken()){
    				continue;
    			}
    		*/
    			LayerDrawEvent beforeEvent = new LayerDrawEvent(lyr, g, viewPort, LayerDrawEvent.LAYER_BEFORE_DRAW);
    			fmap.fireLayerDrawingEvent(beforeEvent);
        		if ((lyr.isDirty()) && (lyr.isCachingDrawnLayers() == false))
        			bNeedRecalculateCache = true;

    			if (lyr.isVisible()) {

    			        long t1 = System.currentTimeMillis();
    			        System.err.println("Layer " + lyr.getName() + " dirty=" + lyr.isDirty()+ " isCachingDrawnLayers= "+lyr.isCachingDrawnLayers());
    			        // synchronized (this) {
    			        	//Sincronizacin del mtodo dibujar de cada Layer, esto es posible hacerlo de otra forma,
    			        	//pero de momento se queda as, para solucionar el refresco de una misma vista que est aadida en el Layout.
    			        try{
    			        	/* if (lyr instanceof IComposedDrawing)
    			        	{
    			        		// Acumulamos las peticiones hasta que sea
    			        		// otro servidor o otro tipo de capa o la ltima
    			        		// Si hay que dibujar:
    			        		lyr.draw(image, g, viewPort, cancel,scale);
    			        		// Si hay que acumlar:
    			        		lyr.acumulateDrawing(antLayer);

    			        	}
    			        	else */

     			        	if (lyr.isCachingDrawnLayers())
    			        	{
    			        		if ((bNeedRecalculateCache) || (lyr.getCacheImageDrawnLayers()==null))
    			        		{
    			        		//if (bNeedRecalculateCache)
    			        		// {

       			        			///// CHEMA ComposedLayer
    			        			if (group != null) {
    			        				//si tenemos un grupo pendiente de pintar, pintamos
    			        				// para que la cache sea fiable
    			        				group.draw(image, g, viewPort, cancel,scale);
    			        				group = null;
    			        			}
       			        			///// CHEMA ComposedLayer


    			        			// Copiamos la imagen actual
    			        			BufferedImage buff = new BufferedImage(image.getWidth(), image.getHeight(), image.getType());
    			        			WritableRaster w = buff.getRaster();
    			        			image.copyData(w);
    			        			lyr.setCacheImageDrawnLayers(buff);
    			        			System.err.println("RECALCULO LA CACHE CON LO QUE HABIA ANTES DE " + lyr.getName());
    			        		}
//    			        		}
//    			        		else
//    			        		{
	    			        		if (lyr.getCacheImageDrawnLayers() != null)
	    			        		{
	    			        			// Previo a esto hemos tenido que fijar
	    			        			// la imagen que queremos usar, en el
	    			        			// prepareDrawing de FMap.
	    			        			if (lyr.isDirty())
	    			        			{


	    	   			        			///// CHEMA ComposedLayer
	    			        				// Checks for draw group (ComposedLayer)
	        			        			if (group != null) {
	        			        				// it's going to load a cache image,
	        			        				// the current draw isn't needed
	        			        				group = null;
	        			        			}
	           			        			///// CHEMA ComposedLayer


	    			        				g.drawImage(lyr.getCacheImageDrawnLayers(), 0, 0, null);
	    			        				System.err.println("Pinto con acelerado lo que haba antes de " + lyr.getName());
	    			        			}
	    			        		}
//    			        		}
    			        	}
    			        	// Si la capa est "sucia" o alguna de las de abajo est sucia
    			        	// hay que volver a dibujar.
   			        		if (lyr.isDirty() || bNeedRecalculateCache)
   			        		{
   			        			if (!lyr.getFLayerStatus().isDriverLoaded())
   			        			{
   			        				continue;
   			        			}

   			        			///// CHEMA ComposedLayer
   			        			//lyr.draw(image, g, viewPort, cancel,scale);
   			        			//bNeedRecalculateCache = true;
   			        			//System.err.println("Pinto sin acelerado " + lyr.getName());
   			        			///// CHEMA ComposedLayer

   			        			///// CHEMA ComposedLayer
   			        			// Checks for draw group (ComposedLayer)
   			        			if (group != null) {
   			        				if (lyr instanceof FLayers){
   			        					group = ((FLayers)lyr).draw(image, g, viewPort, cancel,scale,group);
   			        					LayerDrawEvent afterEvent = new LayerDrawEvent(lyr, g, viewPort, LayerDrawEvent.LAYER_AFTER_DRAW);
   			        					fmap.fireLayerDrawingEvent(afterEvent);
   			        				} else {
   			        					// If layer can be added to the group, does it
   			        					if (group.canAdd(lyr)) {
   			        						group.add(lyr);
   			        					} else {
   			        						// draw the 'pending to draw' layer group
   			        						group.draw(image, g, viewPort, cancel,scale);

   			        						// gets a new group instance
   			        						group = lyr.newComposedLayer();
   			        						// if layer hasn't group, draws it inmediately
   			        						if (group == null) {
   			        							if (lyr instanceof FLayers){
   			        								group = ((FLayers)lyr).draw(image, g, viewPort, cancel,scale,group);
   			        							} else {
   			        								lyr.draw(image, g, viewPort, cancel,scale);
   			        							}
   			        							LayerDrawEvent afterEvent = new LayerDrawEvent(lyr, g, viewPort, LayerDrawEvent.LAYER_AFTER_DRAW);
   			        							fmap.fireLayerDrawingEvent(afterEvent);
   			        						} else {
   			        							// add the layer to the group
   			        							group.setMapContext(fmap);
   			        							group.add(lyr);

   			        						}
   			        					}
   			        				}
   			        			} else {
	   			        			// gets a new group instance
	   			        			group = lyr.newComposedLayer();
	   			        			// if layer hasn't group, draws it inmediately
	   			        			if (group == null) {
	   			        				if (lyr instanceof FLayers){
	   			        					group = ((FLayers)lyr).draw(image, g, viewPort, cancel,scale,group);
	   			        				} else {
	   			        					lyr.draw(image, g, viewPort, cancel,scale);
	   			        				}
	   			        				LayerDrawEvent afterEvent = new LayerDrawEvent(lyr, g, viewPort, LayerDrawEvent.LAYER_AFTER_DRAW);
	   			        				fmap.fireLayerDrawingEvent(afterEvent);
	   			        			} else {
	   			        				// adds the layer to the group
	   			        				group.setMapContext(fmap);
	   			        				group.add(lyr);
	   			        			}
   			        			}
   			        			///// CHEMA ComposedLayer
								bNeedRecalculateCache = true;
   			        		}
    			        } catch (DriverException e){
    			        	// fmap.callNewErrorEvent(new ErrorEvent(lyr.getName(), e));
    			        	fmap.addLayerError("La capa " + lyr.getName() + " es errnea.");
    			        	e.printStackTrace();
    			        	lyr.setAvailable(false);
    			        	// this.removeLayer(lyr);
    			        } catch (Exception e) {
    			        	fmap.addLayerError("La capa " + lyr.getName() + " es errnea.");
    			        	e.printStackTrace();
    			        	lyr.setAvailable(false);
    			        }
    					// }
    					long t2 = System.currentTimeMillis();
    					System.out.println("Layer " + lyr.getName() + " "
    					        + (t2-t1) + " milisecs.");
    					if (!cancel.isCanceled())
    						lyr.setDirty(false);
    			}

    		}
    		///// CHEMA ComposedLayer
    		if (group != null && this.getParentLayer() == null) {
 				//si tenemos un grupo pendiente de pintar, pintamos
 				group.draw(image, g, viewPort, cancel,scale);
 				group = null;

 			}
			///// CHEMA ComposedLayer
    		if (getVirtualLayers() != null) {
    			getVirtualLayers().draw(image, g, viewPort, cancel,scale);
    		}

    		///// CHEMA ComposedLayer
    		return group;
    		///// CHEMA ComposedLayer

//        }
//        catch (ConcurrentModificationException e)
//        {
//            System.err.println(e.getMessage());
//        }
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#print(java.awt.Graphics2D,
	 * 		com.iver.cit.gvsig.fmap.ViewPort,
	 * 		com.iver.utiles.swing.threads.Cancellable)
	 */
	public ComposedLayer print(Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale, PrintRequestAttributeSet properties, ComposedLayer group)
		throws DriverException {

        // TODO: A la hora de imprimir, isWithinScale falla, porque est
        // calculando la escala en pantalla, no para el layout.
        // Revisar esto.
		for (int i=0; i < layers.size(); i++) {
			FLayer lyr = (FLayer) layers.get(i);

			try{

				///// CHEMA ComposedLayer
				// Checks for draw group (ComposedLayer)
				if (group != null) {
					if (lyr instanceof FLayers){
						group = ((FLayers)lyr).print(g, viewPort, cancel,scale,properties,group);
					} else {
						// If layer can be added to the group, does it
						if (group.canAdd(lyr)){
							group.add(lyr);
						} else {
							// draw the 'pending to draw' layer group
							group.print(g,viewPort,cancel,scale,properties);

							// gets a new group instance
							group = lyr.newComposedLayer();
							// if layer hasn't group, draws it inmediately
							if (group == null) {
								if (lyr instanceof FLayers){
									group = ((FLayers)lyr).print(g, viewPort, cancel,scale,properties,group);
								} else {
									lyr.print(g, viewPort, cancel,scale,properties);
								}
							} else {
								// add the layer to the group
								group.setMapContext(fmap);
								group.add(lyr);

							}

						}
					}
				} else {
					// gets a new group instance
					group = lyr.newComposedLayer();
					// if layer hasn't group, draws it inmediately
					if (group == null) {
						if (lyr instanceof FLayers){
							group = ((FLayers)lyr).print(g, viewPort, cancel,scale,properties,group);
						} else {
							lyr.print(g, viewPort, cancel,scale,properties);
						}
					} else {
						// add the layer to the group
						group.setMapContext(fmap);
						group.add(lyr);

					}
				}
				///// CHEMA ComposedLayer

			} catch (Exception e){
				fmap.addLayerError("Error en la impresion de la capa " + lyr.getName() + ": " + e.getMessage());
				e.printStackTrace();
			}

		}

		///// CHEMA ComposedLayer
		if (group != null && this.getParentLayer() == null) {
				//si tenemos un grupo pendiente de pintar, pintamos
				group.print(g, viewPort, cancel,scale,properties);
				group = null;

		}
		///// CHEMA ComposedLayer

		if (getVirtualLayers() != null) {
			getVirtualLayers().print( g, viewPort, cancel, scale, properties);
		}

		///// CHEMA ComposedLayer
		return group;
		///// CHEMA ComposedLayer

	}

	public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale, PrintRequestAttributeSet properties)
	throws DriverException {
		this.print(g, viewPort, cancel, scale, properties, null);
	}
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws DriverException {
		Rectangle2D rAux = null;
		boolean first = true;


		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer capa = (FLayer) iter.next();
			try{
			if (first) {
				rAux = capa.getFullExtent();
				first=false;
			} else {
				rAux.add(capa.getFullExtent());
			}
			}catch (Exception e) {
				e.printStackTrace();//TODO hay que revisar para determinar el comportamiento que espera el usuario.
			}
		}

		return rAux;
	}

	/**
	 * Llama al mtodo layerAdding de los listeners dados de alta.
	 *
	 * @param event
	 *
	 * @throws CancelationException
	 */
	protected void callLayerAdding(LayerCollectionEvent event)
		throws CancelationException {
	    ArrayList aux = (ArrayList) layerCollectionListeners.clone();
		for (Iterator iter = aux.iterator(); iter.hasNext();) {
			((LayerCollectionListener) iter.next()).layerAdding(event);
		}
	}

	/**
	 * Llama al mtodo layerRemoving de los listeners dados de alta.
	 *
	 * @param event
	 *
	 * @throws CancelationException
	 */
	protected void callLayerRemoving(LayerCollectionEvent event)
		throws CancelationException {
	    ArrayList aux = (ArrayList) layerCollectionListeners.clone();
		for (Iterator iter = aux.iterator(); iter.hasNext();) {
			((LayerCollectionListener) iter.next()).layerRemoving(event);
		}
	}


	/**
	 * Llama al mtodo layerMoving de los listeners dados de alta.
	 *
	 * @param event
	 *
	 * @throws CancelationException
	 */
	protected void callLayerMoving(LayerPositionEvent event)
		throws CancelationException {
	    ArrayList aux = (ArrayList) layerCollectionListeners.clone();
		for (Iterator iter = aux.iterator(); iter.hasNext();) {
			((LayerCollectionListener) iter.next()).layerMoving(event);
		}
	}

	/**
	 * Llama al mtodo layerAdded de los listeners dados de alta.
	 *
	 * @param event
	 */
	protected void callLayerAdded(LayerCollectionEvent event) {
	    ArrayList aux = (ArrayList) layerCollectionListeners.clone();
		for (Iterator iter = aux.iterator(); iter.hasNext();) {
			((LayerCollectionListener) iter.next()).layerAdded(event);
		}
	}

	/**
	 * Llama al mtodo layerRemoved de los listeners dados de alta.
	 *
	 * @param event
	 */
	protected void callLayerRemoved(LayerCollectionEvent event) {
	    ArrayList aux = (ArrayList) layerCollectionListeners.clone();
		for (Iterator iter = aux.iterator(); iter.hasNext();) {
			((LayerCollectionListener) iter.next()).layerRemoved(event);
		}
	}

	/**
	 * Llama al mtodo layerMoved de los listeners dados de alta.
	 *
	 * @param event
	 */
	protected void callLayerMoved(LayerPositionEvent event) {
	    ArrayList aux = (ArrayList) layerCollectionListeners.clone();
		for (Iterator iter = aux.iterator(); iter.hasNext();) {
			((LayerCollectionListener) iter.next()).layerMoved(event);
		}
	}

	/**
	 * @throws XMLException
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getProperties()
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = super.getXMLEntity();

		ArrayList names = new ArrayList();


		FLayer layer;
		for (int i = 0; i < layers.size(); i++) {
			layer = (FLayer)layers.get(i);
			// XXX: Parche para no guardar capas raster de memoria en el proyecto
			if (layer instanceof FLyrRaster){
				if (((FLyrRaster)layer).getSource() instanceof RasterFileAdapter){
					if (((RasterFileAdapter)((FLyrRaster)layer).getSource()).getFile() == null){
						continue;
					}
				}
			}
            // XXX: Parche para no guardar capas raster de memoria en el proyecto
			try{
				names.add(layer.getName());
				xml.addChild(layer.getXMLEntity());
			}catch (XMLException e) {
				e.printStackTrace();
			}
		}

		String[] s = new String[names.size()];
		System.arraycopy(names.toArray(),0,s,0,names.size());
		xml.putProperty("numLayers", names.size());
		xml.putProperty("LayerNames", s);

		return xml;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 *
	 * @throws XMLException
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity03(XMLEntity xml) throws XMLException{
		super.setXMLEntity03(xml);
		int numLayers = xml.getIntProperty("numLayers");

		String[] s = xml.getStringArrayProperty("LayerNames");
		try {
			for (int i = 0; i < numLayers; i++) {
				FLayer layer = null;

				String className = xml.getChild(i).getStringProperty("className");

				if (className.equals(FLyrVect.class.getName())) {
					if (xml.getChild(i).contains("file")) {

							layer = LayerFactory.createLayer(s[i],
									(VectorialFileDriver)LayerFactory.getDM().getDriver(xml.getChild(i).getStringProperty("driverName")),
									new File(xml.getChild(i).getStringProperty("file")),
									this.getMapContext().getViewPort().getProjection());

					} else if (true) {
						//TODO falta por implementar
					} else if (true) {
						//TODO falta por implementar
					}

					layer.setXMLEntity03(xml.getChild(i));
					// Comprobar que la proyeccin es la misma que la de FMap
					// Si no lo es, es una capa que est reproyectada al vuelo
					IProjection proj = layer.getProjection();
					if (proj != null)
					    if (proj != fmap.getProjection())
					    {
							ICoordTrans ct = proj.getCT(fmap.getProjection());
							layer.setCoordTrans(ct);
							System.err.println("coordTrans = " +
								proj.getAbrev() + " " +
								fmap.getProjection().getAbrev());
					    }

//				} else if (className.equals(FLyrWMS.class.getName())) {
//					try {
//						layer = LayerFactory.createLayer(s[i],
//								StringUtilities.string2Rect(
//									xml.getChild(i).getStringProperty("fullExtent")),
//								new URL(xml.getChild(i).getStringProperty("host")),
//								xml.getChild(i).getStringProperty("format"),
//								xml.getChild(i).getStringProperty("layerQuery"),
//								xml.getChild(i).getStringProperty("infoLayerQuery"),
//								xml.getChild(i).getStringProperty("srs"));
//					} catch (MalformedURLException e) {
//						throw new XMLException(e);
//					}
				} else if (className.equals((FLyrRaster.class.getName()))) {
					//if (xml.getChild(i).contains("file")) {
					/*layer = LayerFactory.createLayer(s[i],
							xml.getChild(i).getStringProperty("driverName"),
							new File(xml.getChild(i).getStringProperty("file")),
							this.getFMap().getViewPort().getProjection());*/
					layer = new FLyrRaster();
					layer.setName(s[i]);
					if (!xml.getChild(i).contains("proj"))
						((FLyrRaster)layer).setProjection(this.getMapContext().getViewPort().getProjection());
					layer.setXMLEntity03(xml.getChild(i));
					try {
							layer.load();

					} catch (DriverIOException e) {
						throw new XMLException(e);
					}

				}else if (className.equals((FLayers.class.getName()))) {
					layer = new FLayers(getMapContext(),this);
					layer.setXMLEntity(xml.getChild(i));
				} else {
					// Capas Nuevas (externas)
					try {
						Class clase = Class.forName(className);
						layer = (FLayer) clase.newInstance();
						layer.setName(s[i]);
						layer.setXMLEntity03(xml.getChild(i));
						try {
								layer.load();
						} catch (DriverIOException e) {
							throw new XMLException(e);
						}
					} catch (Exception e) {
						//e.printStackTrace();
						throw new XMLException(e);
					}
				}

				this.addLayer(layer);
			}
		} /*azabala-modificaciones en layerfactory-
		catch (DriverException e) {
			throw new XMLException(e);
		}*/
		catch (DriverLoadException e) {
            throw new XMLException(e);
        }

	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 *
	 * @throws XMLException
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml) throws XMLException{
		super.setXMLEntity(xml);
		//LoadLayerException loadLayerException=new LoadLayerException();
		int numLayers = xml.getIntProperty("numLayers");

		String[] s = xml.getStringArrayProperty("LayerNames");
		// try {
		fmap.clearErrors();
			for (int i = 0; i < numLayers; i++) {
				this.addLayerFromXML(xml.getChild(i),s[i]);


			}
		/* } catch (DriverException e) {
			throw new XMLException(e);
		} */
		/*if (loadLayerException.getExceptions().size()>0){
			throw loadLayerException;
		}*/
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor,
	 * 		FBitSet)
	 */
	public void process(FeatureVisitor visitor, FBitSet subset)
		throws DriverException, VisitException {
		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer layer = (FLayer) iter.next();

			if (layer instanceof VectorialData) {
				((VectorialData) layer).process(visitor, subset);
			}
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor)
	 */
	public void process(FeatureVisitor visitor)
		throws DriverException, VisitException {
		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer layer = (FLayer) iter.next();

			if (layer instanceof FLayers){
				FLayers lyrs=(FLayers)layer;
				for (int i=0;i<lyrs.getLayersCount();i++){
					FLayer lyr=lyrs.getLayer(i);
					if (lyr.isActive()) {
						if (lyr instanceof VectorialData) {
							((VectorialData) lyr).process(visitor);
						}
					}
				}
			}
			if (layer.isActive()) {
				if (layer instanceof VectorialData) {
					((VectorialData) layer).process(visitor);
				}
			}
		}
	}
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData#process(com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor, Rectangle2D)
	 */
	public void process(FeatureVisitor visitor, Rectangle2D rect) throws DriverException, VisitException {
		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer layer = (FLayer) iter.next();

			if (layer.isActive()) {
				if (layer instanceof VectorialData) {
					((VectorialData) layer).process(visitor, rect);
				}
			}
		}

	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getMapContext()
	 */
	public MapContext getMapContext() {
		return fmap;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setCoordTrans(org.cresques.cts.ICoordTrans)
	 */
	public void setCoordTrans(ICoordTrans ct) {
		super.setCoordTrans(ct);

		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer layer = (FLayer) iter.next();
			layer.setCoordTrans(ct);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.LayerCollection#setAllActives(boolean)
	 */
	public void setAllActives(boolean active) {
		FLayer lyr;

		for (int i = 0; i < layers.size(); i++) {
			lyr = ((FLayer) layers.get(i));
			lyr.setActive(active);

			if (lyr instanceof LayerCollection) {
				((LayerCollection) lyr).setAllActives(active);
			}
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.layerOperations.LayerCollection#getActives()
	 */
	public FLayer[] getActives() {
		ArrayList ret = new ArrayList();
		LayersIterator it = new LayersIterator(this) {

			public boolean evaluate(FLayer layer) {
				return layer.isActive();
			}

		};

		while (it.hasNext())
		{
			ret.add(it.next());
		}
		return (FLayer[]) ret.toArray(new FLayer[0]);
	}

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#getMinScale()
     */
    public double getMinScale() {
        return -1; // La visibilidad o no la controla cada capa
        	// dentro de una coleccin
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#getMaxScale()
     */
    public double getMaxScale() {
        return -1;
    }
    public void setMinScale(double minScale)
    {
		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer lyr = (FLayer) iter.next();
			lyr.setMinScale(minScale);
		}
    }
    public void setMaxScale(double maxScale)
    {
		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer lyr = (FLayer) iter.next();
			lyr.setMinScale(maxScale);
		}
    }
    public void setActive(boolean b){
    	super.setActive(b);
    	for (int i=0;i<layers.size();i++){
    		((FLayer)layers.get(i)).setActive(b);
    	}
    }

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#addLayerListener(com.iver.cit.gvsig.fmap.layers.LayerListener)
	 */
	public boolean addLayerListener(LayerListener o) {
		for (int i = 0; i < layers.size(); i++)
			((FLayer) layers.get(i)).addLayerListener(o);
		return true;
	}

	public XMLItem[] getInfo(Point p, double tolerance, Cancellable cancel) throws DriverException {
		int i;
		Vector items = new Vector();
		FLayer layer;
		XMLItem[] aux;
		for (i = 0; i < this.layers.size(); i++){
			layer = (FLayer)layers.get(i);
			if (layer instanceof InfoByPoint){
				InfoByPoint queryable_layer = (InfoByPoint) layer;
				aux = queryable_layer.getInfo(p, tolerance, null);
				if (!(queryable_layer instanceof FLayers)){
					for(int j = 0; j < aux.length; j++){
						items.add(aux[j]);
					}
				}
			}
		}
		return (XMLItem[])items.toArray(new XMLItem[0]);

//		for (i = 0; i < this.layers.size(); i++){
//			FLayer laCapa = (FLayer) layers.get(i);
//			if (laCapa instanceof FLyrVect){
//			}
//			else if (laCapa instanceof RasterOperations) {
//				try {
//					RasterOperations layer = (RasterOperations) laCapa;
//					sb.append(layer.getInfo(p, tolerance));
//				} catch (DriverException e) {
//					e.printStackTrace();
//				}
//			}
//			else if (laCapa instanceof InfoByPoint) {
//				try {
//					InfoByPoint layer = (InfoByPoint) laCapa;
//					sb.append(layer.getInfo(p, tolerance));
//				} catch (DriverException e) {
//					e.printStackTrace();
//				}
//			}
//		}

	}

	public ImageIcon getTocImageIcon() {
		return new ImageIcon(MapControl.class.getResource("images/layerGroup.png"));
	}

	public boolean isDirty() {
		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer lyr = (FLayer) iter.next();
			if (lyr.isDirty())
				return true;
		}
		return false;
	}

	public void setDirty(boolean dirty) {
		for (Iterator iter = layers.iterator(); iter.hasNext();) {
			FLayer lyr = (FLayer) iter.next();
			lyr.setDirty(dirty);
		}

	}

	public boolean addLayerFromXMLEntity(XMLEntity xml, String name) {
		fmap.clearErrors();
		this.addLayerFromXML(xml,name);

		return (fmap.getLayersError().size() == 0);

	}

	private void addLayerFromXML(XMLEntity xml, String name) {
		FLayer layer = null;
		long t1 = System.currentTimeMillis();
		try {
		if (name == null) name = xml.getName();


		String className = xml.getStringProperty("className");
		//TODO VCN FLyrAnnotation es un parche para no tener que duplicar todo el cdigo de aq y de los diferentes mtodos de LayerFactory,
		//ya que los drivers de una FLyrAnnotation no sabemos cual es puede ser cualquier Driver Vectorial.
		if (className.equals(FLyrVect.class.getName()) || className.equals(FLyrAnnotation.class.getName())) {
			String type = xml.getStringProperty("type");
			if ("vectorial".equals(type)){
				//String recordsetName = xml.getChild(i).getStringProperty("recordset-name");
                IProjection proj = null;
                if (xml.contains("proj")) {
                    proj = CRSFactory.getCRS(xml.getStringProperty("proj"));
                }
                else
                {
                    proj = this.getMapContext().getViewPort().getProjection();
                }
				if (xml.contains("file")) {
					Driver d;
					try {
						d = LayerFactory.getDM().getDriver(xml.getStringProperty("driverName"));
					} catch (DriverLoadException e1) {
						throw new XMLException(e1);
					}
					layer = LayerFactory.createLayer(name, (VectorialFileDriver) d,
													new File(xml.getStringProperty("file")),
													proj);


				}
                if (xml.contains("db")) {

                    String driverName = xml.getStringProperty("db");
                    IVectorialDatabaseDriver driver;
                    try {
                        driver = (IVectorialDatabaseDriver) LayerFactory.getDM().getDriver(driverName);
                        //Hay que separar la carga de los datos del XMLEntity del load.
                        driver.setXMLEntity(xml.getChild(2));

                        boolean loadOk = false;
//                        if (driver instanceof DefaultJDBCDriver) {
							try {
								((IVectorialDatabaseDriver) driver).load();
								if (((IVectorialDatabaseDriver) driver)
										.getConnection() != null) {
									loadOk = true;
								}
							} catch (DriverException e) {
								e.printStackTrace();
								loadOk = false;
							}
//						} else {
//							try{
//								((IVectorialDatabaseDriver)driver).load();
//								loadOk=true;
//							}catch (Exception e) {
//								loadOk=false;
//							}
//						}

                        layer = LayerFactory.createDBLayer(driver, name, proj);
                        if (!loadOk) {
                        	layer.setAvailable(false);
                        }

                    } catch (DriverLoadException e) {
                        e.printStackTrace();
                        throw new XMLException(e);
                    }

                }
                // Clases con algun driver genrico creado por otro
                // programador
                if (xml.contains("other")) {

                    String driverName = xml.getStringProperty("other");
                    VectorialDriver driver = null;
                    try {
                        driver = (VectorialDriver) LayerFactory.getDM().getDriver(driverName);
                    } catch (DriverLoadException e) {
                        // Si no existe ese driver, no pasa nada.
                        // Puede que el desarrollador no quiera que
                        // aparezca en el cuadro de dilogo y ha metido
                        // el jar con sus clases en nuestro directorio lib.
                        // Intentamos cargar esa clase "a pelo".
                        if (xml.getChild(2).contains("className"))
                        {
                            String className2 = xml.getChild(2).getStringProperty("className");
                            try {
                                driver = (VectorialDriver) Class.forName(className2).newInstance();
                            } catch (Exception e1) {
                                throw new XMLException(e1);
                            }
                        }
                    } catch (NullPointerException npe) {
                        // Si no existe ese driver, no pasa nada.
                        // Puede que el desarrollador no quiera que
                        // aparezca en el cuadro de dilogo y ha metido
                        // el jar con sus clases en nuestro directorio lib.
                        // Intentamos cargar esa clase "a pelo".
                        if (xml.getChild(2).contains("className"))
                        {
                            String className2 = xml.getChild(2).getStringProperty("className");
                            try {
                                driver = (VectorialDriver) Class.forName(className2).newInstance();
                            } catch (Exception e1) {
                                throw new XMLException(e1);
                            }
                        }
                    }
                    if (driver instanceof IPersistance)
                    {
                        IPersistance persist = (IPersistance) driver;
                        persist.setXMLEntity(xml.getChild(2));
                    }
                    layer = LayerFactory.createLayer(name, driver, proj);
                }

			}

			//TODO VCN FLyrAnnotation es un parche para no tener que duplicar todo el cdigo de aq y de los diferentes mtodos de LayerFactory,
			//ya que los drivers de una FLyrAnnotation no sabemos cual es puede ser cualquier Driver Vectorial.
			if (className.equals(FLyrAnnotation.class.getName())){
				layer=FLyrAnnotation.createLayerFromVect((FLyrVect)layer);
			}


			layer.setXMLEntity(xml);

//		} else if (className.equals(FLyrWMS.class.getName())) {
//			try {
//				layer = LayerFactory.createLayer(s[i],
//						StringUtilities.string2Rect(
//							xml.getChild(i).getStringProperty("fullExtent")),
//						new URL(xml.getChild(i).getStringProperty("host")),
//						xml.getChild(i).getStringProperty("format"),
//						xml.getChild(i).getStringProperty("layerQuery"),
//						xml.getChild(i).getStringProperty("infoLayerQuery"),
//						xml.getChild(i).getStringProperty("srs"));
//				layer.setXMLEntity(xml.getChild(i));
//			} catch (MalformedURLException e) {
//				throw new XMLException(e);
//			}
		} else if (className.equals((FLyrRaster.class.getName()))) {
			//if (xml.getChild(i).contains("file")) {
			/*layer = LayerFactory.createLayer(s[i],
					xml.getChild(i).getStringProperty("driverName"),
					new File(xml.getChild(i).getStringProperty("file")),
					this.getFMap().getViewPort().getProjection());*/
			Class clase = LayerFactory.getLayerClassForLayerClassName(className);
			layer = (FLayer) clase.newInstance();
			layer.setName(name);
			if (!xml.contains("proj"))
				((FLyrRaster)layer).setProjection(this.getMapContext().getViewPort().getProjection());
			layer.setXMLEntity(xml);
			try {
				layer.load();
			} catch (DriverIOException e) {
				throw new XMLException(e);
			}

		}else if (className.equals((FLayers.class.getName()))) {
			layer = new FLayers(getMapContext(),this);
			layer.setXMLEntity(xml);
		} else {
			// Capas Nuevas (externas)
				Class clase = LayerFactory.getLayerClassForLayerClassName(className);
				layer = (FLayer) clase.newInstance();
				layer.setName(name);
				layer.setXMLEntity(xml);
				try {
					layer.load();
				}catch (DriverIOException e) {
					throw new XMLException(e);
				}

		}

		this.addLayer(layer);
		long t2 = System.currentTimeMillis();
		logger.debug("layer: "+ layer.getName() +" loaded. " + (t2-t1) + " msecs.");
		// Comprobar que la proyeccin es la misma que la de FMap
        // Si no lo es, es una capa que est reproyectada al vuelo
        IProjection proj = layer.getProjection();
        if ((proj != null))
            if (proj != getMapContext().getProjection())
            {
                ICoordTrans ct = proj.getCT(getMapContext().getProjection());
                // TODO: REVISAR CON LUIS
                // Se lo fijamos a todas, luego cada una que se reproyecte
                // si puede, o que no haga nada
                layer.setCoordTrans(ct);
                // NOTA: DESPUS DE ESTO DEBERAMOS CREAR LAS ETIQUETAS
                // SI ES NECESARIO.
                // TODO: REVISAR A FONDO ESTO, PORQUE LAS ETIQUETAS SE CREAN
                // 2 VECES SI HAY REPROYECCN
                if (layer instanceof FLyrVect)
                {
                    FLyrVect lyrVect = (FLyrVect)layer;
                    VectorialLegend legend = (VectorialLegend) lyrVect.getLegend();
                    if (legend.getLabelField() != null) {
                        lyrVect.createLabelLayer(lyrVect.getRecordset());
                    }

                }
            }
        }catch (Exception e) {
        		fmap.addLayerError(xml.getStringProperty("name"));
				//loadLayerException.addException(e);
			/*if (layer!=null){
        	layer.setActive(false);
			layer.setVisible(false);
			this.addLayer(layer);
        	}*/
			//throw new XMLException(e);
            System.err.println("No se ha podido cargar la capa : "+xml.getStringProperty("name"));
            logger.debug("No se ha podido cargar la capa : "+xml.getStringProperty("name") + ". Razn:");
            // logger.debug(e.fillInStackTrace());
            e.printStackTrace();
        }
	}

	private void addLayerFromXMLNew(XMLEntity xml, String name) {
		FLayer layer = null;


		try {
		String className = xml.getStringProperty("className");

		if (className.equals((FLayers.class.getName()))){
			layer = new FLayers(getMapContext(),this);
		} else {
//		 	Por compatibilidad
			if (className.equals(FLyrVect.class.getName())) {
				if (xml.contains("file")) {
					layer = new FLayerFileVectorial();
				} else if (xml.contains("db")) {
					try {
						layer = (FLayer)((ExtensionPoint)ExtensionPointsSingleton.getInstance().get("Layers")).create("com.iver.cit.gvsig.fmap.layers.FLayerJDBCVectorial");
					} catch (Exception e) {
						throw new XMLException(new Exception("No se tiene registrada la capa de tipo JDBC"));
					}
					//className = FLayerJDBCVectorial.class.getName();
				} else if (xml.contains("other")){
					layer = new FLayerGenericVectorial();
				} else {
					throw new XMLException(new Exception("Capa vectorial de tipo no reconocido"));
				}
//				Fin por compatibilidad
			} else {
				try {
					layer = (FLayer)(((ExtensionPoint)ExtensionPointsSingleton.getInstance().get("Layers")).create(className));
				} catch (Exception e) {
					//puende que no este registrada como punto de extension
					Class clase = Class.forName(className);
					layer = (FLayer) clase.newInstance();
					// FIXME: Hacemos algo aqui o dejamos que suba el error?
				}
			}

		}
		layer.setXMLEntity(xml);
		if (name != null) layer.setName(name);
		try {
			layer.load();
		}catch (DriverIOException e) {
			throw new XMLException(e);
		}

		this.addLayer(layer);
		logger.debug("layer: "+ layer.getName() +" loaded");
		// Comprobar que la proyeccin es la misma que la de FMap
        // Si no lo es, es una capa que est reproyectada al vuelo
        IProjection proj = layer.getProjection();
        if ((proj != null))
            if (proj != getMapContext().getProjection())
            {
                ICoordTrans ct = proj.getCT(getMapContext().getProjection());
                // TODO: REVISAR CON LUIS
                // Se lo fijamos a todas, luego cada una que se reproyecte
                // si puede, o que no haga nada
                layer.setCoordTrans(ct);
                // NOTA: DESPUS DE ESTO DEBERAMOS CREAR LAS ETIQUETAS
                // SI ES NECESARIO.
                // TODO: REVISAR A FONDO ESTO, PORQUE LAS ETIQUETAS SE CREAN
                // 2 VECES SI HAY REPROYECCN
                if (layer instanceof FLyrVect)
                {
                    FLyrVect lyrVect = (FLyrVect)layer;
                    VectorialLegend legend = (VectorialLegend) lyrVect.getLegend();
                    if (legend.getLabelField() != null) {
                        lyrVect.createLabelLayer(lyrVect.getRecordset());
                    }

                }
            }
        }catch (Exception e) {
        		fmap.addLayerError(xml.getStringProperty("name"));
				//loadLayerException.addException(e);
			/*if (layer!=null){
        	layer.setActive(false);
			layer.setVisible(false);
			this.addLayer(layer);
        	}*/
			//throw new XMLException(e);
            System.err.println("No se ha podido cargar la capa : "+xml.getStringProperty("name"));
            logger.debug("No se ha podido cargar la capa : "+xml.getStringProperty("name") + ". Razn:");
            // logger.debug(e.fillInStackTrace());
            e.printStackTrace();
        }
	}

	/**
	 * This method calls recursively to its layer children to tell that the layer is going
	 * to be removed from the view. This overwrites FLyrDefault.removingThisLayer().
	 */
	public void removingThisLayer() {

		for (int i=0; i < layers.size(); i++) {
			FLayer lyr = (FLayer) layers.get(i);
			if (lyr instanceof FLyrDefault) {
				((FLyrDefault) lyr).removingThisLayer();
			}
		}
	}

}
