/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.layers.FLayer;


/**
 * Interfaz que ofrece los mtodos para iniciar, finalizar y visitar una
 * feature.
 */
public interface FeatureVisitor {
	/**
	 * Recibe las geometras a medida que se van recorriendo en la estrategia.
	 *
	 * @param g Geometra que se recorre
	 * @param index ndice de la geometra
	 *
	 * @throws VisitException 
	 */
	void visit(IGeometry g, int index) throws VisitException;
	
	/**
	 * Recibe las geometras con su smbolo correpondiente
	 *
	 * @param g Geometra que se recorre
	 * @param index ndice de la geometra
	 * @param symbol Symbol associated to feature
	 *
	 * @throws VisitException 
	 */
//	void visitWithSymbol(IGeometry g, int index, ISymbol symbol) throws VisitException;
	
	
	/**
	 * All FeatureVisitor is linked with a call to Strategy.processXXX method.
	 * It represents an iterative process over all (or some) features of a Layer.
	 * This method returns a descriptive text of the process that a visitor makes.
	 * 
	 * FIXME Internacionalizamos el mensaje???
	 * @return
	 */
	 String getProcessDescription();

	/**
	 * Mtodo invocado al finalizar las visitas con el fin de que se puedan
	 * liberar los recursos reservados en start
	 *
	 * @param layer Capa sobre la que se actua
	 */
	void stop(FLayer layer);

	/**
	 * Mtodo invocado antes de las visitas para que el visitor pueda reservar
	 * algn tipo de recurso que sea necesario
	 *
	 * @param layer Capa sobre la que se acta
	 *
	 * @return Devuelve true si el visitor se puede aplicar sobre la capa que
	 * 		   se pasa como parmetro
	 */
	boolean start(FLayer layer);
}
