/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import com.hardcode.gdbms.engine.data.driver.DriverException;

import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.XMLException;

import com.iver.utiles.XMLEntity;


/**
 * Clase factoria de las diferentes leyendas.
 *
 * @author Fernando Gonzlez Corts
 */
public class LegendFactory {
	public static FSymbol DEFAULT_POINT_SYMBOL = new FSymbol(FConstant.SYMBOL_TYPE_POINT);
	public static FSymbol DEFAULT_LINE_SYMBOL = new FSymbol(FConstant.SYMBOL_TYPE_LINE);
	public static FSymbol DEFAULT_POLYGON_SYMBOL = new FSymbol(FConstant.SYMBOL_TYPE_FILL);

	// TODO Descomentar esto cuando se tenga la interfaz de datos

	/*
	 * Crea un objeto renderer de valor nico con las caractersticas que se pasan como parmetro y con los smbolos por defecto. En funcin de la carga del sistema se podr crear un FRenderer que almacene la referencia al FRecordset, o un FRenderer que haga cach de los valores necesarios
	 *
	       public static Legend createIntervalLegend(FRecordset data, int valueField, int labelField ) {
	       }
	       /*
	 * Crea un objeto renderer de valor nico con las caractersticas que se pasan como parmetro y con los smbolos por defecto. En funcin de la carga del sistema se podr crear un FRenderer que almacene la referencia al FRecordset, o un FRenderer que haga cach de los valores necesarios
	 *
	       public static Legend createUniqueValueLegend(FRecordset data, int valueField, int labelField ) {
	       }
	 */

	/**
	 * Crea un objeto renderer de smbolo nico con las caractersticas que se
	 * pasan como parmetro
	 *
	 * @param shapeType Tipo de shape.
	 *
	 * @return VectorialLegend.
	 */
	public static VectorialLegend createSingleSymbolLegend(int shapeType) {
		switch (shapeType) {
			case FShape.POINT:
				return new SingleSymbolLegend(new FSymbol(
						FConstant.SYMBOL_TYPE_POINT));

			case FShape.LINE:
				return new SingleSymbolLegend(new FSymbol(
						FConstant.SYMBOL_TYPE_LINE));

			case FShape.POLYGON:
				return new SingleSymbolLegend(new FSymbol(
						FConstant.SYMBOL_TYPE_FILL));
			case FShape.MULTIPOINT:
				return new SingleSymbolLegend(new FSymbol(
						FConstant.SYMBOL_TYPE_POINT));
			case FShape.TEXT:
				return new SingleSymbolLegend(new FSymbol(
						FConstant.SYMBOL_TYPE_TEXT));
		}

		return new SingleSymbolLegend(new FSymbol(FConstant.SYMBOL_TYPE_FILL));
	}

	/**
	 * Crea un objeto VectorialUniqueValueLegend vaco, dispuesto para cargar
	 * smbolos
	 *
	 * @param shapeType Tipo de shape.
	 *
	 * @return VectorialUniqueValueLegend.
	 */
	public static VectorialUniqueValueLegend createVectorialUniqueValueLegend(
		int shapeType) {
		return new VectorialUniqueValueLegend(shapeType);
	}

	/**
	 * Crea un objeto VectorialIntervalLegend vaco, dispuesto para cargar
	 * smbolos
	 *
	 * @param shapeType tipo de shape.
	 *
	 * @return VectorialIntervalLegend
	 */
	public static VectorialIntervalLegend createVectorialIntervalLegend(
		int shapeType) {
		return new VectorialIntervalLegend(shapeType);
	}

	/**
	 * Obtiene un objeto con las propiedades del renderer. Este mtodo se
	 * invocar con el fin de guardar el smbolo en disco.
	 *
	 * @param r VectorialLegend.
	 *
	 * @return XMLEntity.
	 *
	 * @throws DriverException
	 */
	public static XMLEntity getXMLEntity(VectorialLegend r)
		throws DriverException {
		XMLEntity xml = r.getXMLEntity();

		return xml;
	}

	/**
	 * Crea un renderer con la informacin contenida en el objeto XMLEntity
	 *
	 * @param xml XMLEntity.
	 *
	 * @return VectorialLegend
	 *
	 * @throws XMLException
	 */
	public static VectorialLegend createFromXML03(XMLEntity xml)
		throws XMLException {
		//TODO Implementar bien
		try {
			VectorialLegend vl = null;
			Class clase = Class.forName(xml.getStringProperty("className"));
			vl = (VectorialLegend) clase.newInstance();
			vl.setXMLEntity03(xml);

			return vl;
		} catch (ClassNotFoundException e) {
			throw new XMLException(e);
		} catch (InstantiationException e) {
			throw new XMLException(e);
		} catch (IllegalAccessException e) {
			throw new XMLException(e);
		}
	}

	/**
	 * Crea un renderer con la informacin contenida en el objeto XMLEntity
	 *
	 * @param xml XMLEntity.
	 *
	 * @return VectorialLegend
	 *
	 * @throws XMLException
	 */
	public static VectorialLegend createFromXML(XMLEntity xml)
		throws XMLException {
		//TODO Implementar bien
		try {
			VectorialLegend vl = null;
			Class clase = Class.forName(xml.getStringProperty("className"));
			vl = (VectorialLegend) clase.newInstance();
			vl.setXMLEntity(xml);

			return vl;
		} catch (ClassNotFoundException e) {
			throw new XMLException(e);
		} catch (InstantiationException e) {
			throw new XMLException(e);
		} catch (IllegalAccessException e) {
			throw new XMLException(e);
		}
	}

	/**
	 * Clona la leyenda.
	 *
	 * @param l VectorialLegend a clonar.
	 *
	 * @return VectorialLegend cloando.
	 *
	 * @throws XMLException
	 * @throws DriverException
	 */
	public static VectorialLegend cloneLegend(VectorialLegend l)
		throws XMLException, DriverException {
		return createFromXML(getXMLEntity(l));
	}
}
