/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.ISymbol;

import com.iver.utiles.XMLEntity;


/**
 * Interfaz de Leyenda Vectorial.
 *
 * @author Vicente Caballero Navarro
 */
public interface VectorialLegend extends Legend {
	/**
	 * Inserta el DataSource.
	 *
	 * @param ds DataSource.
	 *
	 * @throws FieldNotFoundException cuando el campo no se encuentre.
	 * @throws DriverException Cuando falle el driver.
	 */
	void setDataSource(DataSource ds)
		throws FieldNotFoundException, DriverException;

	/**
	 * Devuelve el smbolo asociado al objeto que se pasa como parmetro. Si se
	 * pasa null se devuelve el smbolo por defecto. En un Legend de smbolo
	 * nico siempre se devuelve dicho smbolo
	 *
	 * @param recordIndex ndice
	 *
	 * @return Smbolo.
	 *
	 * @throws DriverException
	 */
	ISymbol getSymbol(int recordIndex) throws DriverException;
    
    ISymbol getSymbolByFeature(IFeature feat);

	/**
	 * Devuelve el tipo de shape.
	 *
	 * @return Devuelve el tipo de shapes para los que est preparada la
	 * 		   leyenda
	 */
	int getShapeType();

	/**
	 * Inserta el tipo de shape.
	 *
	 * @param shapeType Tipo de shape.
	 */
	void setShapeType(int shapeType);

	/**
	 * Establece el smbolo por defecto del Legend. En un Legend de smbolo
	 * nico, el smbolo se establece invocando este mtodo
	 *
	 * @param s Smbolo por defecto.
	 */
	void setDefaultSymbol(ISymbol s);

	/**
	 * Establece el campo por el que se etiqueta. -1 quita el etiquetado
	 *
	 * @param fieldName Nombre del campo.
	 */
	void setLabelField(String fieldName);
    
    /**
     * @param fieldName
     */
    void setLabelHeightField(String fieldName);
    /**
     * @param fieldName
     */
    void setLabelRotationField(String fieldName);

	/**
	 * Obtiene el ndice del campo que sirve de etiquetado o -1 si no ha sido
	 * etiquetado
	 *
	 * @return nombre del campo.
	 */
	String getLabelField();

	/**
	 * Devuelve el alto de campo.
	 *
	 * @return alto.
	 */
	String getLabelHeightField();

	/**
	 * Devuelve la rotacin.
	 *
	 * @return Rotacin.
	 */
	String getLabelRotationField();

	/**
	 * Devuelve el XMLEntity.
	 *
	 * @return XMLEntity.
	 */
	XMLEntity getXMLEntity();

	/**
	 * Inserta el XMLEntity.
	 *
	 * @param xml XMLEntity.
	 */
	void setXMLEntity(XMLEntity xml);

    /**
     * Inserta el XMLEntity.
     *
     * @param xml XMLEntity.
     */
    void setXMLEntity03(XMLEntity xml);

    void useDefaultSymbol(boolean b);
    /* public boolean isBWithHeightText();
	public void setBWithHeightText(boolean withHeightText); */
	/*
	 * supplierCardinality 0..*
	 *
	       /*# FInterval lnkFInterval; */
	/*supplierCardinality 0..*
	 *
	       private LegendListener[] listeners;*/
    
    /**
     * Se usa para saber antes de dibujar, los campos que vamos
     * a necesitar leer, y leer solo estos. Acelera el dibujado
     * bastante, sobre todo en bases de datos espaciales.
     * @return el array con los nombres de los campos usados.
     * (Por ejemplo para el etiquetado, el campo de clasificacin, etc)
     */
    String[] getUsedFields();
}
