package com.hardcode.gdbms.engine.data.driver;

import com.hardcode.gdbms.engine.values.Value;


/**
 * Interfaz que define los mtodos de acceso de lectura
 *
 * @author Fernando Gonzlez Corts
 */
public interface ReadAccess {
	public final static int FIELD_TYPE_BOOLEAN = 0;
	public final static int FIELD_TYPE_STRING = 5;
	public final static int FIELD_TYPE_DOUBLE = 4;
	public final static int FIELD_TYPE_FLOAT = 3;
	public final static int FIELD_TYPE_LONGINT = 2;
	public final static int FIELD_TYPE_INT = 1;

	/**
	 * Obtiene el valor que se encuentra en la fila y columna indicada
	 *
	 * @param rowIndex fila
	 * @param fieldId columna
	 *
	 * @return subclase de Value con el valor del origen de datos. Never null (use 
     * ValueFactory.createNullValue() instead)
	 *
	 * @throws DriverException Si se produce un error accediendo al DataSource
	 */
	public abstract Value getFieldValue(long rowIndex, int fieldId)
		throws DriverException;

	/**
	 * Obtiene el nmero de campos del DataSource
	 *
	 * @return
	 *
	 * @throws DriverException Si se produce algn error accediendo al
	 * 		   DataSource
	 */
	public abstract int getFieldCount() throws DriverException;

	/**
	 * Devuelve el nombre del campo fieldId-simo
	 *
	 * @param fieldId ndice del campo cuyo nombre se quiere obtener
	 *
	 * @return
	 *
	 * @throws DriverException Si se produce algn error accediendo al
	 * 		   DataSource
	 */
	public abstract String getFieldName(int fieldId) throws DriverException;

	/**
	 * Obtiene el nmero de registros del DataSource
	 *
	 * @return
	 *
	 * @throws DriverException Si se produce algn error accediendo al
	 * 		   DataSource
	 */
	public abstract long getRowCount() throws DriverException;

	/**
	 * Devuelve el tipo del campo isimo. Devuelve una constante de la clase
	 * java.sql.Types
	 *
	 * @param i ndice del campo cuyo tipo se quiere conocer
	 *
	 * @return Class
	 *
	 * @throws DriverException Si se produce un error accediendo a los tipos de
	 * 		   datos
	 */
	public abstract int getFieldType(int i) throws DriverException;
	
	/**
	 * Devuelve el ancho del campo isimo.
	 *
	 * @param i ndice del campo cuyo ancho se quiere conocer
	 *
	 * @return int
	 *
	 * @throws DriverException Si se produce un error accediendo a los tipos de
	 * 		   datos
	 */
	public abstract int getFieldWidth(int i) throws DriverException;
}
