/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs.gui.panels;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import org.cresques.cts.IProjection;
import org.gvsig.crs.CrsException;
import org.gvsig.crs.CrsFactory;
import org.gvsig.crs.ICrs;
import org.gvsig.crs.persistence.CrsData;
import org.gvsig.crs.persistence.RecentCRSsPersistence;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.gui.TableSorter;

/**
 * Clase que genera el panel de recientes
 * 
 * @author Jos Luis Gmez Martnez (jolugomar@gmail.com)
 * @author Diego Guerrero Sevilla (diego.guerrero@uclm.es)
 *
 */
public class CrsRecentsPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public JTable jTable = null;
	private JScrollPane jScrollPane = null;
	public DefaultTableModel dtm = null;
	public TableSorter sorter = null;
	private CrsData crsDataArray[] = null;
	
	public int selectedRowTable = -1;
	private String authority = null;
	private int codeCRS = -1;
	private ICrs crs = null;
	
	public CrsRecentsPanel() {
		super();
		initialize();
	}
	
	private void initialize(){
		this.setLayout(new GridLayout(1,2));
		this.setLayout(new FlowLayout(FlowLayout.LEFT,10,10));
		this.add(getJLabel(), null);
		this.add(getJScrollPane(), null);
	}
	
	private JLabel getJLabel(){
		JLabel label = new JLabel();
		label.setPreferredSize(new Dimension(500, 30));
		label.setText(PluginServices.getText(this, "ultimos_crs_utilizados")+":");
		return label;
	}
	
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setPreferredSize(new Dimension(500,150));
			jScrollPane.setBorder(
				    BorderFactory.createCompoundBorder(
					BorderFactory.createCompoundBorder(
							BorderFactory.createTitledBorder(PluginServices.getText(this,"coordinate_reference_systems")),
							BorderFactory.createEmptyBorder(5,5,5,5)),
							jScrollPane.getBorder()));
			jScrollPane.setViewportView(getJTable());
		}
		return jScrollPane;
	}
	
	public JTable getJTable() {
		if (jTable == null) {
			String[] columnNames= {PluginServices.getText(this,"fuente"),
					PluginServices.getText(this,"codigo"),
					PluginServices.getText(this,"nombre")};
			Object[][]data = {};			
			dtm = new DefaultTableModel(data, columnNames)
			 {
				public boolean isCellEditable(int row, int column) {
					return false;
				}
				/*
				 * metodo necesario para cuando utilizamos tablas ordenadas
				 * ya que sino al ordenar por algun campo no se queda con el orden
				 * actual al seleccionar una fila (non-Javadoc)
				 * @see javax.swing.table.TableModel#getColumnClass(int)
				 */
				public Class getColumnClass(int column)
				{
					return getValueAt(0, column).getClass();
				}
				};
			sorter = new TableSorter(dtm);			

			jTable = new JTable(sorter);
			sorter.setTableHeader(jTable.getTableHeader());
			jTable.setCellSelectionEnabled(false);
			jTable.setRowSelectionAllowed(true);
			jTable.setColumnSelectionAllowed(false);
			jTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			TableColumn column = null;
			for (int i = 0; i < 3; i++) {
			    column = jTable.getColumnModel().getColumn(i);
			    if (i == 0) {
			        column.setPreferredWidth(60); //code column is shorter			     	
			    }else if (i == 2) {
			    	column.setPreferredWidth(275);
			    }
			    else {			    
			        column.setPreferredWidth(160);
			    }
			}	
		}
		return jTable;		
	}
	
	public ICrs getProjection() {		
		return crs;
	}
	
	public void setCodeCRS(int code) {
		codeCRS = code;
	}
	
	public int getCodeCRS() {
		return codeCRS;
	}
	
	public void setProjection(IProjection crs) {
		//setCrs((ICrs) crs);
	}
	
	/**
	 * Carga en la tabla los CRSs leidos del sistema de persistencia.
	 */
	public void loadRecents(){
		RecentCRSsPersistence persistence = new RecentCRSsPersistence();
		crsDataArray = persistence.getArrayOfCrsData();
		
		for (int iRow = crsDataArray.length-1;iRow>=0;iRow--){
			Object row[] ={crsDataArray[iRow].getAuthority(),Integer.toString(crsDataArray[iRow].getCode()),crsDataArray[iRow].getName()};
			dtm.addRow(row);
		}
		
		/*
		/*Seleccionar el primer registro.
		 */
		int numr = dtm.getRowCount();
		if (numr != 0 )
			this.getJTable().setRowSelectionInterval(0,0);
	}

	public ICrs getCrs() {
		return crs;
	}
	
	public void initCrs(){
		
		selectedRowTable = getJTable().getSelectedRow();		        			        	
    	Integer.parseInt((String)sorter.getValueAt(selectedRowTable,1));
		authority = (String)sorter.getValueAt(selectedRowTable,0);
		codeCRS = Integer.parseInt((String)sorter.getValueAt(selectedRowTable,1));
		try {
			crs = new CrsFactory().getCRS(authority+":"+String.valueOf(codeCRS));
		} catch (CrsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
