package org.gvsig.remoteClient.gml.utils;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;

import org.gvsig.remoteClient.gml.GMLTags;
import org.gvsig.remoteClient.gml.GMLFeaturesParser.Extent;
import org.gvsig.remoteClient.gml.exceptions.BaseException;
import org.gvsig.remoteClient.gml.exceptions.GMLFileReadException;
import org.gvsig.remoteClient.gml.exceptions.GMLNoGeometryException;
import org.gvsig.remoteClient.gml.exceptions.GMLParserException;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLUtilsParser.java,v 1.1.2.3 2007/06/19 09:44:46 jorpiell Exp $
 * $Log: GMLUtilsParser.java,v $
 * Revision 1.1.2.3  2007/06/19 09:44:46  jorpiell
 * It always uses geotools to parse the geometries
 *
 * Revision 1.1.2.2  2007/02/23 07:57:27  jorpiell
 * Capacidad de parsear el BBOX que tenga algn problema
 *
 * Revision 1.1.2.1  2007/01/25 16:12:59  jorpiell
 * Se han sustituido las clases por las que hay en el nuevo driver de GML.
 *
 * Revision 1.3  2007/01/15 13:11:00  csanchez
 * Sistema de Warnings y Excepciones adaptado a BasicException
 *
 * Revision 1.2  2006/12/22 11:25:44  csanchez
 * Nuevo parser GML 2.x para gml's sin esquema
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 */
public class GMLUtilsParser {

	/**
	 * Parses the boundedBy GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	public static Extent parseBoundedBy(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		int currentTag;
		boolean end = false;
		Rectangle2D extent = null;	
		String srs = null;
		
		currentTag = parser.getEventType();	
		
		while (!end){
			switch(currentTag){
			case XMLSchemaParser.START_TAG:
				if (parser.getName().compareTo(GMLTags.GML_BOX)==0){
					for (int i=0 ; i<parser.getAttributeCount() ;i++){
						if (parser.getAttributeName(i).compareTo(GMLTags.GML_SRS_NAME) == 0){
							srs = parser.getAttributeValue(i);
						}
					}
					extent = parseBox(parser);					
				}   
				break;
			case XMLSchemaParser.END_TAG:
				if (parser.getName().compareTo(GMLTags.GML_BOUNDEDBY) == 0)
					end = true;
				break;
			case XMLSchemaParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}	
		return new Extent(extent,srs);
	}
	
	/**
	 * Parses the box GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Rectangle2D parseBox(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		int currentTag;
		boolean end = false;
		Point2D ul = null;
		Point2D br = null;		
		Rectangle2D extent = null;
		
		currentTag = parser.getEventType();	
		
		while (!end){
			switch(currentTag){
			case XMLSchemaParser.START_TAG:
				if (parser.getName().compareTo(GMLTags.GML_COORDINATES)==0){
						extent = parseCoordinates(parser);
				}else if(parser.getName().compareTo(GMLTags.GML_COORD)==0){
					if (ul == null){
						ul = parsePoint(parser);
					}else{
						br = parsePoint(parser);
					}
				}   
				break;
			case XMLSchemaParser.END_TAG:
				if (parser.getName().compareTo(GMLTags.GML_BOX) == 0)
					end = true;
				break;
			case XMLSchemaParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}	
		if (extent == null){
			return new Rectangle2D.Double(ul.getX(),ul.getY(),br.getX(),br.getY());
		}else{
			return extent;
		}
	}

	/**
	 * Parses the coordinates GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Rectangle2D parseCoordinates(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		parser.next();
		
		for (int i=0 ; i<parser.getAttributeCount() ; i++){
    		if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_DECIMAL) == 0){
    			
    		}else if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_CS) == 0){
    			
    		}else if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_TS) == 0){
    			
    		}
    	}			
		
		String[] corners = parser.getText().trim().split(" ");
		String[] ul = corners[0].split(",");
		String[] br = corners[1].split(",");
		
		
		return new Rectangle2D.Double(Double.parseDouble(ul[0]),
				Double.parseDouble(ul[1]),
				Double.parseDouble(br[0]),
				Double.parseDouble(br[1]));
		
	}
	
	/**
	 * Parses the point GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Point2D parsePoint(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		int currentTag;
		boolean end = false;		
		String x = "0";
		String y = "0";
		
		currentTag = parser.getEventType();		
		
		while (!end){
			switch(currentTag){
			case XMLSchemaParser.START_TAG:
				if (parser.getName().compareTo(GMLTags.GML_COORDINATES)==0){
					parseCoordinates(parser);					
				}
				if (parser.getName().compareTo(GMLTags.GML_X) == 0){
					parser.next();
					x = parser.getText();
				}else if (parser.getName().compareTo(GMLTags.GML_Y) == 0){
					parser.next();
					y = parser.getText();
					end = true;
				}
				break;
			case XMLSchemaParser.END_TAG:
				break;
			case XMLSchemaParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}	
		return new Point2D.Double(Double.parseDouble(x),Double.parseDouble(y));
	}
	
	/**
	 * Parses a geometry GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	public static Object parseGeometry(XMLSchemaParser parser,String gmGeometryTag,IGeometriesFactory factory) throws BaseException{
		int currentTag;
		boolean end = false;		
		
		try{				
			currentTag = parser.getEventType();
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					return factory.createGeometry(getGeometryInGML(parser,parser.getName()));					
				case KXmlParser.END_TAG:
					if ((parser.getName().compareTo(gmGeometryTag) == 0))
						end = true;
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}catch (XmlPullParserException e) {
			// Captured KXML parsing Exception
			throw new GMLParserException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new GMLFileReadException(e);
		} 					
		return null;
	}
	
	/**
	 * Parses the point GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Point2D parsePoint2D(XMLSchemaParser parser) throws BaseException{
		int currentTag;
		boolean end = false;		
		
		try{				
			currentTag = parser.getEventType();					
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					if ((parser.getName().compareTo(GMLTags.GML_COORDINATES) == 0)){
						parser.next();
						String sCoordinates = parser.getText();
						String[] coordinates = sCoordinates.split(",");
						return new Point2D.Double(Double.parseDouble(coordinates[0]),
								Double.parseDouble(coordinates[1]));
					}
					break;
				case KXmlParser.END_TAG:
					if ((parser.getName().compareTo(GMLTags.GML_POINT) == 0))
						end = true;
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}catch (XmlPullParserException e) {
			//Captured KXML parsing Exception
			throw new GMLParserException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new GMLFileReadException(e);
		} 					
		return null;	
	}
	
	/**
	 * Parses the geometry GML tag
	 * @return
	 * The geometry in GML
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static String getGeometryInGML(XMLSchemaParser parser,String endGeometryTag) throws BaseException{
		int currentTag;
		boolean end = false;
		StringBuffer gmlString = new StringBuffer();
		
		try{		
			currentTag = parser.getEventType();			
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					if (parser.getName().compareTo("geometryMember")==0){
						//Esta etiqueta de gml que nos indica donde empieza y donde acaba un subelemento
						//de una geometria compleja, como un multipoligono no la admite el parser de Geotools
						//Por tanto la debemos de ignorar
						break;
					}
					gmlString.append("<" + parser.getName());
					for (int i=0 ; i<parser.getAttributeCount() ; i++){
						gmlString.append(" " + parser.getAttributeName(i) + 
								"=\"" + parser.getAttributeValue(i) + "\"");
					}
					gmlString.append(">");
					break;
				case KXmlParser.END_TAG:
					if (parser.getName().compareTo("geometryMember")==0){
						//Esta etiqueta de gml que nos indica donde empieza y donde acaba un subelemento
						//de una geometria compleja, como un multipoligono no la admite el parser de Geotools
						//Por tanto la debemos de ignorar
						break;
					}
					if ((parser.getName().compareTo(endGeometryTag) == 0)){
						end = true;
					}
					gmlString.append("</" + parser.getName() + ">");
					break;
				case KXmlParser.TEXT:                 
					gmlString.append(parser.getText());
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}catch (XmlPullParserException e) {
			//Captured KXML parsing Exception
			throw new GMLParserException(e);
		} catch (IOException e) {
			throw new GMLFileReadException(e);
		} 							
		return gmlString.toString();	
	}		
	
}
