package org.gvsig.gui.beans.filterPanel.filterQueryPanel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.Serializable;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.gvsig.gui.beans.Messages;
import org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel;
import org.gvsig.gui.beans.filterPanel.JLabelCellRenderer;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This class is a version of the graphical interface of "FilterDialog" which was made by Fernando and is in 'appgvSIG' project.
 * It's supposed that other classes will extend from this and will add the logic as is in the "FilterDialog" class.
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class FilterQueryJPanel extends AbstractFilterQueryJPanel implements Serializable{
	private static final long serialVersionUID = -5772343767095703604L;

	protected int filterJScrollPaneHeight = defaultBottomJPanelHeight - 10;
	protected int filterJScrollPanelWidth = defaultBottomJPanelWidth;
	
	/**
	 * @see AbstractFilterQueryJPanel#AbstractFilterQueryJPanel()
	 */
	public FilterQueryJPanel() {
		super();
		this.initialize();
	}

	/**
	 * @see AbstractFilterQueryJPanel#AbstractFilterQueryJPanel(String)
	 */
	public FilterQueryJPanel(String _title) {
		super(_title);
		this.initialize();
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#initialize()
	 */
	protected void initialize() {
		super.initialize();
		this.setBorder(BorderFactory.createTitledBorder(Messages.getText("filterOnAWFSLayer")));
		getValuesJList().setCellRenderer(new JLabelCellRenderer());
	}
	
	/*
	 *  (FilterQueryJPanel)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getFieldsJPanel()
	 */
	protected JPanel getFieldsJPanel() {
		if (fieldsJPanel == null) {
			fieldsJPanel = new javax.swing.JPanel();
			fieldsJPanel.setPreferredSize(new java.awt.Dimension(fieldsJPanelWidth, fieldsJPanelHeight));
			fieldsJPanel.setLayout(new GridBagLayout());
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
			gridBagConstraints.anchor = GridBagConstraints.NORTH;			
			fieldsJPanel.add(getFieldsJLabel(), gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.SOUTH;
			fieldsJPanel.add(getFieldsJScrollPane(), gridBagConstraints);			
		}

		return fieldsJPanel;
	}	
	
	/**
	 * @see AbstractFilterQueryJPanel#getValuesJList()
	 */
	protected javax.swing.JList getValuesJList() {
		if (valuesJList == null) {
			valuesJList = new javax.swing.JList(new DefaultListModel());
			valuesListModel = new DefaultListModelForJLabelAsCell();
			valuesJList.setModel(valuesListModel);
			valuesJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
			
//			valuesJList.addMouseListener(new MouseAdapter() {
//				/*
//				 *  (non-Javadoc)
//				 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
//				 */
//				public void mouseClicked(MouseEvent e) {
//					if (e.getClickCount() == 2) {
//						int row = fieldsJTree.getRowForLocation(e.getX(), e.getY());
//	
//						if (row > -1) {
//							putSymbol(((DefaultListModel)valuesJList.getModel()).get(row).toString());
//						}
//					}
//				}
//			});
		}

		return valuesJList;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getFieldsJScrollPane()
	 */
	protected javax.swing.JScrollPane getFieldsJScrollPane() {
		if (fieldsJScrollPane == null) {
			fieldsJScrollPane = new javax.swing.JScrollPane();
			fieldsJScrollPane.setPreferredSize(new java.awt.Dimension(fieldsAndValuesJScrollPaneWidth, fieldsAndValuesJScrollPaneHeight));
			fieldsJScrollPane.setViewportView(getFieldsJTree());
		}

		return fieldsJScrollPane;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getValuesJPanel()
	 */
	protected JPanel getValuesJPanel() {
		if (valuesJPanel == null) {
			valuesJPanel = new javax.swing.JPanel();
			valuesJPanel.setPreferredSize(new java.awt.Dimension(valuesJPanelWidth, valuesJPanelHeight));
			valuesJPanel.setLayout(new GridBagLayout());
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
			gridBagConstraints.anchor = GridBagConstraints.NORTH;
			valuesJPanel.add(getValuesJLabel(), gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.SOUTH;
			valuesJPanel.add(getValuesJScrollPane(), gridBagConstraints);
		}

		return valuesJPanel;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getValuesJScrollPane()
	 */
	protected javax.swing.JScrollPane getValuesJScrollPane() {
		if (valuesJScrollPane == null) {
			valuesJScrollPane = new javax.swing.JScrollPane();
			valuesJScrollPane.setPreferredSize(new Dimension(fieldsAndValuesJScrollPaneWidth, fieldsAndValuesJScrollPaneHeight));
			valuesJScrollPane.setViewportView(getValuesJList());
		}

		return valuesJScrollPane;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getFilterJScrollPane()
	 */
	protected JScrollPane getFilterJScrollPane() {
		if (filterJScrollPane == null) {
			filterJScrollPane = new javax.swing.JScrollPane();
			filterJScrollPane.setPreferredSize(new java.awt.Dimension(filterJScrollPanelWidth, filterJScrollPaneHeight));
			filterJScrollPane.setViewportView(getTxtExpression());
			filterJScrollPane.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		}

		return filterJScrollPane;
	}	

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getTxtExpression()
	 */
	protected javax.swing.JTextArea getTxtExpression() {
		if (txtExpression == null) {
			txtExpression = new javax.swing.JTextArea();
			txtExpression.setLineWrap(true);
		}

		return txtExpression;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#getBottomJPanel()
	 */
	protected JPanel getBottomJPanel() {
		if (bottomJPanel == null) {
			bottomJPanel = new JPanel();
			bottomJPanel.setPreferredSize(new Dimension(defaultBottomJPanelWidth, defaultBottomJPanelHeight));
			bottomJPanel.add(getFilterJScrollPane(), null);
		}
		
		return bottomJPanel;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#resizeWidth(int)
	 */
	public void resizeWidth(int new_Width) {	
		int difference = new_Width - DefaultWidth;
		
		if (difference != 0) {
			this.setPreferredSize(new Dimension(this.getPreferredSize().width + difference, this.getPreferredSize().height));
			getTopJPanel().setPreferredSize(new Dimension(getTopJPanel().getPreferredSize().width + difference, getTopJPanel().getPreferredSize().height));
			getBottomJPanel().setPreferredSize(new Dimension(getBottomJPanel().getPreferredSize().width + difference, getBottomJPanel().getPreferredSize().height));
			getFilterJScrollPane().setPreferredSize(new Dimension(getFilterJScrollPane().getPreferredSize().width + difference, getFilterJScrollPane().getPreferredSize().height));
			
			int halfDifference = difference / 2;
			
			getFieldsJLabel().setPreferredSize(new Dimension(getFieldsJLabel().getPreferredSize().width + halfDifference, getFieldsJLabel().getPreferredSize().height));
			getFieldsJPanel().setPreferredSize(new Dimension(getFieldsJPanel().getPreferredSize().width + halfDifference, getFieldsJPanel().getPreferredSize().height));
			getFieldsJScrollPane().setPreferredSize(new Dimension(getFieldsJScrollPane().getPreferredSize().width + halfDifference, getFieldsJScrollPane().getPreferredSize().height));
		
			getValuesJLabel().setPreferredSize(new Dimension(getValuesJLabel().getPreferredSize().width + halfDifference, getValuesJLabel().getPreferredSize().height));
			getValuesJPanel().setPreferredSize(new Dimension(getValuesJPanel().getPreferredSize().width + halfDifference, getValuesJPanel().getPreferredSize().height));
			getValuesJScrollPane().setPreferredSize(new Dimension(getValuesJScrollPane().getPreferredSize().width + halfDifference, getValuesJScrollPane().getPreferredSize().height));
		}
	}
	
	/**
	 * Extension of JLabel that has some specific characteristics -> Version for values that are loaded in layer in view
	 * 
	 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
	 */
	protected final class JLabelValueLoaded extends JLabel implements Comparable {

		/**
		 * Default constructor without parameters
		 */
		public JLabelValueLoaded() {
			super();
			this.configure();
		}

		/**
		 * Default constructor with a parameter
		 * 
		 * @param text
		 */
		public JLabelValueLoaded(String text) {
			super(text);
			this.configure();
		}
		
		/**
		 * Configures some characteristics 
		 */
		private void configure() {
			this.setBackground(Color.yellow);
			this.setForeground(Color.blue);
		}

		/*
		 *  (non-Javadoc)
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object o) {
			// It's supposed that 'o' is an string
			return this.getText().compareTo((String)o);
		}
		
		/*
		 *  (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		public String toString() {
			return this.getText();
		}
	}
	
	/**
	 * Extension of JLabel that has some specific characteristics -> Version for values that aren't loaded in layer in view
	 * 
	 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
	 */
	protected final class JLabelValueNotLoaded extends JLabel implements Comparable{
		
		/**
		 * Default constructor without parameters 
		 */
		public JLabelValueNotLoaded() {
			super();
			this.configure();
		}

		/**
		 * Default constructor with a parameter
		 * 
		 * @param text
		 */
		public JLabelValueNotLoaded(String text) {
			super(text);
			this.configure();
		}
		
		/**
		 * Configures some characteristics 
		 */
		private void configure() {
			this.setBackground(Color.white);
		}

		/*
		 *  (non-Javadoc)
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object o) {
			// It's supposed that 'o' is an string
			return this.getText().compareTo((String)o);
		}
		
		/*
		 *  (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		public String toString() {
			return this.getText();
		}
	}
	
	/**
	 * This class reimplements the "contains()" method of "DefaultListModel" for 
	 * 
	 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
	 *
	 */
	protected final class DefaultListModelForJLabelAsCell extends DefaultListModel{
		
		/**
		 * Represents the start index of the group of not loaded values (that should be after the group of loaded values) 
		 * -2                 -> if elements aren't grouped (in two lists: first for loaded, after for not loaded)
		 * -1                 -> if there isn't elements
		 *  0                 -> if there are only not loaded elements  
		 *  = (capacity() -1) -> if there are only loaded elements
		 */
		private int startIndexOfNotLoadedValues; 
		
		/**
		 * Default constructor without parameters
		 */
		public DefaultListModelForJLabelAsCell() {
			super();
			startIndexOfNotLoadedValues = -1;
		}
		
		/**
		 * Method that returns true if there is a inner JLabel that has as inner text the same text as 'patternText' 
		 * 
		 * @param patternText
		 * @return A boolean value
		 */
		public boolean containsJLabelText(String patternText) {
			for (int i = 0; i < super.getSize(); i++) {
				if (((JLabel)super.get(i)).getText().compareTo(patternText) == 0)
					return true;
			}				
				
			return false;
		}
		
		/**
		 * Returns the position of first JLabel that has as text the same text as 'patternText', or -1 if no JLabel has that text
		 * 
		 * @param patternText An String
		 * @return An integer value
		 */
		public int getIndexOfJLabelText(String patternText) {
			for (int i = 0; i < super.getSize(); i++) {
				if (((JLabel)super.get(i)).getText().compareTo(patternText) == 0)
					return i;
			}				
				
			return -1; // If not found
		}
		
		/**
		 * Changes element to 
		 * 
		 * @param patternText
		 */
		public void changeElementThatHasTextToJLabelLoadedValue(String patternText) {
			int index = getIndexOfJLabelText(patternText);
			
			if (index != -1)
				super.set(index, new JLabelValueLoaded(patternText));
		}
		
		/**
		 * Sets all elements to 'JLabelValueNotLoaded'
		 */
		public void setAllElementsToNotLoaded() {
			if (startIndexOfNotLoadedValues == -1) {
				for (int i = 0; i < super.getSize(); i++) {
					if (super.get(i) instanceof JLabelValueLoaded) {
						JLabelValueNotLoaded jL = new JLabelValueNotLoaded(((JLabelValueLoaded)super.get(i)).getText());
						super.remove(i);
						super.add(i, jL);///// REIMPLEMENTATION OF SOME METHODS OF 'DefaultListModel' /////
					}
				}
			}
			else {
				for (int i = 0; i < startIndexOfNotLoadedValues; i++) {
					if (super.get(i) instanceof JLabelValueLoaded) {
						JLabelValueNotLoaded jL = new JLabelValueNotLoaded(((JLabelValueLoaded)super.get(i)).getText());
						super.set(i, jL); // replacement
					}
				}
			}
			
			startIndexOfNotLoadedValues = 0;
		}
		
		///// REIMPLEMENTATION OF SOME METHODS OF 'DefaultListModel' /////
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#clear()
		 */		
		public void clear() {
			super.clear();
			startIndexOfNotLoadedValues = -1;
		}
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#insertElementAt(java.lang.Object, int)
		 */
		public void insertElementAt(Object obj, int index) {
			super.insertElementAt(obj, index);
			
			if ((obj instanceof JLabelValueNotLoaded) && (index > -1) && (index < startIndexOfNotLoadedValues))
				startIndexOfNotLoadedValues = index;
		}
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#remove(int)
		 */
		public Object remove(int index) {
			if ( (index > -1) && (index == startIndexOfNotLoadedValues) && (index == (super.capacity() -1)) ) {
				startIndexOfNotLoadedValues = -1;
			}

			return super.remove(index);
		}
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#removeAllElements()
		 */
		public void	removeAllElements() {
			super.removeAllElements();
			startIndexOfNotLoadedValues = -1;
		}
		 
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#removeElement(java.lang.Object)
		 */
		public boolean removeElement(Object obj) {
			int index = super.indexOf(obj);
			
			if ( (index > -1) && (index == startIndexOfNotLoadedValues) && (index == (super.capacity() -1)) ) {
				startIndexOfNotLoadedValues = -1;
			}
			
			return super.removeElement(obj);
		}
		 
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#removeElementAt(int)
		 */
		public void removeElementAt(int index) {
			if ( (index > -1) && (index == startIndexOfNotLoadedValues) && (index == (super.capacity() -1)) ) {
				startIndexOfNotLoadedValues = -1;
			}
			
			super.removeElementAt(index);			
		}
		 
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#removeRange(int, int)
		 */
		public void removeRange(int fromIndex, int toIndex) {			
			super.removeRange(fromIndex, toIndex);
			
			// Try to find any element of type 'JLabelNotLoadedValue'
			for (int i = 0; i < super.capacity(); i++) {
				if (super.get(i) instanceof JLabelValueNotLoaded) {
					if ( ( (i == (super.capacity() -1) || (i < (super.capacity() -1)) && (super.get(super.capacity() -1) instanceof JLabelValueNotLoaded) ) ) )
						startIndexOfNotLoadedValues = i;
					else
						startIndexOfNotLoadedValues = -1;
				}
			}
		}
		 
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#set(int, java.lang.Object)
		 */
		public Object set(int index, Object element) {
			if ((element instanceof JLabelValueNotLoaded) && (index > -1) && (index < startIndexOfNotLoadedValues))
				startIndexOfNotLoadedValues = index;

			return super.set(index, element);
		}
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.DefaultListModel#setElementAt(java.lang.Object, int)
		 */
		public void setElementAt(Object obj, int index) {
			super.setElementAt(obj, index);

			if ((obj instanceof JLabelValueNotLoaded) && (index > -1) && (index < startIndexOfNotLoadedValues))
				startIndexOfNotLoadedValues = index;
		}

		///// END REIMPLEMENTATION OF SOME METHODS OF 'DefaultListModel' /////		
	}
}
